@@ -41,8 +41,10 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "MenuActionBar.h"
 #include "MenuExit.h"
 #include "MenuGameOver.h"
+#include "MenuInventory.h"
 #include "MenuManager.h"
 #include "MessageEngine.h"
+#include "MenuMiniMap.h"
 #include "ModManager.h"
 #include "PowerManager.h"
 #include "RenderDevice.h"
@@ -52,6 +54,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "SharedResources.h"
 #include "SoundManager.h"
 #include "Utils.h"
+#include "UtilsFileSystem.h"
 #include "UtilsMath.h"
 #include "UtilsParsing.h"
 
@@ -74,15 +77,12 @@ Avatar::Avatar()
 	, using_main2(false)
 	, prev_hp(0)
 	, playing_lowhp(false)
-	, teleport_camera_lock(false) {
-
+	, teleport_camera_lock(false)
+	, feet_index(-1)
+{
 	init();
 
-	// load the hero's animations from hero definition file
-	anim->increaseCount("animations/hero.txt");
-	animationSet = anim->getAnimationSet("animations/hero.txt");
-	activeAnimation = animationSet->getAnimation("");
-
+	// TODO
 	// set cooldown_hit to duration of hit animation if undefined
 	if (!stats.cooldown_hit_enabled) {
 		Animation *hit_anim = animationSet->getAnimation("hit");
@@ -174,6 +174,9 @@ void Avatar::init() {
 		power_cooldown_timers[power_it->first] = Timer();
 		power_cast_timers[power_it->first] = Timer();
 	}
+
+	stats.animations = "animations/hero.txt";
+	loadAnimations();
 }
 
 void Avatar::handleNewMap() {
@@ -186,77 +189,24 @@ void Avatar::handleNewMap() {
  * Load avatar sprite layer definitions into vector.
  */
 void Avatar::loadLayerDefinitions() {
-	layer_def = std::vector<std::vector<unsigned> >(8, std::vector<unsigned>());
-	layer_reference_order = std::vector<std::string>();
+	if (!stats.layer_reference_order.empty())
+		return;
+
+	Utils::logError("Avatar: Loading render layers from engine/hero_layers.txt is deprecated! Render layers should be loaded in the 'render_layers' section of engine/stats.txt.");
 
 	FileParser infile;
-	// @CLASS Avatar: Hero layers|Description of engine/hero_layers.txt
 	if (infile.open("engine/hero_layers.txt", FileParser::MOD_FILE, FileParser::ERROR_NORMAL)) {
 		while(infile.next()) {
-			if (infile.key == "layer") {
-				// @ATTR layer|direction, list(string) : Direction, Layer name(s)|Defines the hero avatar sprite layer
-				unsigned dir = Parse::toDirection(Parse::popFirstString(infile.val));
-				if (dir>7) {
-					infile.error("Avatar: Hero layer direction must be in range [0,7]");
-					Utils::logErrorDialog("Avatar: Hero layer direction must be in range [0,7]");
-					mods->resetModConfig();
-					Utils::Exit(1);
-				}
-				std::string layer = Parse::popFirstString(infile.val);
-				while (layer != "") {
-					// check if already in layer_reference:
-					unsigned ref_pos;
-					for (ref_pos = 0; ref_pos < layer_reference_order.size(); ++ref_pos)
-						if (layer == layer_reference_order[ref_pos])
-							break;
-					if (ref_pos == layer_reference_order.size())
-						layer_reference_order.push_back(layer);
-					layer_def[dir].push_back(ref_pos);
+			// hero_layers.txt doesn't have sections, but we now expect the layer key to be under one called "render_layers"
+			if (infile.section.empty())
+				infile.section = "render_layers";
 
-					layer = Parse::popFirstString(infile.val);
-				}
-			}
-			else {
+			if (!stats.loadRenderLayerStat(&infile)) {
 				infile.error("Avatar: '%s' is not a valid key.", infile.key.c_str());
 			}
 		}
 		infile.close();
 	}
-
-	// There are the positions of the items relative to layer_reference_order
-	// so if layer_reference_order=main,body,head,off
-	// and we got a layer=3,off,body,head,main
-	// then the layer_def[3] looks like (3,1,2,0)
-}
-
-void Avatar::loadGraphics(std::vector<Layer_gfx> _img_gfx) {
-
-	for (unsigned int i=0; i<animsets.size(); i++) {
-		if (animsets[i])
-			anim->decreaseCount(animsets[i]->getName());
-		delete anims[i];
-	}
-	animsets.clear();
-	anims.clear();
-
-	for (unsigned int i=0; i<_img_gfx.size(); i++) {
-		if (_img_gfx[i].gfx != "") {
-			std::string name = "animations/avatar/"+stats.gfx_base+"/"+_img_gfx[i].gfx+".txt";
-			anim->increaseCount(name);
-			animsets.push_back(anim->getAnimationSet(name));
-			animsets.back()->setParent(animationSet);
-			anims.push_back(animsets.back()->getAnimation(activeAnimation->getName()));
-			setAnimation("stance");
-			if(!anims.back()->syncTo(activeAnimation)) {
-				Utils::logError("Avatar: Error syncing animation in '%s' to 'animations/hero.txt'.", animsets.back()->getName().c_str());
-			}
-		}
-		else {
-			animsets.push_back(NULL);
-			anims.push_back(NULL);
-		}
-	}
-	anim->cleanUp();
 }
 
 /**
@@ -326,18 +276,30 @@ void Avatar::set_direction() {
 		stats.direction = Utils::calcDirection(stats.pos.x, stats.pos.y, target.x, target.y);
 	}
 	else {
-		if (inpt->pressing[Input::UP] && !inpt->lock[Input::UP] && inpt->pressing[Input::LEFT] && !inpt->lock[Input::LEFT]) stats.direction = 1;
-		else if (inpt->pressing[Input::UP] && !inpt->lock[Input::UP] && inpt->pressing[Input::RIGHT] && !inpt->lock[Input::RIGHT]) stats.direction = 3;
-		else if (inpt->pressing[Input::DOWN] && !inpt->lock[Input::DOWN] && inpt->pressing[Input::RIGHT] && !inpt->lock[Input::RIGHT]) stats.direction = 5;
-		else if (inpt->pressing[Input::DOWN] && !inpt->lock[Input::DOWN] && inpt->pressing[Input::LEFT] && !inpt->lock[Input::LEFT]) stats.direction = 7;
-		else if (inpt->pressing[Input::LEFT] && !inpt->lock[Input::LEFT]) stats.direction = 0;
-		else if (inpt->pressing[Input::UP] && !inpt->lock[Input::UP]) stats.direction = 2;
-		else if (inpt->pressing[Input::RIGHT] && !inpt->lock[Input::RIGHT]) stats.direction = 4;
-		else if (inpt->pressing[Input::DOWN] && !inpt->lock[Input::DOWN]) stats.direction = 6;
+		// movement keys take top priority for setting direction
+		bool press_up = inpt->pressing[Input::UP] && !inpt->lock[Input::UP];
+		bool press_down = inpt->pressing[Input::DOWN] && !inpt->lock[Input::DOWN];
+		bool press_left = inpt->pressing[Input::LEFT] && !inpt->lock[Input::LEFT];
+		bool press_right = inpt->pressing[Input::RIGHT] && !inpt->lock[Input::RIGHT];
+
+		// aiming keys can set direction as well
+		if (!press_up && !press_down && !press_left && !press_right) {
+			press_up = inpt->pressing[Input::AIM_UP] && !inpt->lock[Input::AIM_UP];
+			press_down = inpt->pressing[Input::AIM_DOWN] && !inpt->lock[Input::AIM_DOWN];
+			press_left = inpt->pressing[Input::AIM_LEFT] && !inpt->lock[Input::AIM_LEFT];
+			press_right = inpt->pressing[Input::AIM_RIGHT] && !inpt->lock[Input::AIM_RIGHT];
+		}
+
+		if (press_up && press_left) stats.direction = 1;
+		else if (press_up && press_right) stats.direction = 3;
+		else if (press_down && press_right) stats.direction = 5;
+		else if (press_down && press_left) stats.direction = 7;
+		else if (press_left) stats.direction = 0;
+		else if (press_up) stats.direction = 2;
+		else if (press_right) stats.direction = 4;
+		else if (press_down) stats.direction = 6;
 		// Adjust for ORTHO tilesets
-		if (eset->tileset.orientation == eset->tileset.TILESET_ORTHOGONAL &&
-				((inpt->pressing[Input::UP] && !inpt->lock[Input::UP]) || (inpt->pressing[Input::DOWN] && !inpt->lock[Input::UP]) ||
-				 (inpt->pressing[Input::LEFT] && !inpt->lock[Input::LEFT]) || (inpt->pressing[Input::RIGHT] && !inpt->lock[Input::RIGHT])))
+		if (eset->tileset.orientation == eset->tileset.TILESET_ORTHOGONAL && (press_up || press_down || press_left || press_right))
 			stats.direction = static_cast<unsigned char>((stats.direction == 7) ? 0 : stats.direction + 1);
 	}
 
@@ -447,9 +409,11 @@ void Avatar::logic() {
 
 	// handle animation
 	if (!stats.effects.stun) {
-		activeAnimation->advanceFrame();
-		for (unsigned i=0; i < anims.size(); i++) {
-			if (anims[i] != NULL)
+		if (activeAnimation)
+			activeAnimation->advanceFrame();
+
+		for (size_t i = 0; i < anims.size(); ++i) {
+			if (anims[i])
 				anims[i]->advanceFrame();
 		}
 	}
@@ -497,6 +461,7 @@ void Avatar::logic() {
 
 	if (!stats.effects.stun) {
 		bool allowed_to_move;
+		bool allowed_to_turn;
 		bool allowed_to_use_power = true;
 
 		switch(stats.cur_state) {
@@ -507,19 +472,26 @@ void Avatar::logic() {
 				// allowed to move or use powers?
 				if (settings->mouse_move) {
 					allowed_to_move = restrict_power_use && (!inpt->lock[mm_key] || drag_walking) && !lock_enemy;
+					allowed_to_turn = allowed_to_move;
 					allowed_to_use_power = true;
 
 					if ((inpt->pressing[mm_key] && inpt->pressing[Input::SHIFT]) || lock_enemy) {
 						inpt->lock[mm_key] = false;
 					}
 				}
+				else if (!settings->mouse_aim) {
+					allowed_to_move = !inpt->pressing[Input::SHIFT];
+					allowed_to_turn = true;
+					allowed_to_use_power = true;
+				}
 				else {
 					allowed_to_move = true;
+					allowed_to_turn = true;
 					allowed_to_use_power = true;
 				}
 
 				// handle transitions to RUN
-				if (allowed_to_move)
+				if (allowed_to_turn)
 					set_direction();
 
 				if (pressing_move() && allowed_to_move) {
@@ -563,8 +535,10 @@ void Avatar::logic() {
 					stats.cur_state = StatBlock::ENTITY_STANCE;
 					break;
 				}
-				else if (settings->mouse_move && inpt->pressing[Input::SHIFT]) {
-					// when moving with the mouse, pressing Shift should stop movement and begin attacking
+				else if ((settings->mouse_move || !settings->mouse_aim) && inpt->pressing[Input::SHIFT]) {
+					// Shift should stop movement in some cases.
+					// With mouse_move, it allows the player to stop moving and begin attacking.
+					// With mouse_aim disabled, it allows the player to aim their attacks without having to move.
 					stats.cur_state = StatBlock::ENTITY_STANCE;
 					break;
 				}
@@ -825,6 +799,11 @@ void Avatar::logic() {
 				}
 			}
 
+			for (size_t i = action_queue.size(); i > 0; i--) {
+				if (action_queue[i-1].activated_from_inventory)
+					action_queue.erase(action_queue.begin()+(i-1));
+			}
+
 			stats.blocking = blocking;
 		}
 
@@ -898,12 +877,14 @@ void Avatar::transform() {
 	stats.powers_list = charmed_stats->powers_list;
 	stats.powers_passive = charmed_stats->powers_passive;
 	stats.effects.clearEffects();
+	stats.animations = charmed_stats->animations;
+	stats.layer_reference_order = charmed_stats->layer_reference_order;
+	stats.layer_def = charmed_stats->layer_def;
+	stats.animation_slots = charmed_stats->animation_slots;
 
-	anim->decreaseCount("animations/hero.txt");
-	anim->increaseCount(charmed_stats->animations);
-	animationSet = anim->getAnimationSet(charmed_stats->animations);
-	delete activeAnimation;
-	activeAnimation = animationSet->getAnimation("");
+	anim->decreaseCount(hero_stats->animations);
+	animationSet = NULL;
+	loadAnimations();
 	stats.cur_state = StatBlock::ENTITY_STANCE;
 
 	// base stats
@@ -911,11 +892,6 @@ void Avatar::transform() {
 		stats.starting[i] = std::max(stats.starting[i], charmed_stats->starting[i]);
 	}
 
-	// resistances
-	for (unsigned int i=0; i<stats.vulnerable.size(); i++) {
-		stats.vulnerable[i] = std::min(stats.vulnerable[i], charmed_stats->vulnerable[i]);
-	}
-
 	loadSoundsFromStatBlock(charmed_stats);
 	loadStepFX("NULL");
 
@@ -962,12 +938,14 @@ void Avatar::untransform() {
 	stats.effects = hero_stats->effects;
 	stats.powers_list = hero_stats->powers_list;
 	stats.powers_passive = hero_stats->powers_passive;
+	stats.animations = hero_stats->animations;
+	stats.layer_reference_order = hero_stats->layer_reference_order;
+	stats.layer_def = hero_stats->layer_def;
+	stats.animation_slots = hero_stats->animation_slots;
 
-	anim->increaseCount("animations/hero.txt");
 	anim->decreaseCount(charmed_stats->animations);
-	animationSet = anim->getAnimationSet("animations/hero.txt");
-	delete activeAnimation;
-	activeAnimation = animationSet->getAnimation("");
+	animationSet = NULL;
+	loadAnimations();
 	stats.cur_state = StatBlock::ENTITY_STANCE;
 
 	// This is a bit of a hack.
@@ -978,10 +956,6 @@ void Avatar::untransform() {
 		stats.starting[i] = hero_stats->starting[i];
 	}
 
-	for (unsigned int i=0; i<stats.vulnerable.size(); i++) {
-		stats.vulnerable[i] = hero_stats->vulnerable[i];
-	}
-
 	loadSounds();
 	loadStepFX(stats.sfx_step);
 
@@ -1002,66 +976,6 @@ void Avatar::checkTransform() {
 		untransform();
 }
 
-void Avatar::setAnimation(std::string name) {
-	if (name == activeAnimation->getName())
-		return;
-
-	Entity::setAnimation(name);
-	for (unsigned i=0; i < animsets.size(); i++) {
-		delete anims[i];
-		if (animsets[i])
-			anims[i] = animsets[i]->getAnimation(name);
-		else
-			anims[i] = 0;
-	}
-}
-
-void Avatar::resetActiveAnimation() {
-	activeAnimation->reset(); // shield stutter
-	for (unsigned i=0; i < animsets.size(); i++)
-		if (anims[i])
-			anims[i]->reset();
-}
-
-void Avatar::addRenders(std::vector<Renderable> &r) {
-	if (!stats.transformed) {
-		for (unsigned i = 0; i < layer_def[stats.direction].size(); ++i) {
-			unsigned index = layer_def[stats.direction][i];
-			if (anims[index]) {
-				Renderable ren = anims[index]->getCurrentFrame(stats.direction);
-				ren.map_pos = stats.pos;
-				ren.prio = i+1;
-				stats.effects.getCurrentColor(ren.color_mod);
-				stats.effects.getCurrentAlpha(ren.alpha_mod);
-				if (stats.hp > 0) {
-					ren.type = Renderable::TYPE_HERO;
-				}
-				r.push_back(ren);
-			}
-		}
-	}
-	else {
-		Renderable ren = activeAnimation->getCurrentFrame(stats.direction);
-		ren.map_pos = stats.pos;
-		stats.effects.getCurrentColor(ren.color_mod);
-		stats.effects.getCurrentAlpha(ren.alpha_mod);
-		if (stats.hp > 0) {
-			ren.type = Renderable::TYPE_HERO;
-		}
-		r.push_back(ren);
-	}
-	// add effects
-	for (unsigned i = 0; i < stats.effects.effect_list.size(); ++i) {
-		if (stats.effects.effect_list[i].animation && !stats.effects.effect_list[i].animation->isCompleted()) {
-			Renderable ren = stats.effects.effect_list[i].animation->getCurrentFrame(0);
-			ren.map_pos = stats.pos;
-			if (stats.effects.effect_list[i].render_above) ren.prio = layer_def[stats.direction].size()+1;
-			else ren.prio = 0;
-			r.push_back(ren);
-		}
-	}
-}
-
 void Avatar::logMsg(const std::string& str, int type) {
 	log_msg.push(std::pair<std::string, int>(str, type));
 }
@@ -1102,21 +1016,39 @@ bool Avatar::isLowHpCursorEnabled() {
 		settings->low_hp_warning_type == settings->LHP_WARN_ALL;
 }
 
+std::string Avatar::getGfxFromType(const std::string& gfx_type) {
+	feet_index = -1;
+	std::string gfx;
+
+	if (menu && menu->inv) {
+		for (int i=0; i<menu->inv->inventory[MenuInventory::EQUIPMENT].getSlotNumber(); i++) {
+			if (!menu->inv->isActive(i))
+				continue;
+
+			if (gfx_type == menu->inv->inventory[MenuInventory::EQUIPMENT].slot_type[i]) {
+				gfx = items->items[menu->inv->inventory[MenuInventory::EQUIPMENT][i].item].gfx;
+			}
+			if (menu->inv->inventory[MenuInventory::EQUIPMENT].slot_type[i] == "feet") {
+				feet_index = i;
+			}
+		}
+	}
+
+	// special case: if we don't have a head, use the portrait's head
+	if (gfx.empty() && gfx_type == "head") {
+		gfx = stats.gfx_head;
+	}
+
+	// fall back to default if it exists
+	if (gfx.empty()) {
+		if (Filesystem::fileExists(mods->locate("animations/avatar/" + stats.gfx_base + "/default_" + gfx_type + ".txt")))
+			gfx = "default_" + gfx_type;
+	}
+
+	return gfx;
+}
+
 Avatar::~Avatar() {
-	if (stats.transformed && charmed_stats && charmed_stats->animations != "") {
-		anim->decreaseCount(charmed_stats->animations);
-	}
-	else {
-		anim->decreaseCount("animations/hero.txt");
-	}
-
-	for (unsigned int i=0; i<animsets.size(); i++) {
-		if (animsets[i])
-			anim->decreaseCount(animsets[i]->getName());
-		delete anims[i];
-	}
-	anim->cleanUp();
-
 	delete charmed_stats;
 	delete hero_stats;
 
