@@ -39,12 +39,14 @@ public:
 	PowerID power;
 	unsigned hotkey;
 	bool instant_item;
+	bool activated_from_inventory;
 	FPoint target;
 
 	ActionData()
 		: power(0)
 		, hotkey(0)
 		, instant_item(false)
+		, activated_from_inventory(false)
 		, target(FPoint()) {
 	}
 };
@@ -62,16 +64,12 @@ private:
 	void set_direction();
 	void transform();
 	void untransform();
-	void setAnimation(std::string name);
 
 
 	std::vector<Step_sfx> step_def;
 
 	std::vector<SoundID> sound_steps;
 
-	std::vector<AnimationSet*> animsets; // hold the animations for all equipped items in the right order of drawing.
-	std::vector<Animation*> anims; // hold the animations for all equipped items in the right order of drawing.
-
 	short body;
 
 	bool transform_triggered;
@@ -85,16 +83,7 @@ private:
 
 	bool isDroppedToLowHp();
 
-protected:
-	virtual void resetActiveAnimation();
-
 public:
-	class Layer_gfx {
-	public:
-		std::string gfx;
-		std::string type;
-	};
-
 	enum {
 		MSG_NORMAL = 0,
 		MSG_UNIQUE = 1
@@ -105,7 +94,7 @@ public:
 
 	void init();
 	void handleNewMap();
-	void loadGraphics(std::vector<Layer_gfx> _img_gfx);
+	void loadGraphics(std::vector<Entity::Layer_gfx> _img_gfx);
 	void loadStepFX(const std::string& stepname);
 
 	void logic();
@@ -116,8 +105,6 @@ public:
 	}
 	void checkTransform();
 
-	void addRenders(std::vector<Renderable> &r);
-
 	void logMsg(const std::string& str, int type);
 
 	bool isLowHp();
@@ -125,8 +112,7 @@ public:
 	bool isLowHpSoundEnabled();
 	bool isLowHpCursorEnabled();
 
-	std::vector<std::string> layer_reference_order;
-	std::vector<std::vector<unsigned> > layer_def;
+	std::string getGfxFromType(const std::string& gfx_type);
 
 	std::queue<std::pair<std::string, int> > log_msg;
 
@@ -158,6 +144,7 @@ public:
 	int prev_hp;
 	bool playing_lowhp;
 	bool teleport_camera_lock;
+	int feet_index;
 
 	std::vector<ActionData> action_queue;
 };
