@@ -36,10 +36,12 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "SharedGameResources.h"
 #include "SharedResources.h"
 #include "StatBlock.h"
+#include "UtilsMath.h"
 #include "UtilsParsing.h"
 
 CampaignManager::CampaignManager()
-	: bonus_xp(0.0) {
+	: bonus_xp(0.0)
+	, random_status(0) {
 }
 
 StatusID CampaignManager::registerStatus(const std::string& s) {
@@ -141,7 +143,7 @@ bool CampaignManager::checkItem(ItemStack istack) {
 	if (menu->inv->inventory[MenuInventory::CARRIED].contain(istack.item, istack.quantity))
 		return true;
 	else
-		return menu->inv->inventory[MenuInventory::EQUIPMENT].contain(istack.item, istack.quantity);
+		return menu->inv->equipmentContain(istack.item, istack.quantity);
 }
 
 void CampaignManager::removeCurrency(int quantity) {
@@ -287,5 +289,49 @@ bool CampaignManager::checkAllRequirements(const EventComponent& ec) {
 	return false;
 }
 
+bool CampaignManager::checkRequirementsInVector(const std::vector<EventComponent>& ec_vec) {
+	for (size_t i = 0; i < ec_vec.size(); ++i) {
+		if (!checkAllRequirements(ec_vec[i]))
+			return false;
+	}
+
+	return true;
+}
+
+void CampaignManager::randomStatusAppend(const StatusID s) {
+	if (std::find(random_status_pool.begin(), random_status_pool.end(), s) == random_status_pool.end()) {
+		if (random_status_pool.empty())
+			random_status = s;
+
+		random_status_pool.push_back(s);
+	}
+}
+
+void CampaignManager::randomStatusClear() {
+	random_status_pool.clear();
+	random_status = 0;
+}
+
+void CampaignManager::randomStatusRoll() {
+	if (random_status_pool.empty())
+		return;
+
+	random_status = random_status_pool[Math::randBetween(0, static_cast<int>(random_status_pool.size()) - 1)];
+}
+
+void CampaignManager::randomStatusSet() {
+	if (random_status_pool.empty())
+		return;
+
+	setStatus(random_status);
+}
+
+void CampaignManager::randomStatusUnset() {
+	if (random_status_pool.empty())
+		return;
+
+	unsetStatus(random_status);
+}
+
 CampaignManager::~CampaignManager() {
 }
