@@ -57,6 +57,13 @@ public:
 	void rewardXP(int amount, bool show_message);
 	void restoreHPMP(const std::string& s);
 	bool checkAllRequirements(const EventComponent& ec);
+	bool checkRequirementsInVector(const std::vector<EventComponent>& ec_vec);
+
+	void randomStatusAppend(const StatusID s);
+	void randomStatusClear();
+	void randomStatusRoll();
+	void randomStatusSet();
+	void randomStatusUnset();
 
 	std::queue<ItemStack> drop_stack;
 
@@ -66,6 +73,9 @@ public:
 
 private:
 	StatusMap status;
+
+	std::vector<StatusID> random_status_pool;
+	StatusID random_status;
 };
 
 
