@@ -52,6 +52,7 @@ void EngineSettings::Misc::load() {
 	// reset to defaults
 	save_hpmp = false;
 	corpse_timeout = 60 * settings->max_frames_per_sec;
+	corpse_timeout_enabled = true;
 	sell_without_vendor = true;
 	aim_assist = 0;
 	window_title = "Flare";
@@ -74,6 +75,8 @@ void EngineSettings::Misc::load() {
 	keep_buyback_on_map_change = true;
 	sfx_unable_to_cast = "";
 	combat_aborts_npc_interact = true;
+	fogofwar = 0;
+	save_fogofwar = false;
 
 	FileParser infile;
 	// @CLASS EngineSettings: Misc|Description of engine/misc.txt
@@ -82,7 +85,7 @@ void EngineSettings::Misc::load() {
 			// @ATTR save_hpmp|bool|When saving the game, keep the hero's current HP and MP.
 			if (infile.key == "save_hpmp")
 				save_hpmp = Parse::toBool(infile.val);
-			// @ATTR corpse_timeout|duration|Duration that a corpse can exist on the map in 'ms' or 's'.
+			// @ATTR corpse_timeout|duration|Duration that a corpse can exist on the map in 'ms' or 's'. Use 0 to keep corpses indefinitely.
 			else if (infile.key == "corpse_timeout")
 				corpse_timeout = Parse::toDuration(infile.val);
 			// @ATTR sell_without_vendor|bool|Allows selling items when not at a vendor via CTRL-Click.
@@ -169,6 +172,12 @@ void EngineSettings::Misc::load() {
 			// @ATTR combat_aborts_npc_interact|bool|If true, the NPC dialog and vendor menus will be closed if the player is attacked.
 			else if (infile.key == "combat_aborts_npc_interact")
 				combat_aborts_npc_interact = Parse::toBool(infile.val);
+			// @ATTR fogofwar|int|Set the fog of war type. 0-disabled, 1-minimap, 2-tint, 3-overlay.
+			else if (infile.key == "fogofwar")
+				fogofwar = static_cast<unsigned short>(Parse::toInt(infile.val));
+			// @ATTR save_fogofwar|bool|If true, the fog of war layer keeps track of the progress.
+			else if (infile.key == "save_fogofwar")
+				save_fogofwar = Parse::toBool(infile.val);
 
 			else infile.error("EngineSettings: '%s' is not a valid key.", infile.key.c_str());
 		}
@@ -184,6 +193,11 @@ void EngineSettings::Misc::load() {
 		Utils::logError("EngineSettings: Warning, save_buyback=true is ignored when keep_buyback_on_map_change=false.");
 		save_buyback = false;
 	}
+
+	if (corpse_timeout <= 0) {
+		corpse_timeout_enabled = false;
+		corpse_timeout = settings->max_frames_per_sec + 1;
+	}
 }
 
 void EngineSettings::Resolutions::load() {
@@ -374,8 +388,11 @@ void EngineSettings::Elements::load() {
 			if (list.empty() || infile.section != "element")
 				continue;
 
-			// @ATTR element.id|string|An identifier for this element.
-			if (infile.key == "id") list.back().id = infile.val;
+			// @ATTR element.id|string|An identifier for this element. When used as a resistance, "_resist" is appended to the id. For example, if the id is "fire", the resist id is "fire_resist".
+			if (infile.key == "id") {
+				list.back().id = infile.val;
+				list.back().resist_id = list.back().id + "_resist";
+			}
 			// @ATTR element.name|string|The displayed name of this element.
 			else if (infile.key == "name") list.back().name = msg->get(infile.val);
 
@@ -507,70 +524,68 @@ void EngineSettings::HeroClasses::load() {
 			if (list.empty() || infile.section != "class")
 				continue;
 
-			if (!list.empty()) {
-				// @ATTR name|string|The displayed name of this class.
-				if (infile.key == "name") list.back().name = infile.val;
-				// @ATTR description|string|A description of this class.
-				else if (infile.key == "description") list.back().description = infile.val;
-				// @ATTR currency|int|The amount of currency this class will start with.
-				else if (infile.key == "currency") list.back().currency = Parse::toInt(infile.val);
-				// @ATTR equipment|list(item_id)|A list of items that are equipped when starting with this class.
-				else if (infile.key == "equipment") list.back().equipment = infile.val;
-				// @ATTR carried|list(item_id)|A list of items that are placed in the normal inventorty when starting with this class.
-				else if (infile.key == "carried") list.back().carried = infile.val;
-				// @ATTR primary|predefined_string, int : Primary stat name, Default value|Class starts with this value for the specified stat.
-				else if (infile.key == "primary") {
-					std::string prim_stat = Parse::popFirstString(infile.val);
-					size_t prim_stat_index = eset->primary_stats.getIndexByID(prim_stat);
+			// @ATTR name|string|The displayed name of this class.
+			if (infile.key == "name") list.back().name = infile.val;
+			// @ATTR description|string|A description of this class.
+			else if (infile.key == "description") list.back().description = infile.val;
+			// @ATTR currency|int|The amount of currency this class will start with.
+			else if (infile.key == "currency") list.back().currency = Parse::toInt(infile.val);
+			// @ATTR equipment|list(item_id)|A list of items that are equipped when starting with this class.
+			else if (infile.key == "equipment") list.back().equipment = infile.val;
+			// @ATTR carried|list(item_id)|A list of items that are placed in the normal inventorty when starting with this class.
+			else if (infile.key == "carried") list.back().carried = infile.val;
+			// @ATTR primary|predefined_string, int : Primary stat name, Default value|Class starts with this value for the specified stat.
+			else if (infile.key == "primary") {
+				std::string prim_stat = Parse::popFirstString(infile.val);
+				size_t prim_stat_index = eset->primary_stats.getIndexByID(prim_stat);
 
-					if (prim_stat_index != eset->primary_stats.list.size()) {
-						list.back().primary[prim_stat_index] = Parse::toInt(infile.val);
-					}
-					else {
-						infile.error("EngineSettings: '%s' is not a valid primary stat.", prim_stat.c_str());
-					}
+				if (prim_stat_index != eset->primary_stats.list.size()) {
+					list.back().primary[prim_stat_index] = Parse::toInt(infile.val);
 				}
-
-				else if (infile.key == "actionbar") {
-					// @ATTR actionbar|list(power_id)|A list of powers to place in the action bar for the class.
-					for (int i=0; i<12; i++) {
-						list.back().hotkeys[i] = Parse::toPowerID(Parse::popFirstString(infile.val));
-					}
+				else {
+					infile.error("EngineSettings: '%s' is not a valid primary stat.", prim_stat.c_str());
 				}
-				else if (infile.key == "powers") {
-					// @ATTR powers|list(power_id)|A list of powers that are unlocked when starting this class.
-					std::string power;
-					while ( (power = Parse::popFirstString(infile.val)) != "") {
-						list.back().powers.push_back(Parse::toPowerID(power));
-					}
-				}
-				else if (infile.key == "campaign") {
-					// @ATTR campaign|list(string)|A list of campaign statuses that are set when starting this class.
-					std::string status;
-					while ( (status = Parse::popFirstString(infile.val)) != "") {
-						list.back().statuses.push_back(status);
-					}
-				}
-				else if (infile.key == "power_tree") {
-					// @ATTR power_tree|string|Power tree that will be loaded by MenuPowers
-					list.back().power_tree = infile.val;
-				}
-				else if (infile.key == "hero_options") {
-					// @ATTR hero_options|list(int)|A list of indicies of the hero options this class can use.
-					std::string hero_option;
-					while ( (hero_option = Parse::popFirstString(infile.val)) != "") {
-						list.back().options.push_back(Parse::toInt(hero_option));
-					}
-
-					std::sort(list.back().options.begin(), list.back().options.end());
-				}
-				else if (infile.key == "default_power_tab") {
-					// @ATTR default_power_tab|int|Index of the tab to switch to when opening the Powers menu
-					list.back().default_power_tab = Parse::toInt(infile.val);
-				}
-
-				else infile.error("EngineSettings: '%s' is not a valid key.", infile.key.c_str());
 			}
+
+			else if (infile.key == "actionbar") {
+				// @ATTR actionbar|list(power_id)|A list of powers to place in the action bar for the class.
+				for (int i=0; i<12; i++) {
+					list.back().hotkeys[i] = Parse::toPowerID(Parse::popFirstString(infile.val));
+				}
+			}
+			else if (infile.key == "powers") {
+				// @ATTR powers|list(power_id)|A list of powers that are unlocked when starting this class.
+				std::string power;
+				while ( (power = Parse::popFirstString(infile.val)) != "") {
+					list.back().powers.push_back(Parse::toPowerID(power));
+				}
+			}
+			else if (infile.key == "campaign") {
+				// @ATTR campaign|list(string)|A list of campaign statuses that are set when starting this class.
+				std::string status;
+				while ( (status = Parse::popFirstString(infile.val)) != "") {
+					list.back().statuses.push_back(status);
+				}
+			}
+			else if (infile.key == "power_tree") {
+				// @ATTR power_tree|string|Power tree that will be loaded by MenuPowers
+				list.back().power_tree = infile.val;
+			}
+			else if (infile.key == "hero_options") {
+				// @ATTR hero_options|list(int)|A list of indicies of the hero options this class can use.
+				std::string hero_option;
+				while ( (hero_option = Parse::popFirstString(infile.val)) != "") {
+					list.back().options.push_back(Parse::toInt(hero_option));
+				}
+
+				std::sort(list.back().options.begin(), list.back().options.end());
+			}
+			else if (infile.key == "default_power_tab") {
+				// @ATTR default_power_tab|int|Index of the tab to switch to when opening the Powers menu
+				list.back().default_power_tab = Parse::toInt(infile.val);
+			}
+
+			else infile.error("EngineSettings: '%s' is not a valid key.", infile.key.c_str());
 		}
 		infile.close();
 
@@ -612,10 +627,6 @@ void EngineSettings::DamageTypes::load() {
 		while (infile.next()) {
 			if (infile.new_section) {
 				if (infile.section == "damage_type") {
-					// damage types must have a printable name
-					if (!list.empty() && list.back().name == "") {
-						list.pop_back();
-					}
 					list.resize(list.size()+1);
 				}
 			}
@@ -623,28 +634,39 @@ void EngineSettings::DamageTypes::load() {
 			if (list.empty() || infile.section != "damage_type")
 				continue;
 
-			if (!list.empty()) {
-				// @ATTR damage_type.id|string|The identifier used for Item damage_type and Power base_damage.
-				if (infile.key == "id") list.back().id = infile.val;
-				// @ATTR damage_type.name|string|The displayed name for the value of this damage type.
-				else if (infile.key == "name") list.back().name = msg->get(infile.val);
-				// @ATTR damage_type.name_min|string|The displayed name for the minimum value of this damage type.
-				else if (infile.key == "name_min") list.back().name_min = msg->get(infile.val);
-				// @ATTR damage_type.name_max|string|The displayed name for the maximum value of this damage type.
-				else if (infile.key == "name_max") list.back().name_max = msg->get(infile.val);
-				// @ATTR damage_type.description|string|The description that will be displayed in the Character menu tooltips.
-				else if (infile.key == "description") list.back().description = msg->get(infile.val);
-				// @ATTR damage_type.min|string|The identifier used as a Stat type and an Effect type, for the minimum damage of this type.
-				else if (infile.key == "min") list.back().min = infile.val;
-				// @ATTR damage_type.max|string|The identifier used as a Stat type and an Effect type, for the maximum damage of this type.
-				else if (infile.key == "max") list.back().max = infile.val;
+			// @ATTR damage_type.id|string|The identifier used for Item damage_type and Power base_damage.
+			if (infile.key == "id") list.back().id = infile.val;
+			// @ATTR damage_type.name|string|The displayed name for the value of this damage type.
+			else if (infile.key == "name") list.back().name = msg->get(infile.val);
+			// @ATTR damage_type.name_min|string|The displayed name for the minimum value of this damage type.
+			else if (infile.key == "name_min") list.back().name_min = msg->get(infile.val);
+			// @ATTR damage_type.name_max|string|The displayed name for the maximum value of this damage type.
+			else if (infile.key == "name_max") list.back().name_max = msg->get(infile.val);
+			// @ATTR damage_type.description|string|The description that will be displayed in the Character menu tooltips.
+			else if (infile.key == "description") list.back().description = msg->get(infile.val);
+			// @ATTR damage_type.min|string|The identifier used as a Stat type and an Effect type, for the minimum damage of this type.
+			else if (infile.key == "min") list.back().min = infile.val;
+			// @ATTR damage_type.max|string|The identifier used as a Stat type and an Effect type, for the maximum damage of this type.
+			else if (infile.key == "max") list.back().max = infile.val;
 
-				else infile.error("EngineSettings: '%s' is not a valid key.", infile.key.c_str());
-			}
+			else infile.error("EngineSettings: '%s' is not a valid key.", infile.key.c_str());
 		}
 		infile.close();
 	}
 	count = list.size() * 2;
+
+	// use the IDs if the damage type doesn't have printable names
+	for (size_t i = 0; i < list.size(); ++i) {
+		if (list[i].name.empty()) {
+			list[i].name = list[i].id;
+		}
+		if (list[i].name_min.empty()) {
+			list[i].name_min = list[i].min;
+		}
+		if (list[i].name_max.empty()) {
+			list[i].name_max = list[i].max;
+		}
+	}
 }
 
 void EngineSettings::DeathPenalty::load() {
@@ -687,6 +709,7 @@ void EngineSettings::Tooltips::load() {
 	margin = 0;
 	margin_npc = 0;
 	background_border = 0;
+	visible_max = 3;
 
 	FileParser infile;
 	// @CLASS EngineSettings: Tooltips|Description of engine/tooltips.txt
@@ -707,6 +730,15 @@ void EngineSettings::Tooltips::load() {
 			// @ATTR tooltip_background_border|int|The pixel size of the border in "images/menus/tooltips.png".
 			else if (infile.key == "tooltip_background_border")
 				background_border = Parse::toInt(infile.val);
+			// @ATTR tooltip_visible_max|int|The maximum number of floating tooltips on screen at once. Defaults to 3.
+			else if (infile.key == "tooltip_visible_max") {
+				visible_max = static_cast<size_t>(Parse::toInt(infile.val));
+
+				if (visible_max < 1) {
+					visible_max = 1;
+					infile.error("EngineSettings: tooltip_visible_max must be greater than or equal to 1.");
+				}
+			}
 
 			else infile.error("EngineSettings: '%s' is not a valid key.", infile.key.c_str());
 		}
@@ -720,12 +752,13 @@ void EngineSettings::Loot::load() {
 	autopickup_currency = false;
 	autopickup_range = eset->misc.interact_range;
 	currency = "Gold";
-	vendor_ratio = 0.25f;
-	vendor_ratio_buyback = 0;
+	vendor_ratio_buy = 1.0f;
+	vendor_ratio_sell = 0.25f;
+	vendor_ratio_sell_old = 0;
 	sfx_loot = "";
 	drop_max = 1;
 	drop_radius = 1;
-	hide_radius = 3.f;
+	hide_radius = 3.0f;
 
 	FileParser infile;
 	// @CLASS EngineSettings: Loot|Description of engine/loot.txt
@@ -747,13 +780,17 @@ void EngineSettings::Loot::load() {
 				// @ATTR currency_name|string|Define the name of currency in game
 				currency = msg->get(infile.val);
 			}
-			else if (infile.key == "vendor_ratio") {
-				// @ATTR vendor_ratio|int|Percentage of item buying price to use as selling price. Also used as the buyback price until the player leaves the map.
-				vendor_ratio = static_cast<float>(Parse::toInt(infile.val)) / 100.0f;
+			else if (infile.key == "vendor_ratio_buy") {
+				// @ATTR vendor_ratio_buy|float|Global multiplier for item prices on the "Buy" tab of vendors. Defaults to 1.0.
+				vendor_ratio_buy = Parse::toFloat(infile.val);
 			}
-			else if (infile.key == "vendor_ratio_buyback") {
-				// @ATTR vendor_ratio_buyback|int|Percentage of item buying price to use as the buying price for previously sold items.
-				vendor_ratio_buyback = static_cast<float>(Parse::toInt(infile.val)) / 100.0f;
+			else if (infile.key == "vendor_ratio_sell") {
+				// @ATTR vendor_ratio_sell|float|Global multiplier for the currency gained when selling an item and the price of items on the "Sell" tab of vendors. Defaults to 0.25.
+				vendor_ratio_sell = Parse::toFloat(infile.val);
+			}
+			else if (infile.key == "vendor_ratio_sell_old") {
+				// @ATTR vendor_ratio_sell_old|float|Global multiplier for item prices on the "Sell" tab of vendors after the player has left the map or quit the game. Falls back to the value of vendor_ratio_sell by default.
+				vendor_ratio_sell_old = Parse::toFloat(infile.val);
 			}
 			else if (infile.key == "sfx_loot") {
 				// @ATTR sfx_loot|filename|Filename of a sound effect to play for dropping loot.
@@ -771,6 +808,16 @@ void EngineSettings::Loot::load() {
 				// @ATTR hide_radius|float|If an entity is within this radius relative to a piece of loot, the label will be hidden unless highlighted with the cursor.
 				hide_radius = Parse::toFloat(infile.val);
 			}
+			else if (infile.key == "vendor_ratio") {
+				// @ATTR vendor_ratio|int|(Deprecated in v1.12.85; use 'vendor_ratio_sell' instead) Percentage of item buying price to use as selling price. Also used as the buyback price until the player leaves the map.
+				vendor_ratio_sell = static_cast<float>(Parse::toInt(infile.val)) / 100.0f;
+				infile.error("EngineSettings: vendor_ratio is deprecated. Use 'vendor_ratio_sell=%.2f' instead.", vendor_ratio_sell);
+			}
+			else if (infile.key == "vendor_ratio_buyback") {
+				// @ATTR vendor_ratio_buyback|int|(Deprecated in v1.12.85; use 'vendor_ratio_sell_old' instead) Percentage of item buying price to use as the buying price for previously sold items.
+				vendor_ratio_sell_old = static_cast<float>(Parse::toInt(infile.val)) / 100.0f;
+				infile.error("EngineSettings: vendor_ratio_buyback is deprecated. Use 'vendor_ratio_sell_old=%.2f' instead.", vendor_ratio_sell_old);
+			}
 			else {
 				infile.error("EngineSettings: '%s' is not a valid key.", infile.key.c_str());
 			}
@@ -851,6 +898,7 @@ void EngineSettings::Tileset::load() {
 void EngineSettings::Widgets::load() {
 	// reset to defaults
 	selection_rect_color = Color(255, 248, 220, 255);
+	selection_rect_corner_size = 4;
 	colorblind_highlight_offset = Point(2, 2);
 
 	tab_padding = Point(8, 0);
@@ -878,6 +926,10 @@ void EngineSettings::Widgets::load() {
 					// @ATTR misc.selection_rect_color|color, int : Color, Alpha|Color of the selection rectangle when navigating widgets without a mouse.
 					selection_rect_color = Parse::toRGBA(infile.val);
 				}
+				else if (infile.key == "selection_rect_corner_size") {
+					// @ATTR misc.selection_rect_corner_size|int|Size of the corners on the selection rectangle shown when navigating widgets. Set to 0 to drawn the entire rectangle instead.
+					selection_rect_corner_size = Parse::toInt(infile.val);
+				}
 				else if (infile.key == "colorblind_highlight_offset") {
 					// @ATTR misc.colorblind_highlight_offset|int, int : X offset, Y offset|The pixel offset of the '*' marker on highlighted icons in colorblind mode.
 					colorblind_highlight_offset = Parse::toPoint(infile.val);
