@@ -76,7 +76,9 @@ EntityBehavior::EntityBehavior(Entity *_e)
 void EntityBehavior::logic() {
 	// skip all logic if the enemy is dead and no longer animating
 	if (e->stats.corpse) {
-		e->stats.corpse_timer.tick();
+		if (eset->misc.corpse_timeout_enabled)
+			e->stats.corpse_timer.tick();
+
 		return;
 	}
 
@@ -660,7 +662,13 @@ void EntityBehavior::updateState() {
 	int power_state;
 
 	// continue current animations
-	e->activeAnimation->advanceFrame();
+	if (e->activeAnimation)
+		e->activeAnimation->advanceFrame();
+
+	for (size_t i = 0; i < e->anims.size(); ++i) {
+		if (e->anims[i])
+			e->anims[i]->advanceFrame();
+	}
 
 	switch (e->stats.cur_state) {
 
