@@ -77,22 +77,31 @@ public:
 		RESPEC = 43,
 		SHOW_ON_MINIMAP = 44,
 		PARALLAX_LAYERS = 45,
-		NPC_ID = 46,
-		NPC_HOTSPOT = 47,
-		NPC_DIALOG_THEM = 48,
-		NPC_DIALOG_YOU = 49,
-		NPC_VOICE = 50,
-		NPC_DIALOG_TOPIC = 51,
-		NPC_DIALOG_GROUP = 52,
-		NPC_DIALOG_ID = 53,
-		NPC_DIALOG_RESPONSE = 54,
-		NPC_DIALOG_RESPONSE_ONLY = 55,
-		NPC_ALLOW_MOVEMENT = 56,
-		NPC_PORTRAIT_THEM = 57,
-		NPC_PORTRAIT_YOU = 58,
-		QUEST_TEXT = 59,
-		WAS_INSIDE_EVENT_AREA = 60,
-		NPC_TAKE_A_PARTY = 61
+		RANDOM_STATUS = 46,
+		NPC_ID = 47,
+		NPC_HOTSPOT = 48,
+		NPC_DIALOG_THEM = 49,
+		NPC_DIALOG_YOU = 50,
+		NPC_VOICE = 51,
+		NPC_DIALOG_TOPIC = 52,
+		NPC_DIALOG_GROUP = 53,
+		NPC_DIALOG_ID = 54,
+		NPC_DIALOG_RESPONSE = 55,
+		NPC_DIALOG_RESPONSE_ONLY = 56,
+		NPC_ALLOW_MOVEMENT = 57,
+		NPC_PORTRAIT_THEM = 58,
+		NPC_PORTRAIT_YOU = 59,
+		QUEST_TEXT = 60,
+		WAS_INSIDE_EVENT_AREA = 61,
+		NPC_TAKE_A_PARTY = 62
+	};
+
+	enum {
+		RANDOM_STATUS_MODE_APPEND = 1,
+		RANDOM_STATUS_MODE_CLEAR = 2,
+		RANDOM_STATUS_MODE_ROLL = 3,
+		RANDOM_STATUS_MODE_SET = 4,
+		RANDOM_STATUS_MODE_UNSET = 5,
 	};
 
 	int type;
