@@ -43,6 +43,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "UtilsMath.h"
 #include "UtilsParsing.h"
 #include "WidgetButton.h"
+#include "WidgetHorizontalList.h"
 #include "WidgetScrollBar.h"
 
 bool compareSaveDirs(const std::string& dir1, const std::string& dir2) {
@@ -55,6 +56,7 @@ bool compareSaveDirs(const std::string& dir1, const std::string& dir2) {
 GameSlot::GameSlot()
 	: id(0)
 	, time_played(0)
+	, active_equipment_set(0)
 	, preview_turn_timer(settings->max_frames_per_sec/2)
 {
 	preview_turn_timer.reset(Timer::BEGIN);
@@ -87,7 +89,11 @@ GameStateLoad::GameStateLoad() : GameState()
 	label_loading = new WidgetLabel();
 
 	// Confirmation box to confirm deleting
-	confirm = new MenuConfirm(msg->get("Delete Save"), msg->get("Delete this save?"));
+	confirm = new MenuConfirm();
+	confirm->setTitle(msg->get("Delete this save?"));
+	confirm->action_list->append(msg->get("No"), "");
+	confirm->action_list->append(msg->get("Yes"), "");
+
 	button_exit = new WidgetButton(WidgetButton::DEFAULT_FILE);
 	button_exit->setLabel(msg->get("Exit to Title"));
 
@@ -108,7 +114,6 @@ GameStateLoad::GameStateLoad() : GameState()
 	// Set up tab list
 	tablist = TabList();
 	tablist.setScrollType(Widget::SCROLL_HORIZONTAL);
-	tablist.ignore_no_mouse = true;
 	tablist.add(button_exit);
 	tablist.add(button_new);
 
@@ -205,6 +210,22 @@ GameStateLoad::GameStateLoad() : GameState()
 		infile.close();
 	}
 
+	// We need to read the inventory menu config to properly handle equipment sets
+	if (infile.open("menus/inventory.txt", FileParser::MOD_FILE, FileParser::ERROR_NORMAL)) {
+		while(infile.next()) {
+			if (infile.key == "equipment_slot") {
+				// we don't need the first 3 parameters, so pop them off
+				Parse::popFirstInt(infile.val);
+				Parse::popFirstInt(infile.val);
+				Parse::popFirstString(infile.val);
+
+				int eq_set = Parse::popFirstInt(infile.val);
+				equip_sets.push_back(eq_set);
+			}
+		}
+		infile.close();
+	}
+
 	// prevent text from overflowing on the right edge of game slots
 	if (text_trim_boundary == 0 || text_trim_boundary > gameslot_pos.w)
 		text_trim_boundary = gameslot_pos.w;
@@ -235,6 +256,11 @@ GameStateLoad::GameStateLoad() : GameState()
 		scrollToSelected();
 		updateButtons();
 	}
+	else if (!game_slots.empty()) {
+		setSelectedSlot(0);
+		scrollToSelected();
+		updateButtons();
+	}
 
 	render_device->setBackgroundColor(Color(0,0,0,0));
 }
@@ -341,6 +367,14 @@ void GameStateLoad::readGameSlots() {
 					repeat_val = Parse::popFirstString(infile.val);
 				}
 			}
+			else if (infile.key == "active_equipment_set") {
+				game_slots[i]->active_equipment_set = Parse::toInt(infile.val);
+
+				if (game_slots[i]->active_equipment_set > 0 && std::find(equip_sets.begin(), equip_sets.end(), game_slots[i]->active_equipment_set) == equip_sets.end()) {
+					Utils::logError("GameStateLoad: Save slot %d has an invalid active equipment set. Resetting to 0.", game_slots[i]->id);
+					game_slots[i]->active_equipment_set = 0;
+				}
+			}
 			else if (infile.key == "option") {
 				game_slots[i]->stats.gfx_base = Parse::popFirstString(infile.val);
 				game_slots[i]->stats.gfx_head = Parse::popFirstString(infile.val);
@@ -403,12 +437,20 @@ void GameStateLoad::loadPreview(GameSlot* slot) {
 	}
 
 	for (unsigned int i=0; i<slot->equipped.size(); i++) {
-		if (slot->equipped[i] != 0 && !items->items[slot->equipped[i]].has_name) {
+		if (slot->equipped[i] <= 0)
+			continue;
+
+		if (i >= equip_sets.size()) {
+			Utils::logError("GameStateLoad: Item in save slot %d with id=%d has an invalid position. Your savegame is broken or you might be using an incompatible savegame/mod", slot->id, slot->equipped[i]);
+			continue;
+		}
+
+		if (!items->items[slot->equipped[i]].has_name) {
 			Utils::logError("GameStateLoad: Item in save slot %d with id=%d is unknown. Your savegame is broken or you might be using an incompatible savegame/mod", slot->id, slot->equipped[i]);
 			continue;
 		}
 
-		if (slot->equipped[i] > 0 && !preview_layer.empty()) {
+		if ((slot->active_equipment_set == 0 || slot->active_equipment_set == equip_sets[i]) && !preview_layer.empty()) {
 			std::vector<std::string>::iterator found = find(preview_layer.begin(), preview_layer.end(), items->items[slot->equipped[i]].type);
 			if (found != preview_layer.end())
 				img_gfx[distance(preview_layer.begin(), found)] = items->items[slot->equipped[i]].gfx;
@@ -444,32 +486,50 @@ void GameStateLoad::logic() {
 
 	if (confirm->visible) {
 		confirm->logic();
-		if (confirm->confirmClicked) {
-			Utils::removeSaveDir(game_slots[selected_slot]->id);
+		if (confirm->clicked_confirm) {
+			if (confirm->action_list->getSelected() == DELETE_CONFIRM_OPTION_YES) {
+				Utils::removeSaveDir(game_slots[selected_slot]->id);
 
-			delete game_slots[selected_slot];
-			game_slots[selected_slot] = NULL;
-			game_slots.erase(game_slots.begin()+selected_slot);
+				delete game_slots[selected_slot];
+				game_slots[selected_slot] = NULL;
+				game_slots.erase(game_slots.begin()+selected_slot);
 
-			visible_slots = (game_slot_max > static_cast<int>(game_slots.size()) ? static_cast<int>(game_slots.size()) : game_slot_max);
-			setSelectedSlot(-1);
+				visible_slots = (game_slot_max > static_cast<int>(game_slots.size()) ? static_cast<int>(game_slots.size()) : game_slot_max);
+				if (!game_slots.empty())
+					setSelectedSlot(0);
+				else
+					setSelectedSlot(-1);
 
-			while (scroll_offset + visible_slots > static_cast<int>(game_slots.size())) {
-				scroll_offset--;
+				while (scroll_offset + visible_slots > static_cast<int>(game_slots.size())) {
+					scroll_offset--;
+				}
+
+				scrollToSelected();
+				updateButtons();
+
+				refreshSavePaths();
+				settings->prev_save_slot = -1;
+
+				tablist.defocus();
 			}
 
-			updateButtons();
-
+			// both yes and no close the dialog
 			confirm->visible = false;
-			confirm->confirmClicked = false;
-
-			refreshSavePaths();
-
-			settings->prev_save_slot = -1;
+			confirm->clicked_confirm = false;
 		}
 	}
 	else {
 		tablist.logic();
+
+		if (!inpt->usingMouse() && tablist.getCurrent() == -1) {
+			if (button_load->enabled)
+				tablist.setCurrent(button_load);
+			else if (button_new->enabled)
+				tablist.setCurrent(button_new);
+			else
+				tablist.setCurrent(button_exit);
+		}
+
 		if (button_exit->checkClick() || (inpt->pressing[Input::CANCEL] && !inpt->lock[Input::CANCEL])) {
 			inpt->lock[Input::CANCEL] = true;
 			showLoading();
@@ -495,8 +555,7 @@ void GameStateLoad::logic() {
 		}
 		else if (button_delete->checkClick()) {
 			// Display pop-up to make sure save should be deleted
-			confirm->visible = true;
-			confirm->render();
+			confirm->show();
 		}
 		else if (game_slots.size() > 0) {
 			Rect scroll_area = slot_pos[0];
