@@ -36,6 +36,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EntityManager.h"
 #include "EngineSettings.h"
 #include "FileParser.h"
+#include "FogOfWar.h"
 #include "GameState.h"
 #include "GameStateCutscene.h"
 #include "GameStatePlay.h"
@@ -72,7 +73,6 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "SharedGameResources.h"
 #include "SharedResources.h"
 #include "SoundManager.h"
-#include "UtilsFileSystem.h"
 #include "UtilsParsing.h"
 #include "WidgetLabel.h"
 
@@ -97,6 +97,7 @@ GameStatePlay::GameStatePlay()
 
 	loot = new LootManager();
 	powers = new PowerManager();
+	fow = new FogOfWar();
 	mapr = new MapRenderer();
 	pc = new Avatar();
 	entitym = new EntityManager();
@@ -292,6 +293,10 @@ void GameStatePlay::checkTeleport() {
 	// both map events and player powers can cause teleportation
 	if (mapr->teleportation || pc->stats.teleportation) {
 
+		if (mapr->fogofwar)
+			if(fow->fog_layer_id != 0)
+				fow->handleIntramapTeleport();
+
 		mapr->collider.unblock(pc->stats.pos.x, pc->stats.pos.y);
 
 		if (mapr->teleportation) {
@@ -327,6 +332,7 @@ void GameStatePlay::checkTeleport() {
 			inpt->lock_all = (teleport_mapname == "maps/spawn.txt");
 			mapr->executeOnMapExitEvents();
 			showLoading();
+			save_load->saveGame();
 			mapr->load(teleport_mapname);
 			setLoadingFrame();
 
@@ -578,39 +584,10 @@ void GameStatePlay::checkEquipmentChange() {
 		// force the actionbar to update when we change gear
 		menu->act->updated = true;
 
-		int feet_index = -1;
-		std::vector<Avatar::Layer_gfx> img_gfx;
-		// load only displayable layers
-		for (unsigned int j=0; j<pc->layer_reference_order.size(); j++) {
-			Avatar::Layer_gfx gfx;
-			gfx.gfx = "";
-			gfx.type = pc->layer_reference_order[j];
-			for (int i=0; i<menu->inv->inventory[MenuInventory::EQUIPMENT].getSlotNumber(); i++) {
-				if (pc->layer_reference_order[j] == menu->inv->inventory[MenuInventory::EQUIPMENT].slot_type[i]) {
-					gfx.gfx = items->items[menu->inv->inventory[MenuInventory::EQUIPMENT][i].item].gfx;
-					gfx.type = menu->inv->inventory[MenuInventory::EQUIPMENT].slot_type[i];
-				}
-				if (menu->inv->inventory[MenuInventory::EQUIPMENT].slot_type[i] == "feet") {
-					feet_index = i;
-				}
-			}
-			// special case: if we don't have a head, use the portrait's head
-			if (gfx.gfx == "" && pc->layer_reference_order[j] == "head") {
-				gfx.gfx = pc->stats.gfx_head;
-				gfx.type = "head";
-			}
-			// fall back to default if it exists
-			if (gfx.gfx == "") {
-				bool exists = Filesystem::fileExists(mods->locate("animations/avatar/" + pc->stats.gfx_base + "/default_" + gfx.type + ".txt"));
-				if (exists) gfx.gfx = "default_" + gfx.type;
-			}
-			img_gfx.push_back(gfx);
-		}
-		assert(pc->layer_reference_order.size()==img_gfx.size());
-		pc->loadGraphics(img_gfx);
+		pc->loadAnimations();
 
-		if (feet_index != -1)
-			pc->loadStepFX(items->items[menu->inv->inventory[MenuInventory::EQUIPMENT][feet_index].item].stepfx);
+		if (pc->feet_index != -1)
+			pc->loadStepFX(items->items[menu->inv->inventory[MenuInventory::EQUIPMENT][pc->feet_index].item].stepfx);
 	}
 
 	menu->inv->changed_equipment = false;
@@ -719,9 +696,6 @@ void GameStatePlay::checkNPCInteraction() {
 	mapr->event_npc = "";
 	mapr->npc_id = -1;
 
-	if (npc_id == -1)
-		return;
-
 	if (npc_id != -1) {
 		bool interact_with_npc = false;
 		if (menu->talker->npc_from_map) {
@@ -995,6 +969,8 @@ void GameStatePlay::logic() {
 		pc->stats.hp = pc->stats.get(Stats::HP_MAX);
 		pc->stats.logic();
 		pc->stats.recalc();
+		menu->pow->resetToBasePowers();
+		menu->pow->setUnlockedPowers();
 		powers->activatePassives(&pc->stats);
 		pc->respawn = false;
 	}
@@ -1130,6 +1106,7 @@ GameStatePlay::~GameStatePlay() {
 	delete camp;
 	delete items;
 	delete powers;
+	delete fow;
 
 	delete enemyg;
 
@@ -1145,5 +1122,6 @@ GameStatePlay::~GameStatePlay() {
 	menu_act = NULL;
 	menu_powers = NULL;
 	powers = NULL;
+	fow = NULL;
 }
 
