@@ -125,7 +125,6 @@ GameStateTitle::GameStateTitle()
 	label_version->setColor(font->getColor(FontEngine::COLOR_MENU_NORMAL));
 
 	// Setup tab order
-	tablist.ignore_no_mouse = true;
 	tablist.add(button_play);
 	tablist.add(button_cfg);
 	tablist.add(button_credits);
@@ -168,6 +167,10 @@ void GameStateTitle::logic() {
 	else {
 		tablist.logic();
 
+		if (!inpt->usingMouse() && tablist.getCurrent() == -1) {
+			tablist.getNext(!TabList::GET_INNER, TabList::WIDGET_SELECT_AUTO);
+		}
+
 		if (button_play->checkClick()) {
 			showLoading();
 			setRequestedGameState(new GameStateLoad());
