@@ -34,7 +34,9 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "MenuInventory.h"
 #include "MenuManager.h"
 #include "MenuPowers.h"
+#include "MenuVendor.h"
 #include "MessageEngine.h"
+#include "NPC.h"
 #include "PowerManager.h"
 #include "Settings.h"
 #include "SharedGameResources.h"
@@ -253,7 +255,7 @@ void ItemManager::loadItems(const std::string& filename) {
 			items[id].requires_class = infile.val;
 		}
 		else if (infile.key == "bonus") {
-			// @ATTR bonus|repeatable(predefined_string, int) : Stat name, Value|Adds a bonus to the item by stat name, example: bonus=hp, 50
+			// @ATTR bonus|repeatable(stat_id, int) : Stat ID, Value|Adds a bonus to the item by stat ID, example: bonus=hp, 50
 			if (clear_bonus) {
 				items[id].bonus.clear();
 				clear_bonus = false;
@@ -568,7 +570,7 @@ void ItemManager::loadSets(const std::string& filename) {
 			item_sets[id].color = Parse::toRGB(infile.val);
 		}
 		else if (infile.key == "bonus") {
-			// @ATTR bonus|repeatable(int, string, int) : Required set item count, Stat name, Value|Bonus to append to items in the set.
+			// @ATTR bonus|repeatable(int, stat_id, int) : Required set item count, Stat ID, Value|Bonus to append to items in the set.
 			if (clear_bonus) {
 				item_sets[id].bonus.clear();
 				clear_bonus = false;
@@ -625,7 +627,7 @@ void ItemManager::parseBonus(BonusData& bdata, FileParser& infile) {
 	}
 
 	for (unsigned i=0; i<eset->elements.list.size(); ++i) {
-		if (bonus_str == eset->elements.list[i].id + "_resist") {
+		if (bonus_str == eset->elements.list[i].resist_id) {
 			bdata.resist_index = i;
 			return;
 		}
@@ -698,7 +700,7 @@ TooltipData ItemManager::getShortTooltip(ItemStack stack) {
 
 	// name
 	if (stack.quantity > 1) {
-		ss << stack.quantity << " " << getItemName(stack.item);
+		ss << getItemName(stack.item) << " (" << stack.quantity << ")";
 	}
 	else {
 		ss << getItemName(stack.item);
@@ -711,7 +713,7 @@ TooltipData ItemManager::getShortTooltip(ItemStack stack) {
 /**
  * Create detailed tooltip showing all relevant item info
  */
-TooltipData ItemManager::getTooltip(ItemStack stack, StatBlock *stats, int context) {
+TooltipData ItemManager::getTooltip(ItemStack stack, StatBlock *stats, int context, bool input_hint) {
 	TooltipData tip;
 
 	if (stack.empty()) return tip;
@@ -732,8 +734,11 @@ TooltipData ItemManager::getTooltip(ItemStack stack, StatBlock *stats, int conte
 	}
 
 	// only show the name of the currency item
-	if (stack.item == eset->misc.currency_id)
+	if (stack.item == eset->misc.currency_id) {
+		if (input_hint)
+			getTooltipInputHint(tip, stack, context);
 		return tip;
+	}
 
 	// flavor text
 	if (items[stack.item].flavor != "") {
@@ -848,12 +853,12 @@ TooltipData ItemManager::getTooltip(ItemStack stack, StatBlock *stats, int conte
 	}
 
 	// buy or sell price
-	if (items[stack.item].getPrice() > 0 && stack.item != eset->misc.currency_id) {
+	if (items[stack.item].getPrice(USE_VENDOR_RATIO) > 0 && stack.item != eset->misc.currency_id) {
 		Color currency_color = getItemColor(eset->misc.currency_id);
 
 		int price_per_unit;
 		if (context == VENDOR_BUY) {
-			price_per_unit = items[stack.item].getPrice();
+			price_per_unit = items[stack.item].getPrice(USE_VENDOR_RATIO);
 			if (stats->currency < price_per_unit)
 				color = font->getColor(FontEngine::COLOR_REQUIREMENTS_NOT_MET);
 			else
@@ -913,18 +918,61 @@ TooltipData ItemManager::getTooltip(ItemStack stack, StatBlock *stats, int conte
 		}
 	}
 
-	// input hint for consumables/books
-	// TODO hint when not using mouse control. The action for using an item there is hard to describe
-	if (context == PLAYER_INV && !settings->no_mouse) {
-		if (!items[stack.item].book.empty() && items[stack.item].book_is_readable) {
-			tip.addColoredText('\n' + msg->get("Press [%s] to read", inpt->getBindingString(Input::MAIN2)), font->getColor(FontEngine::COLOR_ITEM_BONUS));
+	if (input_hint)
+		getTooltipInputHint(tip, stack, context);
+
+	return tip;
+}
+
+void ItemManager::getTooltipInputHint(TooltipData& tip, ItemStack stack, int context) {
+	bool show_activate_msg = false;
+	std::string activate_bind_str;
+
+	bool show_more_msg = false;
+	std::string more_bind_str;
+
+	if (inpt->mode == InputState::MODE_TOUCHSCREEN) {
+		tip.addColoredText('\n' + msg->get("Tap icon again for more options"), font->getColor(FontEngine::COLOR_ITEM_BONUS));
+	}
+	else if (inpt->mode == InputState::MODE_JOYSTICK) {
+		if (context == PLAYER_INV && menu->inv->canActivateItem(stack.item)) {
+			show_activate_msg = true;
+			activate_bind_str = inpt->getGamepadBindingString(Input::MENU_ACTIVATE);
 		}
-		else if (menu->inv->canActivateItem(stack.item)) {
-			tip.addColoredText('\n' + msg->get("Press [%s] to use", inpt->getBindingString(Input::MAIN2)), font->getColor(FontEngine::COLOR_ITEM_BONUS));
+		show_more_msg = true;
+		more_bind_str = inpt->getGamepadBindingString(Input::ACCEPT);
+	}
+	else if (!inpt->usingMouse()) {
+		if (context == PLAYER_INV && menu->inv->canActivateItem(stack.item)) {
+			show_activate_msg = true;
+			activate_bind_str = inpt->getBindingString(Input::MENU_ACTIVATE);
+		}
+		show_more_msg = true;
+		more_bind_str = inpt->getBindingString(Input::ACCEPT);
+	}
+	else {
+		if (context == PLAYER_INV && menu->inv->canActivateItem(stack.item)) {
+			show_activate_msg = true;
+			activate_bind_str = inpt->getBindingString(Input::MAIN2);
 		}
 	}
 
-	return tip;
+	if (show_activate_msg || show_more_msg) {
+		tip.addText("");
+	}
+
+	// input hint for consumables/books
+	if (show_activate_msg) {
+		if (!items[stack.item].book.empty() && items[stack.item].book_is_readable) {
+			tip.addColoredText(msg->get("Press [%s] to read", activate_bind_str), font->getColor(FontEngine::COLOR_ITEM_BONUS));
+		}
+		else if (menu->inv->canActivateItem(stack.item)) {
+			tip.addColoredText(msg->get("Press [%s] to use", activate_bind_str), font->getColor(FontEngine::COLOR_ITEM_BONUS));
+		}
+	}
+	if (show_more_msg) {
+		tip.addColoredText(msg->get("Press [%s] for more options", more_bind_str), font->getColor(FontEngine::COLOR_ITEM_BONUS));
+	}
 }
 
 /**
@@ -997,26 +1045,46 @@ void ItemStack::clear() {
 	can_buyback = false;
 }
 
-int Item::getPrice() {
-	return price + (price_per_level * (pc->stats.level - 1));
+int Item::getPrice(bool use_vendor_ratio) {
+	int new_price = price + (price_per_level * (pc->stats.level - 1));
+	if (new_price == 0)
+		return new_price;
+
+	new_price = static_cast<int>(static_cast<float>(new_price) * eset->loot.vendor_ratio_buy);
+
+	NPC* vendor_npc = ((menu && menu->vendor && menu->vendor->visible) ? menu->vendor->npc : NULL);
+
+	if (use_vendor_ratio) {
+		// get vendor ratio from NPC (or fall back to global value)
+		float vendor_ratio_buy = (vendor_npc && vendor_npc->vendor_ratio_buy > 0) ? vendor_npc->vendor_ratio_buy : eset->loot.vendor_ratio_buy;
+
+		new_price = static_cast<int>(static_cast<float>(new_price) * vendor_ratio_buy);
+	}
+
+	return std::max(new_price, 1);
 }
 
 int Item::getSellPrice(bool is_new_buyback) {
 	int new_price = 0;
-	if (is_new_buyback || eset->loot.vendor_ratio_buyback == 0) {
+	NPC* vendor_npc = ((menu && menu->vendor && menu->vendor->visible) ? menu->vendor->npc : NULL);
+
+	// get vendor ratio from NPC (or fall back to global value)
+	float vendor_ratio_sell = (vendor_npc && vendor_npc->vendor_ratio_sell > 0) ? vendor_npc->vendor_ratio_sell : eset->loot.vendor_ratio_sell;
+	float vendor_ratio_sell_old = (vendor_npc && vendor_npc->vendor_ratio_sell_old > 0) ? vendor_npc->vendor_ratio_sell_old : eset->loot.vendor_ratio_sell_old;
+
+	if (is_new_buyback || vendor_ratio_sell_old == 0) {
 		// default sell price
 		if (price_sell != 0)
 			new_price = price_sell;
 		else
-			new_price = static_cast<int>(static_cast<float>(getPrice()) * eset->loot.vendor_ratio);
+			new_price = static_cast<int>(static_cast<float>(getPrice(!ItemManager::USE_VENDOR_RATIO)) * vendor_ratio_sell);
 	}
 	else {
 		// sell price adjusted because the player can no longer buyback the item at the original sell price
-		new_price = static_cast<int>(static_cast<float>(getPrice()) * eset->loot.vendor_ratio_buyback);
+		new_price = static_cast<int>(static_cast<float>(getPrice(!ItemManager::USE_VENDOR_RATIO)) * vendor_ratio_sell_old);
 	}
-	if (new_price == 0) new_price = 1;
 
-	return new_price;
+	return std::max(new_price, 1);
 }
 
 // Bonus documentation
@@ -1024,7 +1092,5 @@ int Item::getSellPrice(bool is_new_buyback) {
 // @CLASS ItemManager|Description of "bonus" attribute in items/items.txt
 // @TYPE speed|Movement speed. A value of 100 is 100% speed (aka normal speed).
 // @TYPE attack_speed|Attack animation speed. A value of 100 is 100% speed (aka normal speed).
-// @TYPE ${STATNAME}|Increases ${STATNAME}, where ${STATNAME} is any of the base stats. Examples: hp, avoidance, xp_gain
-// @TYPE ${DAMAGE_TYPE}|Increases a damage min or max, where ${DAMAGE_TYPE} is any 'min' or 'max' value found in engine/damage_types.txt. Example: dmg_melee_min
-// @TYPE ${ELEMENT}_resist|Increase Resistance % to ${ELEMENT}, where ${ELEMENT} is any found in engine/elements.txt. Example: fire_resist
+// @TYPE ${STAT}|Increases ${STAT}, where ${STAT} is any valid stat_id.
 // @TYPE ${PRIMARYSTAT}|Increases ${PRIMARYSTAT}, where ${PRIMARYSTAT} is any of the primary stats defined in engine/primary_stats.txt. Example: physical
