@@ -143,7 +143,7 @@ public:
 	int no_stash;
 	std::string script;
 
-	int getPrice();
+	int getPrice(bool use_vendor_ratio);
 	int getSellPrice(bool is_new_buyback);
 
 	Item();
@@ -210,6 +210,7 @@ private:
 	void loadAll();
 	void parseBonus(BonusData& bdata, FileParser& infile);
 	void getBonusString(std::stringstream& ss, BonusData* bdata);
+	void getTooltipInputHint(TooltipData& tip, ItemStack stack, int context);
 
 public:
 	enum {
@@ -218,12 +219,14 @@ public:
 		PLAYER_INV = 2
 	};
 
+	static const bool USE_VENDOR_RATIO = true;
 	static const bool DEFAULT_SELL_PRICE = true;
+	static const bool TOOLTIP_INPUT_HINT = true;
 
 	ItemManager();
 	~ItemManager();
 	void playSound(ItemID item, const Point& pos = Point(0,0));
-	TooltipData getTooltip(ItemStack stack, StatBlock *stats, int context);
+	TooltipData getTooltip(ItemStack stack, StatBlock *stats, int context, bool input_hint);
 	TooltipData getShortTooltip(ItemStack item);
 	std::string getItemName(ItemID id);
 	std::string getItemType(const std::string& _type);
