@@ -34,10 +34,12 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EntityManager.h"
 #include "EngineSettings.h"
 #include "FileParser.h"
+#include "FogOfWar.h"
 #include "InputState.h"
 #include "LootManager.h"
 #include "MapRenderer.h"
 #include "Menu.h"
+#include "MessageEngine.h"
 #include "ModManager.h"
 #include "RenderDevice.h"
 #include "Settings.h"
@@ -87,6 +89,20 @@ void LootManager::handleNewMap() {
 }
 
 void LootManager::logic() {
+	if (inpt->pressing[Input::LOOT_TOOLTIP_MODE] && !inpt->lock[Input::LOOT_TOOLTIP_MODE]) {
+		inpt->lock[Input::LOOT_TOOLTIP_MODE] = true;
+		settings->loot_tooltips++;
+		if (settings->loot_tooltips > Settings::LOOT_TIPS_HIDE_ALL)
+			settings->loot_tooltips = Settings::LOOT_TIPS_DEFAULT;
+
+		if (settings->loot_tooltips == Settings::LOOT_TIPS_HIDE_ALL)
+			pc->logMsg(msg->get("Loot tooltip visibility") + ": " + msg->get("Hidden"), Avatar::MSG_UNIQUE);
+		else if (settings->loot_tooltips == Settings::LOOT_TIPS_DEFAULT)
+			pc->logMsg(msg->get("Loot tooltip visibility") + ": " + msg->get("Default"), Avatar::MSG_UNIQUE);
+		else if (settings->loot_tooltips == Settings::LOOT_TIPS_SHOW_ALL)
+			pc->logMsg(msg->get("Loot tooltip visibility") + ": " + msg->get("Show All"), Avatar::MSG_UNIQUE);
+	}
+
 	std::vector<Loot>::iterator it;
 	for (it = loot.begin(); it != loot.end(); ++it) {
 
@@ -132,6 +148,13 @@ void LootManager::renderTooltips(const FPoint& cam) {
 		it->tip_visible = false;
 
 		if (it->on_ground) {
+			if (mapr->fogofwar > FogOfWar::TYPE_MINIMAP) {
+				float delta = Utils::calcDist(pc->stats.pos, it->pos);
+				if (delta > fow->mask_radius-1.0) {
+					break;
+				}
+			}
+
 			Point p = Utils::mapToScreen(it->pos.x, it->pos.y, cam.x, cam.y);
 			if (!Utils::isWithinRect(screen_rect, p)) {
 				++it;
@@ -501,6 +524,13 @@ ItemStack LootManager::checkNearestPickup(const FPoint& hero_pos) {
 void LootManager::addRenders(std::vector<Renderable> &ren, std::vector<Renderable> &ren_dead) {
 	std::vector<Loot>::iterator it;
 	for (it = loot.begin(); it != loot.end(); ++it) {
+		if (mapr->fogofwar > FogOfWar::TYPE_MINIMAP) {
+			float delta = Utils::calcDist(pc->stats.pos, it->pos);
+			if (delta > fow->mask_radius-1.0) {
+				continue;
+			}
+		}
+
 		if (it->animation) {
 			Renderable r = it->animation->getCurrentFrame(0);
 			r.map_pos.x = it->pos.x;
@@ -706,7 +736,7 @@ void LootManager::checkLootComponent(EventComponent* ec, FPoint *pos, std::vecto
 		addLoot(new_loot, p, !DROPPED_BY_HERO);
 }
 
-void LootManager::removeFromEnemiesDroppingLoot(StatBlock* sb) {
+void LootManager::removeFromEnemiesDroppingLoot(const StatBlock* sb) {
 	for (size_t i = enemiesDroppingLoot.size(); i > 0; i--) {
 		// enemies will actually be removed the next time checkEnemiesForLoot() runs
 		if (enemiesDroppingLoot[i-1] == sb)
