@@ -24,13 +24,18 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EngineSettings.h"
 #include "EventManager.h"
 #include "FileParser.h"
+#include "FogOfWar.h"
 #include "Map.h"
+#include "MapRenderer.h"
 #include "MessageEngine.h"
 #include "ModManager.h"
 #include "PowerManager.h"
+#include "SaveLoad.h"
+#include "Settings.h"
 #include "SharedResources.h"
 #include "SharedGameResources.h"
 #include "StatBlock.h"
+#include "Utils.h"
 #include "UtilsParsing.h"
 
 Map::Map()
@@ -41,7 +46,9 @@ Map::Map()
 	, h(1)
 	, hero_pos_enabled(false)
 	, hero_pos()
-	, background_color(0,0,0,0) {
+	, background_color(0,0,0,0)
+	, fogofwar(eset->misc.fogofwar)
+	, save_fogofwar(eset->misc.save_fogofwar) {
 }
 
 Map::~Map() {
@@ -55,7 +62,7 @@ void Map::clearLayers() {
 
 void Map::clearQueues() {
 	enemies = std::queue<Map_Enemy>();
-	npcs = std::queue<Map_NPC>();
+	map_npcs = std::queue<Map_NPC>();
 }
 
 void Map::clearEvents() {
@@ -79,6 +86,8 @@ int Map::load(const std::string& fname) {
 	music_filename = "";
 	parallax_filename = "";
 	background_color = Color(0,0,0,0);
+	fogofwar = eset->misc.fogofwar;
+	save_fogofwar = eset->misc.save_fogofwar;
 
 	w = 1;
 	h = 1;
@@ -101,7 +110,7 @@ int Map::load(const std::string& fname) {
 			if (infile.section == "enemy")
 				enemy_groups.push(Map_Group());
 			else if (infile.section == "npc")
-				npcs.push(Map_NPC());
+				map_npcs.push(Map_NPC());
 			else if (infile.section == "event")
 				events.push_back(Event());
 
@@ -120,6 +129,8 @@ int Map::load(const std::string& fname) {
 
 	infile.close();
 
+	if (fogofwar) fow->load();
+
 	// create StatBlocks for events that need powers
 	for (unsigned i=0; i<events.size(); ++i) {
 		EventComponent *ec_power = events[i].getComponent(EventComponent::POWER);
@@ -139,6 +150,40 @@ int Map::load(const std::string& fname) {
 		}
 	}
 
+	// ensure that our map contains a fog of war layer
+	if (fogofwar) {
+		if (save_fogofwar) {
+			std::stringstream ss;
+			ss.str("");
+			ss << settings->path_user << "saves/" << eset->misc.save_prefix << "/" << save_load->getGameSlot() << "/fow/" << Utils::hashString(mapr->getFilename()) << ".txt";
+			if (infile.open(ss.str(), !FileParser::MOD_FILE, FileParser::ERROR_NORMAL)) {
+				while (infile.next()) {
+					if (infile.section == "layer") {
+						loadLayer(infile);
+					}
+				}
+				infile.close();
+			}
+		}
+		if (std::find(layernames.begin(), layernames.end(), "fow_fog") == layernames.end()) {
+			layernames.push_back("fow_fog");
+			layers.resize(layers.size()+1);
+			layers.back().resize(w);
+			for (size_t i=0; i<layers.back().size(); ++i) {
+				layers.back()[i].resize(h, FogOfWar::TILE_HIDDEN);
+			}
+		}
+
+		if (std::find(layernames.begin(), layernames.end(), "fow_dark") == layernames.end()) {
+			layernames.push_back("fow_dark");
+			layers.resize(layers.size()+1);
+			layers.back().resize(w);
+			for (size_t i=0; i<layers.back().size(); ++i) {
+				layers.back()[i].resize(h, FogOfWar::TILE_HIDDEN);
+			}
+		}
+	}
+
 	if (!hero_pos_enabled) {
 		Utils::logError("Map: Hero spawn position (hero_pos) not defined in map header. Defaulting to (0,0).");
 	}
@@ -181,6 +226,14 @@ void Map::loadHeader(FileParser &infile) {
 		// @ATTR background_color|color, int : Color, alpha|Background color for the map.
 		background_color = Parse::toRGBA(infile.val);
 	}
+	else if (infile.key == "fogofwar") {
+		// @ATTR fogofwar|int|Set the fog of war type. 0-disabled, 1-minimap, 2-tint, 3-overlay. Overrides engine settings.
+		fogofwar = static_cast<unsigned short>(Parse::toInt(infile.val));
+	}
+	else if (infile.key == "save_fogofwar") {
+		// @ATTR save_fogofwar|bool|If true, the fog of war layer keeps track of the progress. Overrides engine settings.
+		save_fogofwar = Parse::toBool(infile.val);
+	}
 	else if (infile.key == "tilewidth") {
 		// @ATTR tilewidth|int|Inherited from Tiled map file. Unused by engine.
 	}
@@ -308,31 +361,109 @@ void Map::loadEnemyGroup(FileParser &infile, Map_Group *group) {
 		}
 	}
 	else if (infile.key == "requires_status") {
-		// @ATTR enemygroup.requires_status|list(string)|Status required for loading enemies
+		// @ATTR enemygroup.requires_status|list(string)|Statuses required to be set for enemy group to load
 		std::string s;
-		while ((s = Parse::popFirstString(infile.val)) != "") {
-			group->requires_status.push_back(camp->registerStatus(s));
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_STATUS;
+			ec.status = camp->registerStatus(s);
+			group->requirements.push_back(ec);
 		}
 	}
 	else if (infile.key == "requires_not_status") {
-		// @ATTR enemygroup.requires_not_status|list(string)|Status required to be missing for loading enemies
+		// @ATTR enemygroup.requires_not_status|list(string)|Statuses required to be unset for enemy group to load
 		std::string s;
-		while ((s = Parse::popFirstString(infile.val)) != "") {
-			group->requires_not_status.push_back(camp->registerStatus(s));
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_NOT_STATUS;
+			ec.status = camp->registerStatus(s);
+			group->requirements.push_back(ec);
 		}
 	}
+	else if (infile.key == "requires_level") {
+		// @ATTR enemygroup.requires_level|int|Player level must be equal or greater to load enemy group
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_LEVEL;
+		ec.x = Parse::popFirstInt(infile.val);
+		group->requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_not_level") {
+		// @ATTR enemygroup.requires_not_level|int|Player level must be lesser to load enemy group
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_NOT_LEVEL;
+		ec.x = Parse::popFirstInt(infile.val);
+		group->requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_currency") {
+		// @ATTR enemygroup.requires_currency|int|Player currency must be equal or greater to load enemy group
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_CURRENCY;
+		ec.x = Parse::popFirstInt(infile.val);
+		group->requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_not_currency") {
+		// @ATTR enemygroup.requires_not_currency|int|Player currency must be lesser to load enemy group
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_NOT_CURRENCY;
+		ec.x = Parse::popFirstInt(infile.val);
+		group->requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_item") {
+		// @ATTR enemygroup.requires_item|list(item_id)|Item required to exist in player inventory to load enemy group. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
+		std::string s;
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			ItemStack item_stack = Parse::toItemQuantityPair(s);
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_ITEM;
+			ec.id = item_stack.item;
+			ec.x = item_stack.quantity;
+			group->requirements.push_back(ec);
+		}
+	}
+	else if (infile.key == "requires_not_item") {
+		// @ATTR enemygroup.requires_not_item|list(item_id)|Item required to not exist in player inventory to load enemy group. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
+		std::string s;
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			ItemStack item_stack = Parse::toItemQuantityPair(s);
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_NOT_ITEM;
+			ec.id = item_stack.item;
+			ec.x = item_stack.quantity;
+			group->requirements.push_back(ec);
+		}
+	}
+	else if (infile.key == "requires_class") {
+		// @ATTR enemygroup.requires_class|predefined_string|Player base class required to load enemy group
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_CLASS;
+		ec.s = Parse::popFirstString(infile.val);
+		group->requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_not_class") {
+		// @ATTR enemygroup.requires_not_class|predefined_string|Player base class not required to load enemy group
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_NOT_CLASS;
+		ec.s = Parse::popFirstString(infile.val);
+		group->requirements.push_back(ec);
+	}
 	else if (infile.key == "invincible_requires_status") {
 		// @ATTR enemygroup.invincible_requires_status|list(string)|Enemies in this group are invincible to hero attacks when these statuses are set.
 		std::string s;
 		while ((s = Parse::popFirstString(infile.val)) != "") {
-			group->invincible_requires_status.push_back(camp->registerStatus(s));
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_STATUS;
+			ec.status = camp->registerStatus(s);
+			group->invincible_requirements.push_back(ec);
 		}
 	}
 	else if (infile.key == "invincible_requires_not_status") {
 		// @ATTR enemygroup.invincible_requires_not_status|list(string)|Enemies in this group are invincible to hero attacks when these statuses are not set.
 		std::string s;
 		while ((s = Parse::popFirstString(infile.val)) != "") {
-			group->invincible_requires_not_status.push_back(camp->registerStatus(s));
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_NOT_STATUS;
+			ec.status = camp->registerStatus(s);
+			group->invincible_requirements.push_back(ec);
 		}
 	}
 	else {
@@ -341,29 +472,108 @@ void Map::loadEnemyGroup(FileParser &infile, Map_Group *group) {
 }
 
 void Map::loadNPC(FileParser &infile) {
-	std::string s;
 	if (infile.key == "type") {
 		// @ATTR npc.type|string|(IGNORED BY ENGINE) The "type" field, as used by Tiled and other mapping tools.
-		npcs.back().type = infile.val;
+		map_npcs.back().type = infile.val;
 	}
 	else if (infile.key == "filename") {
 		// @ATTR npc.filename|string|Filename of an NPC definition.
-		npcs.back().id = infile.val;
-	}
-	else if (infile.key == "requires_status") {
-		// @ATTR npc.requires_status|list(string)|Status required for NPC load. There can be multiple states, separated by comma
-		while ( (s = Parse::popFirstString(infile.val)) != "")
-			npcs.back().requires_status.push_back(camp->registerStatus(s));
-	}
-	else if (infile.key == "requires_not_status") {
-		// @ATTR npc.requires_not_status|list(string)|Status required to be missing for NPC load. There can be multiple states, separated by comma
-		while ( (s = Parse::popFirstString(infile.val)) != "")
-			npcs.back().requires_not_status.push_back(camp->registerStatus(s));
+		map_npcs.back().id = infile.val;
 	}
 	else if (infile.key == "location") {
 		// @ATTR npc.location|point|Location of NPC
-		npcs.back().pos.x = static_cast<float>(Parse::popFirstInt(infile.val)) + 0.5f;
-		npcs.back().pos.y = static_cast<float>(Parse::popFirstInt(infile.val)) + 0.5f;
+		map_npcs.back().pos.x = static_cast<float>(Parse::popFirstInt(infile.val)) + 0.5f;
+		map_npcs.back().pos.y = static_cast<float>(Parse::popFirstInt(infile.val)) + 0.5f;
+	}
+	else if (infile.key == "requires_status") {
+		// @ATTR npc.requires_status|list(string)|Statuses required to be set for NPC load
+		std::string s;
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_STATUS;
+			ec.status = camp->registerStatus(s);
+			map_npcs.back().requirements.push_back(ec);
+		}
+	}
+	else if (infile.key == "requires_not_status") {
+		// @ATTR npc.requires_not_status|list(string)|Statuses required to be unset for NPC load
+		std::string s;
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_NOT_STATUS;
+			ec.status = camp->registerStatus(s);
+			map_npcs.back().requirements.push_back(ec);
+		}
+	}
+	else if (infile.key == "requires_level") {
+		// @ATTR npc.requires_level|int|Player level must be equal or greater to load NPC
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_LEVEL;
+		ec.x = Parse::popFirstInt(infile.val);
+		map_npcs.back().requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_not_level") {
+		// @ATTR npc.requires_not_level|int|Player level must be lesser to load NPC
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_NOT_LEVEL;
+		ec.x = Parse::popFirstInt(infile.val);
+		map_npcs.back().requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_currency") {
+		// @ATTR npc.requires_currency|int|Player currency must be equal or greater to load NPC
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_CURRENCY;
+		ec.x = Parse::popFirstInt(infile.val);
+		map_npcs.back().requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_not_currency") {
+		// @ATTR npc.requires_not_currency|int|Player currency must be lesser to load NPC
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_NOT_CURRENCY;
+		ec.x = Parse::popFirstInt(infile.val);
+		map_npcs.back().requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_item") {
+		// @ATTR npc.requires_item|list(item_id)|Item required to exist in player inventory to load NPC. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
+		std::string s;
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			ItemStack item_stack = Parse::toItemQuantityPair(s);
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_ITEM;
+			ec.id = item_stack.item;
+			ec.x = item_stack.quantity;
+			map_npcs.back().requirements.push_back(ec);
+		}
+	}
+	else if (infile.key == "requires_not_item") {
+		// @ATTR npc.requires_not_item|list(item_id)|Item required to not exist in player inventory to load NPC. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
+		std::string s;
+		while ( (s = Parse::popFirstString(infile.val)) != "") {
+			ItemStack item_stack = Parse::toItemQuantityPair(s);
+			EventComponent ec;
+			ec.type = EventComponent::REQUIRES_NOT_ITEM;
+			ec.id = item_stack.item;
+			ec.x = item_stack.quantity;
+			map_npcs.back().requirements.push_back(ec);
+		}
+	}
+	else if (infile.key == "requires_class") {
+		// @ATTR npc.requires_class|predefined_string|Player base class required to load NPC
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_CLASS;
+		ec.s = Parse::popFirstString(infile.val);
+		map_npcs.back().requirements.push_back(ec);
+	}
+	else if (infile.key == "requires_not_class") {
+		// @ATTR npc.requires_not_class|predefined_string|Player base class not required to load NPC
+		EventComponent ec;
+		ec.type = EventComponent::REQUIRES_NOT_CLASS;
+		ec.s = Parse::popFirstString(infile.val);
+		map_npcs.back().requirements.push_back(ec);
+	}
+	else if (infile.key == "direction") {
+		// @ATTR npc.direction|direction|Direction that NPC will initially face.
+		map_npcs.back().direction = Parse::toDirection(infile.val);
 	}
 	else {
 		infile.error("Map: '%s' is not a valid key.", infile.key.c_str());
