@@ -30,12 +30,16 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EntityManager.h"
 #include "EngineSettings.h"
 #include "EventManager.h"
+#include "FogOfWar.h"
+#include "FontEngine.h"
 #include "Hazard.h"
 #include "HazardManager.h"
 #include "InputState.h"
 #include "MapRenderer.h"
 #include "MenuDevConsole.h"
 #include "MenuManager.h"
+#include "NPC.h"
+#include "NPCManager.h"
 #include "PowerManager.h"
 #include "RenderDevice.h"
 #include "Settings.h"
@@ -104,10 +108,8 @@ bool MapRenderer::enemyGroupPlaceEnemy(float x, float y, const Map_Group &g) {
 
 			group_member.direction = (g.direction == -1 ? rand()%8 : g.direction);
 			group_member.wander_radius = g.wander_radius;
-			group_member.requires_status = g.requires_status;
-			group_member.requires_not_status = g.requires_not_status;
-			group_member.invincible_requires_status = g.invincible_requires_status;
-			group_member.invincible_requires_not_status = g.invincible_requires_not_status;
+			group_member.requirements = g.requirements;
+			group_member.invincible_requirements = g.invincible_requirements;
 
 			if (g.area.x == 1 && g.area.y == 1) {
 				// this is a single enemy
@@ -217,6 +219,14 @@ int MapRenderer::load(const std::string& fname) {
 	for (unsigned i = 0; i < layers.size(); ++i)
 		if (layernames[i] == "object")
 			index_objectlayer = i;
+	if (fogofwar) {
+		for (unsigned short i = 0; i < layers.size(); ++i) {
+			if (layernames[i] == "fow_dark")
+				fow->dark_layer_id = i;
+			if (layernames[i] == "fow_fog")
+				fow->fog_layer_id = i;
+		}
+	}
 
 	while (!enemy_groups.empty()) {
 		pushEnemyGroup(enemy_groups.front());
@@ -230,7 +240,17 @@ int MapRenderer::load(const std::string& fname) {
 		for (unsigned x = 0; x < layers[i].size(); ++x) {
 			for (unsigned y = 0; y < layers[i][x].size(); ++y) {
 				const unsigned tile_id = layers[i][x][y];
-				if (tile_id > 0 && (tile_id >= tset.tiles.size() || tset.tiles[tile_id].tile == NULL)) {
+				TileSet* tile_set = &tset;
+
+				if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+					if (i == fow->dark_layer_id) tile_set = &fow->tset_dark;
+					if (i == fow->fog_layer_id) tile_set = &fow->tset_fog;
+			    }
+			    if (fogofwar)
+					if (i == fow->dark_layer_id || i == fow->fog_layer_id)
+						continue;
+
+				if (tile_id > 0 && (tile_id >= tile_set->tiles.size() || tile_set->tiles[tile_id].tile == NULL)) {
 					if (std::find(corrupted.begin(), corrupted.end(), tile_id) == corrupted.end()) {
 						corrupted.push_back(tile_id);
 					}
@@ -268,9 +288,16 @@ void MapRenderer::loadMusic() {
 }
 
 void MapRenderer::logic(bool paused) {
+	if (fogofwar) {
+		fow->logic();
+	}
 
 	// handle tile set logic e.g. animations
 	tset.logic();
+	if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+		fow->tset_dark.logic();
+		fow->tset_fog.logic();
+	}
 
 	// TODO there's a bit too much "logic" here for a class that's supposed to be dedicated to rendering
 	// some of these timers should be moved out at some point
@@ -320,9 +347,9 @@ void calculatePriosIso(std::vector<Renderable> &r) {
 	for (std::vector<Renderable>::iterator it = r.begin(); it != r.end(); ++it) {
 		const unsigned tilex = static_cast<unsigned>(floorf(it->map_pos.x));
 		const unsigned tiley = static_cast<unsigned>(floorf(it->map_pos.y));
-		const int commax = static_cast<int>((it->map_pos.x - static_cast<float>(tilex)) * (2<<16));
-		const int commay = static_cast<int>((it->map_pos.y - static_cast<float>(tiley)) * (2<<16));
-		it->prio += (static_cast<uint64_t>(tilex + tiley) << 54) + (static_cast<uint64_t>(tilex) << 42) + (static_cast<uint64_t>(commax + commay) << 16);
+		const int commax = static_cast<int>((it->map_pos.x - static_cast<float>(tilex)) * (1<<10));
+		const int commay = static_cast<int>((it->map_pos.y - static_cast<float>(tiley)) * (1<<10));
+		it->prio += (static_cast<uint64_t>(tilex + tiley) << 37) + (static_cast<uint64_t>(tilex) << 20) + (static_cast<uint64_t>(commax + commay) << 8);
 	}
 }
 
@@ -330,8 +357,8 @@ void calculatePriosOrtho(std::vector<Renderable> &r) {
 	for (std::vector<Renderable>::iterator it = r.begin(); it != r.end(); ++it) {
 		const unsigned tilex = static_cast<unsigned>(floorf(it->map_pos.x));
 		const unsigned tiley = static_cast<unsigned>(floorf(it->map_pos.y));
-		const int commay = static_cast<int>(1024 * it->map_pos.y);
-		it->prio += (static_cast<uint64_t>(tiley) << 48) + (static_cast<uint64_t>(tilex) << 32) + (static_cast<uint64_t>(commay) << 16);
+		const int commay = static_cast<int>(it->map_pos.y * (1<<10));
+		it->prio += (static_cast<uint64_t>(tiley) << 37) + (static_cast<uint64_t>(tilex) << 20) + (static_cast<uint64_t>(commay) << 8);
 	}
 }
 
@@ -367,7 +394,7 @@ void MapRenderer::drawRenderable(std::vector<Renderable>::iterator r_cursor) {
 	}
 }
 
-void MapRenderer::renderIsoLayer(const Map_Layer& layerdata) {
+void MapRenderer::renderIsoLayer(const Map_Layer& layerdata, const TileSet& tile_set) {
 	int_fast16_t i; // first index of the map array
 	int_fast16_t j; // second index of the map array
 	Point dest;
@@ -411,12 +438,61 @@ void MapRenderer::renderIsoLayer(const Map_Layer& layerdata) {
 			p.x += eset->tileset.tile_w;
 
 			if (const uint_fast16_t current_tile = layerdata[i][j]) {
-				const Tile_Def &tile = tset.tiles[current_tile];
+				const Tile_Def &tile = tile_set.tiles[current_tile];
 				dest.x = p.x - tile.offset.x;
 				dest.y = p.y - tile.offset.y;
+
+				//skip rendering tiles that are underneath fow hidden tiles
+				if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+					if (&layerdata != &layers[fow->dark_layer_id]) {
+						if (layers[fow->dark_layer_id][i][j] == FogOfWar::TILE_HIDDEN) {
+
+							//check tile's corners
+							Point t_l(Utils::screenToMap(dest.x, dest.y, cam.shake.x, cam.shake.y));
+							Point t_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y, cam.shake.x, cam.shake.y));
+							Point b_l(Utils::screenToMap(dest.x, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+							Point b_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+
+							//limit to map bounds
+							if (t_l.x < 0) t_l.x = 0;
+							if (t_l.x >= w) t_l.x = w-1;
+							if (t_l.y < 0) t_l.y = 0;
+							if (t_l.y >= h) t_l.y = h-1;
+
+							if (t_r.x < 0) t_r.x = 0;
+							if (t_r.x >= w) t_r.x = w-1;
+							if (t_r.y < 0) t_r.y = 0;
+							if (t_r.y >= h) t_r.y = h-1;
+
+							if (b_l.x < 0) b_l.x = 0;
+							if (b_l.x >= w) b_l.x = w-1;
+							if (b_l.y < 0) b_l.y = 0;
+							if (b_l.y >= h) b_l.y = h-1;
+
+							if (b_r.x < 0) b_r.x = 0;
+							if (b_r.x >= w) b_r.x = w-1;
+							if (b_r.y < 0) b_r.y = 0;
+							if (b_r.y >= h) b_r.y = h-1;
+
+							if (layers[fow->dark_layer_id][t_l.x][t_l.y] == FogOfWar::TILE_HIDDEN) {
+								if (layers[fow->dark_layer_id][t_r.x][t_r.y] == FogOfWar::TILE_HIDDEN) {
+									if (layers[fow->dark_layer_id][b_l.x][b_l.y] == FogOfWar::TILE_HIDDEN) {
+										if (layers[fow->dark_layer_id][b_r.x][b_r.y] == FogOfWar::TILE_HIDDEN) {
+											continue;
+										}
+									}
+								}
+							}
+						}
+					}
+				}
+
 				// no need to set w and h in dest, as it is ignored
 				// by SDL_BlitSurface
 				tile.tile->setDestFromPoint(dest);
+				if (fogofwar == FogOfWar::TYPE_TINT) {
+					tile.tile->color_mod = fow->getTileColorMod(i, j);
+				}
 				render_device->render(tile.tile);
 			}
 		}
@@ -513,7 +589,56 @@ void MapRenderer::renderIsoFrontObjects(std::vector<Renderable> &r) {
 					dest.x = p.x - tile.offset.x;
 					dest.y = p.y - tile.offset.y;
 					tile.tile->setDestFromPoint(dest);
+
+					//skip rendering tiles that are underneath fow hidden tiles
+					if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+						if (&current_layer != &layers[fow->dark_layer_id]) {
+							if (layers[fow->dark_layer_id][i][j] == FogOfWar::TILE_HIDDEN) {
+
+								//check tile's corners
+								Point t_l(Utils::screenToMap(dest.x, dest.y, cam.shake.x, cam.shake.y));
+								Point t_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y, cam.shake.x, cam.shake.y));
+								Point b_l(Utils::screenToMap(dest.x, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+								Point b_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+
+								//limit to map bounds
+								if (t_l.x < 0) t_l.x = 0;
+								if (t_l.x >= w) t_l.x = w-1;
+								if (t_l.y < 0) t_l.y = 0;
+								if (t_l.y >= h) t_l.y = h-1;
+
+								if (t_r.x < 0) t_r.x = 0;
+								if (t_r.x >= w) t_r.x = w-1;
+								if (t_r.y < 0) t_r.y = 0;
+								if (t_r.y >= h) t_r.y = h-1;
+
+								if (b_l.x < 0) b_l.x = 0;
+								if (b_l.x >= w) b_l.x = w-1;
+								if (b_l.y < 0) b_l.y = 0;
+								if (b_l.y >= h) b_l.y = h-1;
+
+								if (b_r.x < 0) b_r.x = 0;
+								if (b_r.x >= w) b_r.x = w-1;
+								if (b_r.y < 0) b_r.y = 0;
+								if (b_r.y >= h) b_r.y = h-1;
+
+								if (layers[fow->dark_layer_id][t_l.x][t_l.y] == FogOfWar::TILE_HIDDEN) {
+									if (layers[fow->dark_layer_id][t_r.x][t_r.y] == FogOfWar::TILE_HIDDEN) {
+										if (layers[fow->dark_layer_id][b_l.x][b_l.y] == FogOfWar::TILE_HIDDEN) {
+											if (layers[fow->dark_layer_id][b_r.x][b_r.y] == FogOfWar::TILE_HIDDEN) {
+												continue;
+											}
+										}
+									}
+								}
+							}
+						}
+					}
+
 					checkHiddenEntities(i, j, current_layer, r);
+					if (fogofwar == FogOfWar::TYPE_TINT) {
+						tile.tile->color_mod = fow->getTileColorMod(i, j);
+					}
 					render_device->render(tile.tile);
 					drawn_tiles[i][j] = 1;
 				}
@@ -596,6 +721,9 @@ do_last_NE_tile:
 					dest.y = tile_SW_center.y - tile.offset.y;
 					tile.tile->setDestFromPoint(dest);
 					checkHiddenEntities(i, j, current_layer, r);
+					if (fogofwar == FogOfWar::TYPE_TINT) {
+						tile.tile->color_mod = fow->getTileColorMod(i, j);
+					}
 					render_device->render(tile.tile);
 					drawn_tiles[i-2][j+2] = 1;
 				}
@@ -614,6 +742,9 @@ do_last_NE_tile:
 					dest.y = tile_NE_center.y - tile.offset.y;
 					tile.tile->setDestFromPoint(dest);
 					checkHiddenEntities(i, j, current_layer, r);
+					if (fogofwar == FogOfWar::TYPE_TINT) {
+						tile.tile->color_mod = fow->getTileColorMod(i, j);
+					}
 					render_device->render(tile.tile);
 					drawn_tiles[i][j] = 1;
 				}
@@ -653,8 +784,9 @@ do_last_NE_tile:
 
 void MapRenderer::renderIso(std::vector<Renderable> &r, std::vector<Renderable> &r_dead) {
 	size_t index = 0;
+
 	while (index < index_objectlayer) {
-		renderIsoLayer(layers[index]);
+		renderIsoLayer(layers[index], tset);
 		map_parallax.render(cam.shake, layernames[index]);
 		index++;
 	}
@@ -665,7 +797,20 @@ void MapRenderer::renderIso(std::vector<Renderable> &r, std::vector<Renderable>
 
 	index++;
 	while (index < layers.size()) {
-		renderIsoLayer(layers[index]);
+		if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+			if (layernames[index] == "fow_dark") {
+				renderIsoLayer(layers[index],fow->tset_dark);
+			}
+			else if (layernames[index] == "fow_fog") {
+				renderIsoLayer(layers[index],fow->tset_fog);
+			}
+			else {
+				renderIsoLayer(layers[index], tset);
+			}
+		}
+		else if (layernames[index] != "fow_dark" && layernames[index] != "fow_fog") {
+			renderIsoLayer(layers[index], tset);
+		}
 		map_parallax.render(cam.shake, layernames[index]);
 		index++;
 	}
@@ -676,7 +821,7 @@ void MapRenderer::renderIso(std::vector<Renderable> &r, std::vector<Renderable>
 	drawDevCursor();
 }
 
-void MapRenderer::renderOrthoLayer(const Map_Layer& layerdata) {
+void MapRenderer::renderOrthoLayer(const Map_Layer& layerdata, const TileSet& tile_set) {
 
 	Point dest;
 	const Point upperleft(Utils::screenToMap(0, 0, cam.shake.x, cam.shake.y));
@@ -695,11 +840,64 @@ void MapRenderer::renderOrthoLayer(const Map_Layer& layerdata) {
 		for (i = starti; i < max_tiles_width; i++) {
 
 			if (const unsigned short current_tile = layerdata[i][j]) {
-				const Tile_Def &tile = tset.tiles[current_tile];
+				const Tile_Def &tile = tile_set.tiles[current_tile];
 				dest.x = p.x - tile.offset.x;
 				dest.y = p.y - tile.offset.y;
+
+				bool skip_tile_render = false;
+
+				//skip rendering tiles that are underneath fow hidden tiles
+				if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+					if (&layerdata != &layers[fow->dark_layer_id]) {
+						if (layers[fow->dark_layer_id][i][j] == FogOfWar::TILE_HIDDEN) {
+
+							//check tile's corners
+							Point t_l(Utils::screenToMap(dest.x, dest.y, cam.shake.x, cam.shake.y));
+							Point t_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y, cam.shake.x, cam.shake.y));
+							Point b_l(Utils::screenToMap(dest.x, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+							Point b_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+
+							//limit to map bounds
+							if (t_l.x < 0) t_l.x = 0;
+							if (t_l.x >= w) t_l.x = w-1;
+							if (t_l.y < 0) t_l.y = 0;
+							if (t_l.y >= h) t_l.y = h-1;
+
+							if (t_r.x < 0) t_r.x = 0;
+							if (t_r.x >= w) t_r.x = w-1;
+							if (t_r.y < 0) t_r.y = 0;
+							if (t_r.y >= h) t_r.y = h-1;
+
+							if (b_l.x < 0) b_l.x = 0;
+							if (b_l.x >= w) b_l.x = w-1;
+							if (b_l.y < 0) b_l.y = 0;
+							if (b_l.y >= h) b_l.y = h-1;
+
+							if (b_r.x < 0) b_r.x = 0;
+							if (b_r.x >= w) b_r.x = w-1;
+							if (b_r.y < 0) b_r.y = 0;
+							if (b_r.y >= h) b_r.y = h-1;
+
+							if (layers[fow->dark_layer_id][t_l.x][t_l.y] == FogOfWar::TILE_HIDDEN) {
+								if (layers[fow->dark_layer_id][t_r.x][t_r.y] == FogOfWar::TILE_HIDDEN) {
+									if (layers[fow->dark_layer_id][b_l.x][b_l.y] == FogOfWar::TILE_HIDDEN) {
+										if (layers[fow->dark_layer_id][b_r.x][b_r.y] == FogOfWar::TILE_HIDDEN) {
+											skip_tile_render = true;
+										}
+									}
+								}
+							}
+						}
+					}
+				}
+
 				tile.tile->setDestFromPoint(dest);
-				render_device->render(tile.tile);
+				if (!skip_tile_render) {
+					if (fogofwar == FogOfWar::TYPE_TINT) {
+						tile.tile->color_mod = fow->getTileColorMod(i, j);
+					}
+					render_device->render(tile.tile);
+				}
 			}
 			p.x += eset->tileset.tile_w;
 		}
@@ -744,8 +942,61 @@ void MapRenderer::renderOrthoFrontObjects(std::vector<Renderable> &r) {
 				dest.x = p.x - tile.offset.x;
 				dest.y = p.y - tile.offset.y;
 				tile.tile->setDestFromPoint(dest);
+
+				bool skip_tile_render = false;
+
+				//skip rendering tiles that are underneath fow hidden tiles
+				if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+					if (&layers[index_objectlayer] != &layers[fow->dark_layer_id]) {
+						if (layers[fow->dark_layer_id][i][j] == FogOfWar::TILE_HIDDEN) {
+
+							//check tile's corners
+							Point t_l(Utils::screenToMap(dest.x, dest.y, cam.shake.x, cam.shake.y));
+							Point t_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y, cam.shake.x, cam.shake.y));
+							Point b_l(Utils::screenToMap(dest.x, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+							Point b_r(Utils::screenToMap(dest.x + tile.tile->getClip().w, dest.y + tile.tile->getClip().h, cam.shake.x, cam.shake.y));
+
+							//limit to map bounds
+							if (t_l.x < 0) t_l.x = 0;
+							if (t_l.x >= w) t_l.x = w-1;
+							if (t_l.y < 0) t_l.y = 0;
+							if (t_l.y >= h) t_l.y = h-1;
+
+							if (t_r.x < 0) t_r.x = 0;
+							if (t_r.x >= w) t_r.x = w-1;
+							if (t_r.y < 0) t_r.y = 0;
+							if (t_r.y >= h) t_r.y = h-1;
+
+							if (b_l.x < 0) b_l.x = 0;
+							if (b_l.x >= w) b_l.x = w-1;
+							if (b_l.y < 0) b_l.y = 0;
+							if (b_l.y >= h) b_l.y = h-1;
+
+							if (b_r.x < 0) b_r.x = 0;
+							if (b_r.x >= w) b_r.x = w-1;
+							if (b_r.y < 0) b_r.y = 0;
+							if (b_r.y >= h) b_r.y = h-1;
+
+							if (layers[fow->dark_layer_id][t_l.x][t_l.y] == FogOfWar::TILE_HIDDEN) {
+								if (layers[fow->dark_layer_id][t_r.x][t_r.y] == FogOfWar::TILE_HIDDEN) {
+									if (layers[fow->dark_layer_id][b_l.x][b_l.y] == FogOfWar::TILE_HIDDEN) {
+										if (layers[fow->dark_layer_id][b_r.x][b_r.y] == FogOfWar::TILE_HIDDEN) {
+											skip_tile_render = true;
+										}
+									}
+								}
+							}
+						}
+					}
+				}
+
 				checkHiddenEntities(i, j, layers[index_objectlayer], r);
-				render_device->render(tile.tile);
+				if (!skip_tile_render) {
+					if (fogofwar == FogOfWar::TYPE_TINT) {
+						tile.tile->color_mod = fow->getTileColorMod(i, j);
+					}
+					render_device->render(tile.tile);
+				}
 			}
 			p.x += eset->tileset.tile_w;
 
@@ -764,7 +1015,7 @@ void MapRenderer::renderOrthoFrontObjects(std::vector<Renderable> &r) {
 void MapRenderer::renderOrtho(std::vector<Renderable> &r, std::vector<Renderable> &r_dead) {
 	unsigned index = 0;
 	while (index < index_objectlayer) {
-		renderOrthoLayer(layers[index]);
+		renderOrthoLayer(layers[index], tset);
 		map_parallax.render(cam.shake, layernames[index]);
 		index++;
 	}
@@ -775,7 +1026,20 @@ void MapRenderer::renderOrtho(std::vector<Renderable> &r, std::vector<Renderable
 
 	index++;
 	while (index < layers.size()) {
-		renderOrthoLayer(layers[index]);
+		if (fogofwar == FogOfWar::TYPE_OVERLAY) {
+			if (layernames[index] == "fow_dark") {
+				renderOrthoLayer(layers[index],fow->tset_dark);
+			}
+			else if (layernames[index] == "fow_fog") {
+				renderOrthoLayer(layers[index],fow->tset_fog);
+			}
+			else {
+				renderOrthoLayer(layers[index], tset);
+			}
+		}
+		else if (layernames[index] != "fow_dark" && layernames[index] != "fow_fog") {
+			renderOrthoLayer(layers[index], tset);
+		}
 		map_parallax.render(cam.shake, layernames[index]);
 		index++;
 	}
@@ -948,10 +1212,9 @@ void MapRenderer::checkHotspots() {
 					p = centerTile(p);
 
 					Rect dest;
-					dest.x = p.x - npc->z;
-					dest.y = p.y - npc->a;
-					dest.w = npc->b;
-					dest.h = npc->c;
+					if (npc->id < npcs->npcs.size()) {
+						dest = npcs->npcs[npc->id]->getRenderBounds(mapr->cam.pos);
+					}
 
 					if (Utils::isWithinRect(dest, inpt->mouse)) {
 						matched = true;
