@@ -30,6 +30,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EngineSettings.h"
 #include "FileParser.h"
 #include "FontEngine.h"
+#include "InputState.h"
 #include "MapRenderer.h"
 #include "Menu.h"
 #include "MenuActionBar.h"
@@ -62,16 +63,17 @@ MenuActionBar::MenuActionBar()
 	, slots_count(0)
 	, drag_prev_slot(-1)
 	, updated(false)
-	, twostep_slot(-1) {
+	, twostep_slot(-1)
+	, touch_slot(NULL) {
 
 	menu_labels.resize(MENU_COUNT);
 
 	tablist = TabList();
-	tablist.setScrollType(Widget::SCROLL_HORIZONTAL);
-	tablist.setInputs(Input::ACTIONBAR_BACK, Input::ACTIONBAR_FORWARD, Input::ACTIONBAR);
+	tablist.setScrollType(Widget::SCROLL_TWO_DIRECTIONS);
+	tablist.lock();
 
 	for (unsigned i=0; i<MENU_COUNT; i++) {
-		menus[i] = new WidgetSlot(-1, Input::ACTIONBAR);
+		menus[i] = new WidgetSlot(WidgetSlot::NO_ICON);
 		menus[i]->setHotkey(Input::CHARACTER + i);
 
 		// NOTE: This prevents these buttons from being clickable unless they get defined in the config file.
@@ -205,7 +207,7 @@ void MenuActionBar::addSlot(unsigned index, int x, int y, bool is_locked) {
 		slots.resize(index+1, NULL);
 	}
 
-	slots[index] = new WidgetSlot(-1, Input::ACTIONBAR);
+	slots[index] = new WidgetSlot(WidgetSlot::NO_ICON);
 	slots[index]->setBasePos(x, y, Utils::ALIGN_TOPLEFT);
 	slots[index]->pos.w = slots[index]->pos.h = eset->resolutions.icon_size;
 	slots[index]->continuous = true;
@@ -289,7 +291,8 @@ void MenuActionBar::clear(bool skip_items) {
 void MenuActionBar::loadGraphics() {
 	Image *graphics;
 
-	setBackground("images/menus/actionbar_trim.png");
+	if (!background)
+		setBackground("images/menus/actionbar_trim.png");
 
 	Rect icon_clip;
 	icon_clip.w = icon_clip.h = eset->resolutions.icon_size;
@@ -317,6 +320,24 @@ void MenuActionBar::loadGraphics() {
 
 void MenuActionBar::logic() {
 	tablist.logic();
+	if (!inpt->usingMouse() && inpt->pressing[Input::ACTIONBAR] && !inpt->lock[Input::ACTIONBAR]) {
+		inpt->lock[Input::ACTIONBAR] = true;
+		if (tablist.getCurrent() == -1) {
+			tablist.unlock();
+			if (menu->isDragging())
+				tablist.getNext(!TabList::GET_INNER, TabList::WIDGET_SELECT_AUTO);
+			else
+				tablist.setCurrent(menus[MENU_INVENTORY]);
+			menu->defocusLeft();
+			menu->defocusRight();
+		}
+		else {
+			tablist.defocus();
+		}
+	}
+	if (tablist.getCurrent() == -1) {
+		tablist.lock();
+	}
 
 	// hero has no powers
 	if (pc->power_cast_timers.empty())
@@ -334,7 +355,7 @@ void MenuActionBar::logic() {
 			else {
 				for (size_t j = 0; j < power.required_items.size(); ++j) {
 					if (power.required_items[j].equipped) {
-						if (!menu->inv->inventory[MenuInventory::EQUIPMENT].contain(power.required_items[j].id, 1))
+						if (!menu->inv->equipmentContain(power.required_items[j].id, 1))
 							setItemCount(i, 0, IS_EQUIPPED);
 						else
 							setItemCount(i, 1, IS_EQUIPPED);
@@ -534,11 +555,6 @@ void MenuActionBar::remove(const Point& mouse) {
  * add that power to the action queue
  */
 void MenuActionBar::checkAction(std::vector<ActionData> &action_queue) {
-	if (inpt->pressing[Input::ACTIONBAR_USE] && tablist.getCurrent() == -1 && slots_count > 10) {
-		tablist.setCurrent(slots[10]);
-		slots[10]->in_focus = true;
-	}
-
 	bool enable_mm_attack = (!settings->mouse_move || inpt->pressing[Input::SHIFT] || pc->lock_enemy);
 	bool enable_main1 = (!settings->touchscreen || (!menu->menus_open && menu->touch_controls->checkAllowMain1())) && (settings->mouse_move_swap || enable_mm_attack);
 	bool enable_main2 = !settings->mouse_move_swap || enable_mm_attack;
@@ -561,7 +577,8 @@ void MenuActionBar::checkAction(std::vector<ActionData> &action_queue) {
 		}
 
 		// mouse/touch click
-		else if (inpt->usingMouse() && slots[i]->checkClick() == WidgetSlot::ACTIVATED) {
+		else if ((inpt->mode == InputState::MODE_TOUCHSCREEN && touch_slot == slots[i]) || (inpt->mode != InputState::MODE_TOUCHSCREEN && inpt->usingMouse() && slots[i]->checkClick() == WidgetSlot::ACTIVATE)) {
+			touch_slot = NULL;
 			have_aim = false;
 			slot_activated[i] = true;
 			action.power = hotkeys_mod[i];
@@ -588,7 +605,7 @@ void MenuActionBar::checkAction(std::vector<ActionData> &action_queue) {
 		}
 
 		// joystick/keyboard action button
-		else if (inpt->pressing[Input::ACTIONBAR_USE] && static_cast<unsigned>(tablist.getCurrent()) == i) {
+		else if (!inpt->usingMouse() && slots[i]->checkClick() == WidgetSlot::ACTIVATE) {
 			have_aim = false;
 			slot_activated[i] = true;
 			action.power = hotkeys_mod[i];
@@ -665,9 +682,9 @@ void MenuActionBar::checkAction(std::vector<ActionData> &action_queue) {
 		else {
 			// if we're not triggering an action that is currently in the queue,
 			// remove it from the queue
-			for (size_t j=0; j<action_queue.size(); j++) {
-				if (action_queue[j].hotkey == i)
-					action_queue.erase(action_queue.begin()+j);
+			for (size_t j = action_queue.size(); j > 0; --j) {
+				if (!action_queue[j-1].activated_from_inventory && action_queue[j-1].hotkey == i)
+					action_queue.erase(action_queue.begin()+(j-1));
 			}
 		}
 	}
@@ -704,34 +721,22 @@ void MenuActionBar::checkMenu(bool &menu_c, bool &menu_i, bool &menu_p, bool &me
 	if (menus[MENU_CHARACTER]->checkClick()) {
 		menu_c = true;
 		menus[MENU_CHARACTER]->deactivate();
+		defocusTabLists();
 	}
 	else if (menus[MENU_INVENTORY]->checkClick()) {
 		menu_i = true;
 		menus[MENU_INVENTORY]->deactivate();
+		defocusTabLists();
 	}
 	else if (menus[MENU_POWERS]->checkClick()) {
 		menu_p = true;
 		menus[MENU_POWERS]->deactivate();
+		defocusTabLists();
 	}
 	else if (menus[MENU_LOG]->checkClick()) {
 		menu_l = true;
 		menus[MENU_LOG]->deactivate();
-	}
-
-	// also allow ACTIONBAR_USE to open menus
-	if (inpt->pressing[Input::ACTIONBAR_USE] && !inpt->lock[Input::ACTIONBAR_USE]) {
-		inpt->lock[Input::ACTIONBAR_USE] = true;
-
-		unsigned cur_slot = static_cast<unsigned>(tablist.getCurrent());
-
-		if (cur_slot == tablist.size() - MENU_COUNT)
-			menu_c = true;
-		else if (cur_slot == tablist.size() - (MENU_COUNT - 1))
-			menu_i = true;
-		else if (cur_slot == tablist.size() - (MENU_COUNT - 2))
-			menu_p = true;
-		else if (cur_slot == tablist.size() - (MENU_COUNT - 3))
-			menu_l = true;
+		defocusTabLists();
 	}
 }
 
@@ -750,12 +755,7 @@ void MenuActionBar::set(std::vector<PowerID> power_id, bool skip_empty) {
 }
 
 void MenuActionBar::resetSlots() {
-	for (unsigned i = 0; i < slots_count; i++) {
-		if (slots[i]) {
-			slots[i]->checked = false;
-			slots[i]->pressed = false;
-		}
-	}
+	// TODO remove
 }
 
 /**
@@ -889,6 +889,14 @@ PowerID MenuActionBar::getSlotPower(int slot) {
 	return 0;
 }
 
+WidgetSlot* MenuActionBar::getSlotFromPosition(const Point& position) {
+	for (size_t i = 0; i < slots.size(); ++i) {
+		if (slots[i] && Utils::isWithinRect(slots[i]->pos, position))
+			return slots[i];
+	}
+	return NULL;
+}
+
 MenuActionBar::~MenuActionBar() {
 
 	menu_act = NULL;
