@@ -81,8 +81,8 @@ MenuCharacter::MenuCharacter()
 		cstat[i+2].label->setText(eset->primary_stats.list[i].name);
 	}
 
-	show_stat.resize(Stats::COUNT + eset->damage_types.count);
-	for (size_t i = 0; i < Stats::COUNT + eset->damage_types.count; i++) {
+	show_stat.resize(Stats::COUNT + eset->damage_types.count + eset->elements.list.size());
+	for (size_t i = 0; i < Stats::COUNT + eset->damage_types.count + eset->elements.list.size(); i++) {
 		show_stat[i] = true;
 	}
 
@@ -187,7 +187,7 @@ MenuCharacter::MenuCharacter()
 			// @ATTR show_resists|bool|Hide the elemental "Resistance" stats in the statlist if set to false.
 			else if (infile.key == "show_resists") show_resists = Parse::toBool(infile.val);
 
-			// @ATTR show_stat|predefined_string, bool : Stat name / Damage type, Visible|Hide the matching stat or damage type min/max in the statlist if set to false.
+			// @ATTR show_stat|stat_id, bool : Stat ID, Visible|Hide the matching stat ID in the statlist if set to false.
 			else if (infile.key == "show_stat") {
 				std::string stat_name = Parse::popFirstString(infile.val);
 
@@ -205,6 +205,11 @@ MenuCharacter::MenuCharacter()
 						show_stat[Stats::COUNT + (i*2) + 1] = Parse::toBool(Parse::popFirstString(infile.val));
 					}
 				}
+				for (size_t i = 0; i < eset->elements.list.size(); ++i) {
+					if (stat_name == eset->elements.list[i].resist_id) {
+						show_stat[Stats::COUNT + eset->damage_types.count + i] = Parse::toBool(Parse::popFirstString(infile.val));
+					}
+				}
 			}
 
 			// @ATTR name_max_width|int|The maxiumum width, in pixels, that the character name can occupy until it is abbreviated.
@@ -228,7 +233,8 @@ MenuCharacter::MenuCharacter()
 	// frequently refreshing trimmed text is slow for Cyrillic characters, so disable it here
 	statList->disable_text_trim = true;
 
-	setBackground("images/menus/character.png");
+	if (!background)
+		setBackground("images/menus/character.png");
 
 	align();
 
@@ -347,10 +353,14 @@ void MenuCharacter::refreshStats() {
 	}
 
 	if (show_resists) {
-		for (unsigned int j=0; j<pc->stats.vulnerable.size(); ++j) {
+		for (size_t i=0; i<eset->elements.list.size(); ++i) {
+			if (!show_stat[Stats::COUNT + eset->damage_types.count + i])
+				continue;
+
 			ss.str("");
-			ss << msg->get("Resistance (%s)", eset->elements.list[j].name) << ": " << (100 - pc->stats.vulnerable[j]) << "%";
-			statList->set(j+stat_index, ss.str(), msg->get("Reduces the damage taken from \"%s\" elemental attacks.", eset->elements.list[j].name));
+			ss << msg->get("Resistance (%s)", eset->elements.list[i].name) << ": " << (pc->stats.getResist(i)) << "%";
+			statList->set(stat_index, ss.str(), resistTooltip(i));
+			stat_index++;
 		}
 	}
 
@@ -373,22 +383,28 @@ void MenuCharacter::refreshStats() {
 		for (int i = 0; i < Stats::COUNT; ++i) {
 			// damage types are displayed before absorb
 			if (i == Stats::ABS_MIN) {
-				for (size_t k = 0; k < eset->damage_types.count; ++k) {
-					if (base_bonus[j-2]->at(Stats::COUNT + k) > 0) {
+				for (size_t k = 0; k < eset->damage_types.list.size(); ++k) {
+					// damage min
+					if (base_bonus[j-2]->at(Stats::COUNT + (k*2)) > 0 && show_stat[Stats::COUNT + (k*2)]) {
 						if (!have_bonus) {
 							cstat[j].tip.addText("\n" + msg->get("Related stats:"));
 							have_bonus = true;
 						}
-						if (k % 2 == 0)
-							cstat[j].tip.addText(eset->damage_types.list[k / 2].name_min);
-						else
-							cstat[j].tip.addText(eset->damage_types.list[k / 2].name_max);
+						cstat[j].tip.addText(eset->damage_types.list[k].name_min);
+					}
+					// damage max
+					if (base_bonus[j-2]->at(Stats::COUNT + (k*2) + 1) > 0 && show_stat[Stats::COUNT + (k*2) + 1]) {
+						if (!have_bonus) {
+							cstat[j].tip.addText("\n" + msg->get("Related stats:"));
+							have_bonus = true;
+						}
+						cstat[j].tip.addText(eset->damage_types.list[k].name_max);
 					}
 				}
 			}
 
 			// non-damage bonuses
-			if (base_bonus[j-2]->at(i) > 0) {
+			if (base_bonus[j-2]->at(i) > 0 && show_stat[i]) {
 				if (!have_bonus) {
 					cstat[j].tip.addText("\n" + msg->get("Related stats:"));
 					have_bonus = true;
@@ -396,6 +412,18 @@ void MenuCharacter::refreshStats() {
 				cstat[j].tip.addText(Stats::NAME[i]);
 			}
 		}
+
+		for (size_t i = 0; i < eset->elements.list.size(); ++i) {
+			size_t resist_index = Stats::COUNT + eset->damage_types.count + i;
+
+			if (base_bonus[j-2]->at(resist_index) > 0 && show_stat[resist_index]) {
+				if (!have_bonus) {
+					cstat[j].tip.addText("\n" + msg->get("Related stats:"));
+					have_bonus = true;
+				}
+				cstat[j].tip.addText(msg->get("Resistance (%s)", eset->elements.list[i].name));
+			}
+		}
 	}
 }
 
@@ -459,6 +487,29 @@ std::string MenuCharacter::damageTooltip(size_t dmg_type) {
 	return full_tooltip;
 }
 
+/**
+ * Create tooltip text showing the per_* values of a resistance stat
+ */
+std::string MenuCharacter::resistTooltip(size_t resist_type) {
+	std::string tooltip_text;
+	size_t resist_index = Stats::COUNT + eset->damage_types.count + resist_type;
+
+	if (pc->stats.per_level[resist_index] > 0)
+		tooltip_text += msg->get("Each level grants %d.", pc->stats.per_level[resist_index]) + ' ';
+
+	for (size_t i = 0; i < eset->primary_stats.list.size(); ++i) {
+		if (pc->stats.per_primary[i][resist_index] > 0)
+			tooltip_text += msg->get("Each point of %s grants %d.", eset->primary_stats.list[i].name, pc->stats.per_primary[i][resist_index]) + ' ';
+	}
+
+	std::string full_tooltip = msg->get("Reduces the damage taken from \"%s\" elemental attacks.", eset->elements.list[resist_type].name);
+	if (tooltip_text != "") {
+		full_tooltip += "\n" + tooltip_text;
+	}
+
+	return full_tooltip;
+}
+
 void MenuCharacter::logic() {
 	if (!visible) return;
 
