@@ -83,7 +83,7 @@ void MenuConfig::ConfigTab::setOptionWidgets(int index, WidgetLabel* lb, Widget*
 	options[index].label = lb;
 	options[index].label->setText(lb_text);
 	options[index].widget = w;
-	options[index].widget->tablist_nav_right = true;
+	options[index].widget->tablist_nav_align = TabList::NAV_ALIGN_RIGHT;
 }
 
 void MenuConfig::ConfigTab::setOptionEnabled(int index, bool enable) {
@@ -119,8 +119,8 @@ MenuConfig::MenuConfig (bool _is_game_state)
 	, defaults_button(new WidgetButton(WidgetButton::DEFAULT_FILE))
 	, cancel_button(new WidgetButton(WidgetButton::DEFAULT_FILE))
 	, background(NULL)
-	, input_confirm(new MenuConfirm(msg->get("Clear"),msg->get("Assign:")))
-	, defaults_confirm(new MenuConfirm(msg->get("Defaults"), msg->get("Reset ALL settings?")))
+	, input_confirm(new MenuConfirm())
+	, defaults_confirm(new MenuConfirm())
 
 	, pause_continue_lb(new WidgetLabel())
 	, pause_continue_btn(new WidgetButton(WidgetButton::DEFAULT_FILE))
@@ -229,8 +229,7 @@ MenuConfig::MenuConfig (bool _is_game_state)
 	, scrollpane_separator_color(font->getColor(FontEngine::COLOR_WIDGET_DISABLED))
 	, new_render_device(settings->render_device_name)
 	, input_confirm_timer(settings->max_frames_per_sec * 10) // 10 seconds
-	, input_key(0)
-	, key_count(0)
+	, input_action(0)
 	, keybind_tip_timer(settings->max_frames_per_sec * 5) // 5 seconds
 	, keybind_tip(new WidgetTooltip())
 	, clicked_accept(false)
@@ -241,6 +240,13 @@ MenuConfig::MenuConfig (bool _is_game_state)
 	, clicked_pause_exit(false)
 	, clicked_pause_save(false)
 {
+	input_confirm->setTitle(msg->get("Assign:"));
+	input_confirm->action_list->append(msg->get("New"), "");
+	input_confirm->action_list->append(msg->get("Clear"), "");
+
+	defaults_confirm->setTitle(msg->get("Reset ALL settings?"));
+	defaults_confirm->action_list->append(msg->get("No"), "");
+	defaults_confirm->action_list->append(msg->get("Yes"), "");
 
 	Image *graphics;
 	graphics = render_device->loadImage("images/menus/config.png", RenderDevice::ERROR_NORMAL);
@@ -287,16 +293,12 @@ MenuConfig::MenuConfig (bool _is_game_state)
 	refreshJoysticks();
 
 	// Allocate KeyBindings
-	for (int i = 0; i < inpt->KEY_COUNT_USER * 3; i++) {
+	for (int i = 0; i < inpt->KEY_COUNT_USER; i++) {
 		keybinds_lb.push_back(new WidgetLabel());
-		keybinds_btn.push_back(new WidgetButton(WidgetButton::DEFAULT_FILE));
+		keybinds_lstb.push_back(new WidgetHorizontalList());
+		keybinds_lstb.back()->has_action = true;
 	}
 
-	key_count = static_cast<unsigned>(keybinds_btn.size()/3);
-
-	// don't allow remapping the primary Main1 binding
-	keybinds_btn[Input::MAIN1 * 3]->enabled = false;
-
 	// set up loot tooltip setting
 	loot_tooltip_lstb->append(msg->get("Default"), msg->get("Show all loot tooltips, except for those that would be obscured by the player or an enemy. Temporarily show all loot tooltips with 'Alt'."));
 	loot_tooltip_lstb->append(msg->get("Show all"), msg->get("Always show loot tooltips. Temporarily hide all loot tooltips with 'Alt'."));
@@ -361,6 +363,9 @@ MenuConfig::MenuConfig (bool _is_game_state)
 		max_render_size_lstb->append(ss.str(), max_render_size_tooltip);
 	}
 
+	// reset this flag, as it may have been set when plugging in gamepads outside the config menu
+	inpt->joysticks_changed = false;
+
 	init();
 
 	render_device->setBackgroundColor(Color(0,0,0,0));
@@ -371,13 +376,13 @@ MenuConfig::~MenuConfig() {
 }
 
 void MenuConfig::init() {
-	tab_control->setTabTitle(EXIT_TAB, msg->get("Exit"));
-	tab_control->setTabTitle(VIDEO_TAB, msg->get("Video"));
-	tab_control->setTabTitle(AUDIO_TAB, msg->get("Audio"));
-	tab_control->setTabTitle(INTERFACE_TAB, msg->get("Interface"));
-	tab_control->setTabTitle(INPUT_TAB, msg->get("Input"));
-	tab_control->setTabTitle(KEYBINDS_TAB, msg->get("Keybindings"));
-	tab_control->setTabTitle(MODS_TAB, msg->get("Mods"));
+	tab_control->setupTab(EXIT_TAB, msg->get("Exit"), &tablist_exit);
+	tab_control->setupTab(VIDEO_TAB, msg->get("Video"), &tablist_video);
+	tab_control->setupTab(AUDIO_TAB, msg->get("Audio"), &tablist_audio);
+	tab_control->setupTab(INTERFACE_TAB, msg->get("Interface"), &tablist_interface);
+	tab_control->setupTab(INPUT_TAB, msg->get("Input"), &tablist_input);
+	tab_control->setupTab(KEYBINDS_TAB, msg->get("Keybindings"), &tablist_keybinds);
+	tab_control->setupTab(MODS_TAB, msg->get("Mods"), &tablist_mods);
 
 	readConfig();
 
@@ -387,7 +392,7 @@ void MenuConfig::init() {
 	cfg_tabs[AUDIO_TAB].options.resize(Platform::Audio::COUNT);
 	cfg_tabs[INTERFACE_TAB].options.resize(Platform::Interface::COUNT);
 	cfg_tabs[INPUT_TAB].options.resize(Platform::Input::COUNT);
-	cfg_tabs[KEYBINDS_TAB].options.resize(inpt->KEY_COUNT_USER * 3);
+	cfg_tabs[KEYBINDS_TAB].options.resize(inpt->KEY_COUNT_USER);
 
 	cfg_tabs[EXIT_TAB].setOptionWidgets(EXIT_OPTION_CONTINUE, pause_continue_lb, pause_continue_btn, msg->get("Paused"));
 	cfg_tabs[EXIT_TAB].setOptionWidgets(EXIT_OPTION_SAVE, pause_save_lb, pause_save_btn, "");
@@ -442,17 +447,8 @@ void MenuConfig::init() {
 	cfg_tabs[INPUT_TAB].setOptionWidgets(Platform::Input::TOUCH_SCALE, touch_scale_lb, touch_scale_sl, msg->get("Touch Gamepad Scaling"));
 
 
-	for (size_t i = 0; i < keybinds_btn.size(); ++i) {
-		if (i % 3 == 0) {
-			cfg_tabs[KEYBINDS_TAB].setOptionWidgets(static_cast<int>(i), keybinds_lb[i], keybinds_btn[i], inpt->binding_name[i/3]);
-			// TODO since these are blank, don't allocate?
-			cfg_tabs[KEYBINDS_TAB].setOptionWidgets(static_cast<int>(i+1), keybinds_lb[i+1], keybinds_btn[i+1], "");
-			cfg_tabs[KEYBINDS_TAB].setOptionWidgets(static_cast<int>(i+2), keybinds_lb[i+2], keybinds_btn[i+2], "");
-
-			keybinds_btn[i]->tooltip = msg->get("Primary binding: %s", inpt->binding_name[i/3].c_str());
-			keybinds_btn[i+1]->tooltip = msg->get("Alternate binding: %s", inpt->binding_name[i/3].c_str());
-			keybinds_btn[i+2]->tooltip = msg->get("Joystick binding: %s", inpt->binding_name[i/3].c_str());
-		}
+	for (size_t i = 0; i < keybinds_lstb.size(); ++i) {
+		cfg_tabs[KEYBINDS_TAB].setOptionWidgets(static_cast<int>(i), keybinds_lb[i], keybinds_lstb[i], inpt->binding_name[i]);
 	}
 
 	// disable some options
@@ -497,7 +493,7 @@ void MenuConfig::init() {
 			cfg_tabs[INPUT_TAB].setOptionEnabled(i, false);
 	}
 	if (!platform.config_misc[Platform::Misc::KEYBINDS]) {
-		for (size_t i = 0; i < keybinds_btn.size(); ++i) {
+		for (size_t i = 0; i < keybinds_lstb.size(); ++i) {
 			cfg_tabs[KEYBINDS_TAB].setOptionEnabled(static_cast<int>(i), false);
 		}
 	}
@@ -712,52 +708,35 @@ void MenuConfig::addChildWidgets() {
 }
 
 void MenuConfig::setupTabList() {
-	tablist.add(tab_control);
-	tablist.setPrevTabList(&tablist_main);
-
 	if (enable_gamestate_buttons) {
 		tablist_main.add(ok_button);
 		tablist_main.add(defaults_button);
 		tablist_main.add(cancel_button);
 	}
-	tablist_main.setPrevTabList(&tablist);
-	tablist_main.setNextTabList(&tablist);
 	tablist_main.lock();
 
 	tablist_exit.setScrollType(Widget::SCROLL_VERTICAL);
 	tablist_exit.add(cfg_tabs[EXIT_TAB].scrollbox);
-	tablist_exit.setPrevTabList(&tablist);
-	tablist_exit.setNextTabList(&tablist_main);
 	tablist_exit.lock();
 
 	tablist_video.setScrollType(Widget::SCROLL_VERTICAL);
 	tablist_video.add(cfg_tabs[VIDEO_TAB].scrollbox);
-	tablist_video.setPrevTabList(&tablist);
-	tablist_video.setNextTabList(&tablist_main);
 	tablist_video.lock();
 
 	tablist_audio.setScrollType(Widget::SCROLL_VERTICAL);
 	tablist_audio.add(cfg_tabs[AUDIO_TAB].scrollbox);
-	tablist_audio.setPrevTabList(&tablist);
-	tablist_audio.setNextTabList(&tablist_main);
 	tablist_audio.lock();
 
 	tablist_interface.setScrollType(Widget::SCROLL_VERTICAL);
 	tablist_interface.add(cfg_tabs[INTERFACE_TAB].scrollbox);
-	tablist_interface.setPrevTabList(&tablist);
-	tablist_interface.setNextTabList(&tablist_main);
 	tablist_interface.lock();
 
 	tablist_input.setScrollType(Widget::SCROLL_VERTICAL);
 	tablist_input.add(cfg_tabs[INPUT_TAB].scrollbox);
-	tablist_input.setPrevTabList(&tablist);
-	tablist_input.setNextTabList(&tablist_main);
 	tablist_input.lock();
 
 	tablist_keybinds.setScrollType(Widget::SCROLL_VERTICAL);
 	tablist_keybinds.add(cfg_tabs[KEYBINDS_TAB].scrollbox);
-	tablist_keybinds.setPrevTabList(&tablist);
-	tablist_keybinds.setNextTabList(&tablist_main);
 	tablist_keybinds.lock();
 
 	tablist_mods.add(inactivemods_lstb);
@@ -766,9 +745,18 @@ void MenuConfig::setupTabList() {
 	tablist_mods.add(activemods_deactivate_btn);
 	tablist_mods.add(activemods_shiftup_btn);
 	tablist_mods.add(activemods_shiftdown_btn);
-	tablist_mods.setPrevTabList(&tablist);
-	tablist_mods.setNextTabList(&tablist_main);
 	tablist_mods.lock();
+
+	tablists.clear();
+	tablists.push_back(&tablist);
+	tablists.push_back(&tablist_main);
+	tablists.push_back(&tablist_exit);
+	tablists.push_back(&tablist_video);
+	tablists.push_back(&tablist_audio);
+	tablists.push_back(&tablist_interface);
+	tablists.push_back(&tablist_input);
+	tablists.push_back(&tablist_keybinds);
+	tablists.push_back(&tablist_mods);
 }
 
 void MenuConfig::update() {
@@ -875,24 +863,36 @@ void MenuConfig::updateInput() {
 		joystick_device_lstb->select(settings->joystick_device + 1);
 	}
 
-	joystick_deadzone_sl->set(0, 32768, settings->joy_deadzone);
+	joystick_deadzone_sl->set(Settings::JOY_DEADZONE_MIN, Settings::JOY_DEADZONE_MAX, settings->joy_deadzone);
 	touch_scale_sl->set(TOUCH_SCALE_MIN, TOUCH_SCALE_MAX, static_cast<int>(settings->touch_scale * 100.0));
 
 	cfg_tabs[INPUT_TAB].scrollbox->refresh();
 }
 
 void MenuConfig::updateKeybinds() {
-	// now do labels for keybinds that are set
-	for (unsigned int i = 0; i < keybinds_btn.size(); i++) {
-		if (i % 3 == 0) {
-			keybinds_btn[i]->setLabel(inpt->getBindingString(i/3));
-			keybinds_btn[i+1]->setLabel(inpt->getBindingString(i/3, InputState::BINDING_ALT));
-			keybinds_btn[i+2]->setLabel(inpt->getBindingString(i/3, InputState::BINDING_JOYSTICK));
+	std::stringstream ss;
 
-			keybinds_btn[i]->refresh();
-			keybinds_btn[i+1]->refresh();
-			keybinds_btn[i+2]->refresh();
+	// now do labels for keybinds that are set
+	for (unsigned int i = 0; i < keybinds_lstb.size(); i++) {
+		keybinds_lstb[i]->clear();
+		if (inpt->binding[i].empty()) {
+			keybinds_lstb[i]->append(inpt->getBindingStringByIndex(static_cast<int>(i), -1), "");
 		}
+		else {
+			ss.str("");
+			ss << msg->get("Bindings for:") << " " << inpt->binding_name[i] << "\n";
+
+			for (size_t j = 0; j < inpt->binding[i].size(); ++j) {
+				ss << inpt->getBindingStringByIndex(static_cast<int>(i), static_cast<int>(j));
+				if (j+1 != inpt->binding[i].size()) {
+					ss << "\n";
+				}
+			}
+			for (size_t j = 0; j < inpt->binding[i].size(); ++j) {
+				keybinds_lstb[i]->append(inpt->getBindingStringByIndex(static_cast<int>(i), static_cast<int>(j)), ss.str());
+			}
+		}
+		keybinds_lstb[i]->refresh();
 	}
 	cfg_tabs[KEYBINDS_TAB].scrollbox->refresh();
 }
@@ -914,10 +914,14 @@ void MenuConfig::logic() {
 	else if (input_confirm->visible) {
 		// assign a keybind
 		input_confirm->logic();
-		scanKey(input_key);
-		input_confirm_timer.tick();
+		scanKey(input_action);
+
+		if (!input_confirm->action_list->enabled)
+			input_confirm_timer.tick();
+
 		if (input_confirm_timer.isEnd())
 			input_confirm->visible = false;
+
 		return;
 	}
 	else {
@@ -933,18 +937,22 @@ void MenuConfig::logic() {
 			pause_time_text->setText(Utils::getTimeString(hero->time_played));
 		}
 		tablist.setNextTabList(&tablist_exit);
+		tablist_main.setPrevTabList(&tablist_exit);
 		logicExit();
 	}
 	if (active_tab == VIDEO_TAB) {
 		tablist.setNextTabList(&tablist_video);
+		tablist_main.setPrevTabList(&tablist_video);
 		logicVideo();
 	}
 	else if (active_tab == AUDIO_TAB) {
 		tablist.setNextTabList(&tablist_audio);
+		tablist_main.setPrevTabList(&tablist_audio);
 		logicAudio();
 	}
 	else if (active_tab == INTERFACE_TAB) {
 		tablist.setNextTabList(&tablist_interface);
+		tablist_main.setPrevTabList(&tablist_interface);
 		logicInterface();
 
 		// TODO remove this?
@@ -956,14 +964,17 @@ void MenuConfig::logic() {
 	}
 	else if (active_tab == INPUT_TAB) {
 		tablist.setNextTabList(&tablist_input);
+		tablist_main.setPrevTabList(&tablist_input);
 		logicInput();
 	}
 	else if (active_tab == KEYBINDS_TAB) {
 		tablist.setNextTabList(&tablist_keybinds);
+		tablist_main.setPrevTabList(&tablist_keybinds);
 		logicKeybinds();
 	}
 	else if (active_tab == MODS_TAB) {
 		tablist.setNextTabList(&tablist_mods);
+		tablist_main.setPrevTabList(&tablist_mods);
 		logicMods();
 	}
 }
@@ -978,15 +989,13 @@ bool MenuConfig::logicMain() {
 
 	// tabs & the bottom 3 main buttons
 	tab_control->logic();
-	tablist.logic();
-	tablist_main.logic();
-	tablist_exit.logic();
-	tablist_video.logic();
-	tablist_audio.logic();
-	tablist_interface.logic();
-	tablist_input.logic();
-	tablist_keybinds.logic();
-	tablist_mods.logic();
+
+	for (size_t i = 0; i < tablists.size(); ++i) {
+		tablists[i]->logic();
+		if (!inpt->usingMouse() && !tablists[i]->isLocked() && tablists[i]->getCurrent() == -1 && tablist_main.getCurrent() == -1) {
+			tablists[i]->getNext(!TabList::GET_INNER, TabList::WIDGET_SELECT_AUTO);
+		}
+	}
 
 	if (enable_gamestate_buttons) {
 		// Ok/Cancel Buttons
@@ -997,15 +1006,39 @@ bool MenuConfig::logicMain() {
 			return false;
 		}
 		else if (defaults_button->checkClick()) {
-			defaults_confirm->visible = true;
+			defaults_confirm->show();
 			return true;
 		}
-		else if (cancel_button->checkClick() || (inpt->pressing[Input::CANCEL] && !inpt->lock[Input::CANCEL])) {
+		else if (cancel_button->checkClick() || (inpt->usingMouse() && inpt->pressing[Input::CANCEL] && !inpt->lock[Input::CANCEL])) {
+			if (inpt->pressing[Input::CANCEL])
+				inpt->lock[Input::CANCEL] = true;
+
 			clicked_cancel = true;
 
 			// MenuConfig deconstructed, proceed with caution
 			return false;
 		}
+		else if (!inpt->usingMouse() && inpt->pressing[Input::CANCEL] && !inpt->lock[Input::CANCEL]) {
+			inpt->lock[Input::CANCEL] = true;
+
+			// toggle between GameState buttons and tab widgets
+			if (tablist_main.getCurrent() == -1) {
+				// switch to GameState buttons
+				for (size_t i = 0; i < tablists.size(); ++i) {
+					tablists[i]->defocus();
+				}
+				tablist.lock();
+				tablist_main.unlock();
+				tablist_main.getNext(!TabList::GET_INNER, TabList::WIDGET_SELECT_AUTO);
+			}
+			else {
+				// switch to tab widgets
+				tablist_main.defocus();
+				tablist_main.lock();
+				tablist.unlock();
+				tablist.getNext(!TabList::GET_INNER, TabList::WIDGET_SELECT_AUTO);
+			}
+		}
 	}
 
 	return true;
@@ -1013,17 +1046,19 @@ bool MenuConfig::logicMain() {
 
 void MenuConfig::logicDefaults() {
 	defaults_confirm->logic();
-	if (defaults_confirm->confirmClicked) {
-		settings->fullscreen = false;
-		render_device->setFullscreen(settings->fullscreen);
-		settings->loadDefaults();
-		eset->load();
-		inpt->defaultQwertyKeyBindings();
-		inpt->defaultJoystickBindings();
-		update();
-		refreshWindowSize();
+	if (defaults_confirm->clicked_confirm) {
+		if (defaults_confirm->action_list->getSelected() == DEFAULTS_CONFIRM_OPTION_YES) {
+			settings->fullscreen = false;
+			settings->loadDefaults();
+			render_device->setFullscreen(settings->fullscreen);
+			eset->load();
+			inpt->initBindings();
+			update();
+			refreshWindowSize();
+		}
+		// both yes and no close the window
 		defaults_confirm->visible = false;
-		defaults_confirm->confirmClicked = false;
+		defaults_confirm->clicked_confirm = false;
 	}
 }
 
@@ -1180,8 +1215,15 @@ void MenuConfig::logicInput() {
 	Point mouse = cfg_tabs[INPUT_TAB].scrollbox->input_assist(inpt->mouse);
 
 	if (inpt->joysticks_changed) {
-		disableJoystickOptions();
 		refreshJoysticks();
+		if (settings->enable_joystick && inpt->getNumJoysticks() > 0) {
+			inpt->initJoystick();
+			joystick_device_lstb->select(settings->joystick_device + 1);
+		}
+		else {
+			joystick_device_lstb->select(0);
+		}
+
 		inpt->joysticks_changed = false;
 	}
 
@@ -1217,15 +1259,10 @@ void MenuConfig::logicInput() {
 	}
 	else if (cfg_tabs[INPUT_TAB].options[Platform::Input::JOYSTICK].enabled && joystick_device_lstb->checkClickAt(mouse.x, mouse.y)) {
 		settings->joystick_device = static_cast<int>(joystick_device_lstb->getSelected()) - 1;
-		if (settings->joystick_device != -1) {
-			settings->enable_joystick = true;
-			if (inpt->getNumJoysticks() > 0) {
-				inpt->initJoystick();
-			}
-		}
-		else {
-			settings->enable_joystick = false;
-		}
+		settings->enable_joystick = (settings->joystick_device != -1);
+		inpt->joysticks_changed = true;
+		inpt->initJoystick();
+		inpt->joysticks_changed = false;
 	}
 	else if (cfg_tabs[INPUT_TAB].options[Platform::Input::TOUCH_CONTROLS].enabled && touch_controls_cb->checkClickAt(mouse.x, mouse.y)) {
 		settings->touchscreen = touch_controls_cb->isChecked();
@@ -1239,26 +1276,20 @@ void MenuConfig::logicKeybinds() {
 	cfg_tabs[KEYBINDS_TAB].scrollbox->logic();
 	Point mouse = cfg_tabs[KEYBINDS_TAB].scrollbox->input_assist(inpt->mouse);
 
-	for (unsigned int i = 0; i < keybinds_btn.size(); i++) {
-		if ((i+1) % 3 == 0) {
-			keybinds_btn[i]->enabled = settings->enable_joystick;
-			keybinds_btn[i]->refresh();
-		}
-
+	for (unsigned int i = 0; i < keybinds_lstb.size(); i++) {
 		if (!cfg_tabs[KEYBINDS_TAB].options[i].enabled)
 			continue;
 
-		if (keybinds_btn[i]->checkClickAt(mouse.x,mouse.y)) {
-			std::string confirm_msg;
-			confirm_msg = msg->get("Assign:") + ' ' + inpt->binding_name[i/3];
-			delete input_confirm;
-			input_confirm = new MenuConfirm(msg->get("Clear"),confirm_msg);
-			input_confirm_timer.reset(Timer::BEGIN);
-			input_confirm->visible = true;
-			input_key = i;
-			inpt->last_button = -1;
-			inpt->last_key = -1;
-			inpt->last_joybutton = -1;
+		if (keybinds_lstb[i]->checkClickAt(mouse.x,mouse.y)) {
+			if (keybinds_lstb[i]->checkAction()) {
+				input_confirm->setTitle(msg->get("Assign:") + ' ' + inpt->binding_name[i]);
+				input_confirm_timer.reset(Timer::BEGIN);
+				input_confirm->show();
+				input_action = i;
+				inpt->last_button = -1;
+				inpt->last_key = -1;
+				inpt->last_joybutton = -1;
+			}
 		}
 	}
 }
@@ -1327,10 +1358,6 @@ void MenuConfig::renderTabContents() {
 			Image* render_target = cfg_tabs[active_tab].scrollbox->contents->getGraphics();
 
 			for (int i = 1; i < cfg_tabs[active_tab].enabled_count; ++i) {
-				if (active_tab == KEYBINDS_TAB) {
-					if (i % 3 != 0)
-						continue;
-				}
 				render_target->drawLine(scrollpane_padding.x, i * scrollpane_padding.y, scrollpane.w - scrollpane_padding.x - 1, i * scrollpane_padding.y, scrollpane_separator_color);
 			}
 		}
@@ -1567,9 +1594,9 @@ std::string MenuConfig::createModTooltip(Mod *mod) {
 	return ret;
 }
 
-void MenuConfig::confirmKey(int button) {
-	inpt->pressing[button] = false;
-	inpt->lock[button] = false;
+void MenuConfig::confirmKey(int action) {
+	inpt->pressing[action] = false;
+	inpt->lock[action] = false;
 
 	input_confirm->visible = false;
 	input_confirm_timer.reset(Timer::END);
@@ -1580,39 +1607,54 @@ void MenuConfig::confirmKey(int button) {
 	updateKeybinds();
 }
 
-void MenuConfig::scanKey(int button) {
-	int column = button % 3;
-	int real_button = button / 3;
+void MenuConfig::scanKey(int action) {
+	if (!input_confirm->visible)
+		return;
 
 	// clear the keybind if the user clicks "Clear" in the dialog
-	if (input_confirm->visible && input_confirm->confirmClicked) {
-		inpt->setKeybind(-1, real_button, column, keybind_msg);
-		confirmKey(real_button);
-		return;
+	if (input_confirm->clicked_confirm) {
+		input_confirm->clicked_confirm = false;
+
+		if (input_confirm->action_list->getSelected() == INPUT_CONFIRM_OPTION_NEW) {
+			input_confirm->action_list->enabled = false;
+			input_confirm->align();
+
+			inpt->last_button = -1;
+			inpt->last_key = -1;
+			inpt->last_joybutton = -1;
+		}
+		else if (input_confirm->action_list->getSelected() == INPUT_CONFIRM_OPTION_CLEAR) {
+			unsigned selected_bind = keybinds_lstb[action]->getSelected();
+			if (action == Input::MAIN1 && selected_bind == 0) {
+				keybind_msg = msg->get("Can not remove this binding.");
+			}
+			else {
+				inpt->removeBind(action, keybinds_lstb[action]->getSelected());
+				confirmKey(action);
+			}
+		}
 	}
 
-	if (input_confirm->visible && !input_confirm->isWithinButtons) {
-		// keyboard & mouse
-		if (column == InputState::BINDING_DEFAULT || column == InputState::BINDING_ALT) {
-			if (inpt->last_button != -1) {
-				// mouse
-				inpt->setKeybind(inpt->last_button, real_button, column, keybind_msg);
-				confirmKey(real_button);
-			}
-			else if (inpt->last_key != -1) {
-				// keyboard
-				inpt->setKeybind(inpt->last_key, real_button, column, keybind_msg);
-				confirmKey(real_button);
-			}
+	if (!input_confirm->action_list->enabled) {
+		if (inpt->last_key != -1) {
+			// keyboard
+			inpt->setBind(action, InputBind::KEY, inpt->last_key, &keybind_msg);
+			confirmKey(action);
 		}
-		// joystick
-		else if (column == InputState::BINDING_JOYSTICK && inpt->last_joybutton != -1) {
-			inpt->setKeybind(inpt->last_joybutton, real_button, column, keybind_msg);
-			confirmKey(real_button);
+		else if (inpt->last_button != -1) {
+			// mouse
+			inpt->setBind(action, InputBind::MOUSE, inpt->last_button, &keybind_msg);
+			confirmKey(action);
 		}
-		else if (column == InputState::BINDING_JOYSTICK && inpt->last_joyaxis != -1) {
-			inpt->setKeybind(inpt->last_joyaxis, real_button, column, keybind_msg);
-			confirmKey(real_button);
+		else if (inpt->last_joybutton != -1) {
+			// gamepad button
+			inpt->setBind(action, InputBind::GAMEPAD, inpt->last_joybutton, &keybind_msg);
+			confirmKey(action);
+		}
+		else if (inpt->last_joyaxis != -1) {
+			// gamepad axis
+			inpt->setBind(action, InputBind::GAMEPAD_AXIS, inpt->last_joyaxis, &keybind_msg);
+			confirmKey(action);
 		}
 	}
 }
@@ -1633,15 +1675,6 @@ void MenuConfig::disableMouseOptions() {
 	no_mouse_cb->setChecked(settings->no_mouse);
 }
 
-void MenuConfig::disableJoystickOptions() {
-	settings->enable_joystick = false;
-
-	joystick_device_lstb->select(0);
-
-	if (inpt->getNumJoysticks() > 0)
-		joystick_device_lstb->refresh();
-}
-
 void MenuConfig::refreshRenderers() {
 	renderer_lstb->clear();
 
@@ -1691,15 +1724,9 @@ void MenuConfig::resetSelectedTab() {
 		cfg_tabs[i].scrollbox->scrollToTop();
 	}
 
-	tablist.defocus();
-	tablist_main.defocus();
-	tablist_exit.defocus();
-	tablist_video.defocus();
-	tablist_audio.defocus();
-	tablist_interface.defocus();
-	tablist_input.defocus();
-	tablist_keybinds.defocus();
-	tablist_mods.defocus();
+	for (size_t i = 0; i < tablists.size(); ++i) {
+		tablists[i]->defocus();
+	}
 
 	input_confirm->visible = false;
 	input_confirm_timer.reset(Timer::END);
