@@ -43,6 +43,7 @@ MenuEnemy::MenuEnemy()
 	, custom_text_pos(false)
 	, bar_fill_offset()
 	, bar_fill_size(-1, -1)
+	, bar_gfx("images/menus/enemy_bar_hp.png")
 	, enemy(NULL)
 {
 	// disappear after 10 seconds
@@ -56,8 +57,6 @@ MenuEnemy::MenuEnemy()
 			if (parseMenuKey(infile.key, infile.val))
 				continue;
 
-			infile.val = infile.val + ',';
-
 			// @ATTR bar_pos|rectangle|Position and dimensions of the health bar.
 			if(infile.key == "bar_pos") {
 				bar_pos = Parse::toRect(infile.val);
@@ -75,6 +74,10 @@ MenuEnemy::MenuEnemy()
 			else if (infile.key == "bar_fill_size") {
 				bar_fill_size = Parse::toPoint(infile.val);
 			}
+			// @ATTR bar_gfx|filename|Filename of the image to use for the "fill" of the bar.
+			else if (infile.key == "bar_gfx") {
+				bar_gfx = infile.val;
+			}
 			else {
 				infile.error("MenuEnemy: '%s' is not a valid key.", infile.key.c_str());
 			}
@@ -96,9 +99,10 @@ MenuEnemy::MenuEnemy()
 void MenuEnemy::loadGraphics() {
 	Image *graphics;
 
-	setBackground("images/menus/enemy_bar.png");
+	if (!background)
+		setBackground("images/menus/enemy_bar.png");
 
-	graphics = render_device->loadImage("images/menus/enemy_bar_hp.png", RenderDevice::ERROR_NORMAL);
+	graphics = render_device->loadImage(bar_gfx, RenderDevice::ERROR_NORMAL);
 	if (graphics) {
 		bar_hp = graphics->createSprite();
 		graphics->unref();
