@@ -35,6 +35,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "ItemManager.h"
 #include "Menu.h"
 #include "MenuActionBar.h"
+#include "MenuHUDLog.h"
 #include "MenuInventory.h"
 #include "MenuManager.h"
 #include "MenuPowers.h"
@@ -51,13 +52,19 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "WidgetSlot.h"
 
 MenuInventory::MenuInventory()
-	: MAX_EQUIPPED(4)
+	: closeButton(new WidgetButton("images/menus/buttons/button_x.png"))
+	, equipmentSetPrevious(NULL)
+	, equipmentSetNext(NULL)
+	, equipmentSetLabel(NULL)
+	, MAX_EQUIPPED(4)
 	, MAX_CARRIED(64)
 	, carried_cols(4)
 	, carried_rows(4)
 	, tap_to_activate_timer(settings->max_frames_per_sec / 3)
 	, activated_slot(-1)
 	, activated_item(0)
+	, active_equipment_set(0)
+	, max_equipment_set(0)
 	, currency(0)
 	, drag_prev_src(-1)
 	, changed_equipment(true)
@@ -66,9 +73,11 @@ MenuInventory::MenuInventory()
 {
 	visible = false;
 
-	setBackground("images/menus/inventory.png");
-
-	closeButton = new WidgetButton("images/menus/buttons/button_x.png");
+	// raw data for equipment swap buttons
+	std::map<unsigned, std::string> raw_set_button;
+	std::string raw_previous;
+	std::string raw_next;
+	std::string raw_label;
 
 	// Load config settings
 	FileParser infile;
@@ -83,17 +92,42 @@ MenuInventory::MenuInventory()
 				Point pos = Parse::toPoint(infile.val);
 				closeButton->setBasePos(pos.x, pos.y, Utils::ALIGN_TOPLEFT);
 			}
-			// @ATTR equipment_slot|repeatable(int, int, string) : X, Y, Slot Type|Position and item type of an equipment slot.
+			// @ATTR set_button|int, int, int, filename : ID, Widget X, Widget Y, Image file|Set number, position and image filename for an equipment swap set button.
+			else if(infile.key == "set_button") {
+				unsigned id = static_cast<unsigned>(Parse::popFirstInt(infile.val));
+				std::pair<unsigned, std::string> set_button(id, infile.val);
+				raw_set_button.insert(set_button);
+				raw_set_button[id] = infile.val;
+			}
+			// @ATTR set_previous|int, int, filename : Widget X, Widget Y, Image file|Position and image filename for an equipment swap set previous button.
+			else if(infile.key == "set_previous") {
+				raw_previous = infile.val;
+			}
+			// @ATTR set_next|int, int, filename : Widget X, Widget Y, Image file|Position and image filename for an equipment swap set next button.
+			else if(infile.key == "set_next") {
+				raw_next = infile.val;
+			}
+			// @ATTR label_equipment_set|label|Label showing the active equipment set.
+			else if(infile.key == "label_equipment_set") {
+				raw_label = infile.val;
+			}
+			// @ATTR equipment_slot|repeatable(int, int, string, int) : X, Y, Slot Type, Equipment set|Position, item type and equipment set number of an equipment slot. Equipment set number is "0" for shared items."
 			else if(infile.key == "equipment_slot") {
 				Rect area;
 				Point pos;
+				std::string slt_type;
+				int eq_set;
 
 				pos.x = area.x = Parse::popFirstInt(infile.val);
 				pos.y = area.y = Parse::popFirstInt(infile.val);
+				slt_type = Parse::popFirstString(infile.val);
+				eq_set = Parse::popFirstInt(infile.val);
 				area.w = area.h = eset->resolutions.icon_size;
+
 				equipped_area.push_back(area);
 				equipped_pos.push_back(pos);
-				slot_type.push_back(Parse::popFirstString(infile.val));
+				slot_type.push_back(slt_type);
+				equipment_set.push_back(eq_set);
 			}
 			// @ATTR carried_area|point|Position of the first normal inventory slot.
 			else if(infile.key == "carried_area") {
@@ -142,6 +176,59 @@ MenuInventory::MenuInventory()
 		tablist.add(inventory[CARRIED].slots[i]);
 	}
 
+	for (size_t i=0; i<equipment_set.size(); i++) {
+		if (equipment_set[i] > max_equipment_set) {
+			max_equipment_set = equipment_set[i];
+		}
+	}
+
+	// create equipment swap buttons
+	std::map<unsigned, std::string>::iterator it;
+	for (it = raw_set_button.begin(); it != raw_set_button.end(); ++it) {
+		int px = Parse::popFirstInt(it->second);
+		int py = Parse::popFirstInt(it->second);
+		std::string icon = Parse::popFirstString(it->second);
+		equipmentSetButton.push_back(new WidgetButton(icon));
+		equipmentSetButton.back()->setBasePos(px, py, Utils::ALIGN_TOPLEFT);
+		tablist.add(equipmentSetButton.back());
+	}
+	raw_set_button.clear();
+
+	if (!raw_previous.empty()) {
+		int px = Parse::popFirstInt(raw_previous);
+		int py = Parse::popFirstInt(raw_previous);
+		std::string icon = Parse::popFirstString(raw_previous);
+		equipmentSetPrevious = new WidgetButton(icon);
+		equipmentSetPrevious->setBasePos(px, py, Utils::ALIGN_TOPLEFT);
+		tablist.add(equipmentSetPrevious);
+	}
+
+	if (!raw_next.empty()) {
+		int px = Parse::popFirstInt(raw_next);
+		int py = Parse::popFirstInt(raw_next);
+		std::string icon = Parse::popFirstString(raw_next);
+		equipmentSetNext = new WidgetButton(icon);
+		equipmentSetNext->setBasePos(px, py, Utils::ALIGN_TOPLEFT);
+		tablist.add(equipmentSetNext);
+	}
+
+	if (!raw_label.empty()) {
+		equipmentSetLabel = new WidgetLabel;
+		equipmentSetLabel->setFromLabelInfo(Parse::popLabelInfo(raw_label));
+
+		std::stringstream label;
+		label << active_equipment_set << "/" << max_equipment_set;
+		equipmentSetLabel->setText(label.str());
+		equipmentSetLabel->setColor(font->getColor(FontEngine::COLOR_MENU_NORMAL));
+	}
+
+	if (max_equipment_set > 0) {
+		applyEquipmentSet(1);
+	}
+
+	if (!background)
+		setBackground("images/menus/inventory.png");
+
 	align();
 }
 
@@ -161,6 +248,16 @@ void MenuInventory::align() {
 
 	closeButton->setPos(window_area.x, window_area.y);
 
+	if (!equipmentSetButton.empty()) {
+		for (size_t i=0; i<equipmentSetButton.size(); i++) {
+			equipmentSetButton[i]->setPos(window_area.x, window_area.y);
+		}
+	}
+
+	if (equipmentSetPrevious) equipmentSetPrevious->setPos(window_area.x, window_area.y);
+	if (equipmentSetNext) equipmentSetNext->setPos(window_area.x, window_area.y);
+	if (equipmentSetLabel) equipmentSetLabel->setPos(window_area.x, window_area.y);
+
 	label_inventory.setPos(window_area.x, window_area.y);
 	label_currency.setPos(window_area.x, window_area.y);
 }
@@ -225,10 +322,10 @@ void MenuInventory::logic() {
 	// a copy of currency is kept in stats, to help with various situations
 	pc->stats.currency = currency = inventory[CARRIED].count(eset->misc.currency_id);
 
-	// check close button
 	if (visible) {
 		tablist.logic();
 
+		// check close button
 		if (closeButton->checkClick()) {
 			visible = false;
 			snd->play(sfx_close, snd->DEFAULT_CHANNEL, snd->NO_POS, !snd->LOOP);
@@ -237,6 +334,39 @@ void MenuInventory::logic() {
 		if (drag_prev_src == -1) {
 			clearHighlight();
 		}
+
+		//check equipment set buttons
+		if (!equipmentSetButton.empty()) {
+			for (size_t i=0; i<equipmentSetButton.size(); i++) {
+				if(equipmentSetButton[i]->checkClick()) {
+					applyEquipmentSet(static_cast<unsigned>(i)+1);
+					applyEquipment();
+				}
+			}
+		}
+
+		if (equipmentSetNext) {
+			if (equipmentSetNext->checkClick()) {
+				applyNextEquipmentSet();
+				applyEquipment();
+			}
+		}
+
+		if (equipmentSetPrevious) {
+			if (equipmentSetPrevious->checkClick()) {
+				applyPreviousEquipmentSet();
+				applyEquipment();
+			}
+		}
+	}
+
+	if (max_equipment_set > 0) {
+		if (inpt->pressing[Input::EQUIPMENT_SWAP] && !inpt->lock[Input::EQUIPMENT_SWAP]) {
+			inpt->lock[Input::EQUIPMENT_SWAP] = true;
+			applyNextEquipmentSet();
+			applyEquipment();
+			clearHighlight();
+		}
 	}
 
 	tap_to_activate_timer.tick();
@@ -251,6 +381,16 @@ void MenuInventory::render() {
 	// close button
 	closeButton->render();
 
+	// equipment set buttons
+	if (!equipmentSetButton.empty()) {
+		for (size_t i=0; i<equipmentSetButton.size(); i++) {
+			equipmentSetButton[i]->render();
+		}
+	}
+	if (equipmentSetPrevious) equipmentSetPrevious->render();
+	if (equipmentSetNext) equipmentSetNext->render();
+	if (equipmentSetLabel) equipmentSetLabel->render();
+
 	// text overlay
 	label_inventory.render();
 
@@ -268,7 +408,7 @@ int MenuInventory::areaOver(const Point& position) {
 		return CARRIED;
 	}
 	else {
-		for (unsigned int i=0; i<equipped_area.size(); i++) {
+		for (unsigned i=0; i<equipped_area.size(); i++) {
 			if (Utils::isWithinRect(equipped_area[i], position)) {
 				return EQUIPMENT;
 			}
@@ -301,12 +441,12 @@ void MenuInventory::renderTooltips(const Point& position) {
 			tip_data.addText(msg->get("Pick up item(s):") + " " + inpt->getBindingString(Input::MAIN1));
 			tip_data.addText(msg->get("Use or equip item:") + " " + inpt->getBindingString(Input::MAIN2) + "\n");
 			tip_data.addText(msg->get("%s modifiers", inpt->getBindingString(Input::MAIN1)));
-			tip_data.addText(msg->get("Select a quantity of item:") + " " + inpt->getBindingString(Input::SHIFT) + " / " + inpt->getBindingString(Input::SHIFT, InputState::BINDING_ALT));
+			tip_data.addText(msg->get("Select a quantity of item:") + " " + inpt->getBindingString(Input::SHIFT));
 
 			if (inv_ctrl == CTRL_STASH)
-				tip_data.addText(msg->get("Stash item stack:") + " " + inpt->getBindingString(Input::CTRL) + " / " + inpt->getBindingString(Input::CTRL, InputState::BINDING_ALT));
-			else if (inv_ctrl == CTRL_VENDOR || (eset->misc.sell_without_vendor && inv_ctrl != CTRL_STASH))
-				tip_data.addText(msg->get("Sell item stack:") + " " + inpt->getBindingString(Input::CTRL) + " / " + inpt->getBindingString(Input::CTRL, InputState::BINDING_ALT));
+				tip_data.addText(msg->get("Stash item stack:") + " " + inpt->getBindingString(Input::CTRL));
+			else if (inv_ctrl == CTRL_VENDOR || eset->misc.sell_without_vendor)
+				tip_data.addText(msg->get("Sell item stack:") + " " + inpt->getBindingString(Input::CTRL));
 		}
 		tooltipm->push(tip_data, position, TooltipData::STYLE_FLOAT);
 	}
@@ -319,8 +459,12 @@ void MenuInventory::renderTooltips(const Point& position) {
 
 	tip_data.clear();
 
+	if (area == EQUIPMENT)
+		if (!isActive(slot))
+			return;
+
 	if (inventory[area][slot].item > 0) {
-		tip_data = inventory[area].checkTooltip(position, &pc->stats, ItemManager::PLAYER_INV);
+		tip_data = inventory[area].checkTooltip(position, &pc->stats, ItemManager::PLAYER_INV, ItemManager::TOOLTIP_INPUT_HINT);
 	}
 	else if (area == EQUIPMENT && inventory[area][slot].empty()) {
 		tip_data.addText(msg->get(items->getItemType(slot_type[slot])));
@@ -589,11 +733,24 @@ void MenuInventory::activate(const Point& position) {
 			return;
 		}
 
-		pc->power_cooldown_timers[power_id].setDuration(powers->powers[power_id].cooldown);
-
 		// if this item requires targeting it can't be used this way
 		if (!powers->powers[power_id].requires_targeting) {
-			powers->activate(power_id, &pc->stats, nullpt);
+			ActionData action_data;
+			action_data.power = power_id;
+			action_data.activated_from_inventory = true;
+
+			action_data.target = Utils::calcVector(pc->stats.pos, pc->stats.direction, pc->stats.melee_range);
+
+			if (powers->powers[power_id].new_state == Power::STATE_INSTANT) {
+				for (size_t j = 0; j < powers->powers[power_id].required_items.size(); ++j) {
+					if (powers->powers[power_id].required_items[j].id > 0 && !powers->powers[power_id].required_items[j].equipped) {
+						action_data.instant_item = true;
+						break;
+					}
+				}
+			}
+
+			pc->action_queue.push_back(action_data);
 		}
 		else {
 			// let player know this can only be used from the action bar
@@ -821,7 +978,7 @@ bool MenuInventory::buy(ItemStack stack, int tab, bool dragging) {
 	}
 
 	int value_each;
-	if (tab == ItemManager::VENDOR_BUY) value_each = items->items[stack.item].getPrice();
+	if (tab == ItemManager::VENDOR_BUY) value_each = items->items[stack.item].getPrice(ItemManager::USE_VENDOR_RATIO);
 	else value_each = items->items[stack.item].getSellPrice(stack.can_buyback);
 
 	int count = value_each * stack.quantity;
@@ -858,7 +1015,7 @@ bool MenuInventory::sell(ItemStack stack) {
 	if (stack.item == eset->misc.currency_id) return false;
 
 	// items that have no price cannot be sold
-	if (items->items[stack.item].getPrice() == 0) {
+	if (items->items[stack.item].getPrice(ItemManager::USE_VENDOR_RATIO) == 0) {
 		items->playSound(stack.item);
 		pc->logMsg(msg->get("This item can not be sold."), Avatar::MSG_NORMAL);
 		return false;
@@ -909,16 +1066,18 @@ void MenuInventory::applyEquipment() {
 		}
 
 		for (int i = 0; i < MAX_EQUIPPED; i++) {
-			item_id = inventory[EQUIPMENT].storage[i].item;
-			const Item &item = items->items[item_id];
-			unsigned bonus_counter = 0;
-			while (bonus_counter < item.bonus.size()) {
-				for (size_t j = 0; j < eset->primary_stats.list.size(); ++j) {
-					if (item.bonus[bonus_counter].base_index == static_cast<int>(j))
-						pc->stats.primary_additional[j] += item.bonus[bonus_counter].value;
-				}
+			if (isActive(i)) {
+				item_id = inventory[EQUIPMENT].storage[i].item;
+				const Item &item = items->items[item_id];
+				unsigned bonus_counter = 0;
+				while (bonus_counter < item.bonus.size()) {
+					for (size_t j = 0; j < eset->primary_stats.list.size(); ++j) {
+						if (item.bonus[bonus_counter].base_index == static_cast<int>(j))
+							pc->stats.primary_additional[j] += item.bonus[bonus_counter].value;
+					}
 
-				bonus_counter++;
+					bonus_counter++;
+				}
 			}
 		}
 
@@ -928,14 +1087,16 @@ void MenuInventory::applyEquipment() {
 		std::vector<int> quantity;
 
 		for (int i=0; i<MAX_EQUIPPED; i++) {
-			item_id = inventory[EQUIPMENT].storage[i].item;
-			it = std::find(set.begin(), set.end(), items->items[item_id].set);
-			if (items->items[item_id].set > 0 && it != set.end()) {
-				quantity[std::distance(set.begin(), it)] += 1;
-			}
-			else if (items->items[item_id].set > 0) {
-				set.push_back(items->items[item_id].set);
-				quantity.push_back(1);
+			if (isActive(i)) {
+				item_id = inventory[EQUIPMENT].storage[i].item;
+				it = std::find(set.begin(), set.end(), items->items[item_id].set);
+				if (items->items[item_id].set > 0 && it != set.end()) {
+					quantity[std::distance(set.begin(), it)] += 1;
+				}
+				else if (items->items[item_id].set > 0) {
+					set.push_back(items->items[item_id].set);
+					quantity.push_back(1);
+				}
 			}
 		}
 		// calculate bonuses to basic stats, added by item sets
@@ -951,9 +1112,18 @@ void MenuInventory::applyEquipment() {
 				}
 			}
 		}
-		// check that each equipped item fit requirements
+		// check that each equipped item fit requirements and is in the proper type of slot
 		for (int i = 0; i < MAX_EQUIPPED; i++) {
-			if (!items->requirementsMet(&pc->stats, inventory[EQUIPMENT].storage[i].item)) {
+			bool remove_item = false;
+
+			if (isActive(i) && !items->requirementsMet(&pc->stats, inventory[EQUIPMENT].storage[i].item)) {
+				remove_item = true;
+			}
+			else if (!inventory[EQUIPMENT].storage[i].empty() && slot_type[i] != items->items[inventory[EQUIPMENT].storage[i].item].type) {
+				remove_item = true;
+			}
+
+			if (remove_item) {
 				add(inventory[EQUIPMENT].storage[i], CARRIED, ItemStorage::NO_SLOT, ADD_PLAY_SOUND, !ADD_AUTO_EQUIP);
 				inventory[EQUIPMENT].storage[i].clear();
 				checkRequired = true;
@@ -989,6 +1159,9 @@ void MenuInventory::applyEquipment() {
 	}
 	// disable any incompatible slots, unequipping items if neccessary
 	for (int i=0; i<MAX_EQUIPPED; ++i) {
+		if (!isActive(i))
+			continue;
+
 		ItemID id = inventory[EQUIPMENT][i].item;
 		for (unsigned j=0; j<items->items[id].disable_slots.size(); ++j) {
 			disableEquipmentSlot(items->items[id].disable_slots[j]);
@@ -1031,36 +1204,38 @@ void MenuInventory::applyItemStats() {
 
 	// apply stats from all items
 	for (int i=0; i<MAX_EQUIPPED; i++) {
-		ItemID item_id = inventory[EQUIPMENT].storage[i].item;
-		const Item &item = items->items[item_id];
+		if (isActive(i)) {
+			ItemID item_id = inventory[EQUIPMENT].storage[i].item;
+			const Item &item = items->items[item_id];
 
-		// apply base stats
-		for (size_t j = 0; j < eset->damage_types.list.size(); ++j) {
-			pc->stats.dmg_min_add[j] += item.dmg_min[j];
-			pc->stats.dmg_max_add[j] += item.dmg_max[j];
-		}
+			// apply base stats
+			for (size_t j = 0; j < eset->damage_types.list.size(); ++j) {
+				pc->stats.dmg_min_add[j] += item.dmg_min[j];
+				pc->stats.dmg_max_add[j] += item.dmg_max[j];
+			}
 
-		// set equip flags
-		for (unsigned j=0; j<item.equip_flags.size(); ++j) {
-			pc->stats.equip_flags.insert(item.equip_flags[j]);
-		}
+			// set equip flags
+			for (unsigned j=0; j<item.equip_flags.size(); ++j) {
+				pc->stats.equip_flags.insert(item.equip_flags[j]);
+			}
 
-		// apply absorb bonus
-		pc->stats.absorb_min_add += item.abs_min;
-		pc->stats.absorb_max_add += item.abs_max;
+			// apply absorb bonus
+			pc->stats.absorb_min_add += item.abs_min;
+			pc->stats.absorb_max_add += item.abs_max;
 
-		// apply various bonuses
-		unsigned bonus_counter = 0;
-		while (bonus_counter < item.bonus.size()) {
-			applyBonus(&item.bonus[bonus_counter]);
-			bonus_counter++;
-		}
+			// apply various bonuses
+			unsigned bonus_counter = 0;
+			while (bonus_counter < item.bonus.size()) {
+				applyBonus(&item.bonus[bonus_counter]);
+				bonus_counter++;
+			}
 
-		// add item powers
-		if (item.power > 0) {
-			pc->stats.powers_list_items.push_back(item.power);
-			if (pc->stats.effects.triggered_others)
-				powers->activateSinglePassive(&pc->stats, item.power);
+			// add item powers
+			if (item.power > 0) {
+				pc->stats.powers_list_items.push_back(item.power);
+				if (pc->stats.effects.triggered_others)
+					powers->activateSinglePassive(&pc->stats, item.power);
+			}
 		}
 	}
 }
@@ -1072,14 +1247,16 @@ void MenuInventory::applyItemSetBonuses() {
 	std::vector<int> quantity;
 
 	for (int i=0; i<MAX_EQUIPPED; i++) {
-		ItemID item_id = inventory[EQUIPMENT].storage[i].item;
-		it = std::find(set.begin(), set.end(), items->items[item_id].set);
-		if (items->items[item_id].set > 0 && it != set.end()) {
-			quantity[std::distance(set.begin(), it)] += 1;
-		}
-		else if (items->items[item_id].set > 0) {
-			set.push_back(items->items[item_id].set);
-			quantity.push_back(1);
+		if (isActive(i)) {
+			ItemID item_id = inventory[EQUIPMENT].storage[i].item;
+			it = std::find(set.begin(), set.end(), items->items[item_id].set);
+			if (items->items[item_id].set > 0 && it != set.end()) {
+				quantity[std::distance(set.begin(), it)] += 1;
+			}
+			else if (items->items[item_id].set > 0) {
+				set.push_back(items->items[item_id].set);
+				quantity.push_back(1);
+			}
 		}
 	}
 	// apply item set bonuses
@@ -1112,7 +1289,7 @@ void MenuInventory::applyBonus(const BonusData* bdata) {
 		ed.id = eset->damage_types.list[bdata->damage_index_max].max;
 	}
 	else if (bdata->resist_index != -1) {
-		ed.id = eset->elements.list[bdata->resist_index].id + "_resist";
+		ed.id = eset->elements.list[bdata->resist_index].resist_id;
 	}
 	else if (bdata->base_index > -1 && static_cast<size_t>(bdata->base_index) < eset->primary_stats.list.size()) {
 		ed.id = eset->primary_stats.list[bdata->base_index].id;
@@ -1127,6 +1304,101 @@ void MenuInventory::applyBonus(const BonusData* bdata) {
 	pc->stats.effects.addItemEffect(ed, 0, bdata->value);
 }
 
+void MenuInventory::applyEquipmentSet(unsigned set) {
+	unsigned prev_equipment_set = active_equipment_set;
+
+	if (set > 0 && set <= max_equipment_set) {
+		active_equipment_set = set;
+		updateEquipmentSetWidgets();
+	}
+
+	if (active_equipment_set > 0 && prev_equipment_set != active_equipment_set) {
+		if (!visible && menu && menu->hudlog) {
+			menu->hudlog->add(msg->get("Equipped set %d.", static_cast<int>(active_equipment_set)), MenuHUDLog::MSG_NORMAL);
+		}
+	}
+}
+
+void MenuInventory::applyNextEquipmentSet() {
+	if (active_equipment_set < max_equipment_set) {
+		applyEquipmentSet(active_equipment_set+1);
+	}
+	else {
+		applyEquipmentSet(1);
+	}
+}
+
+void MenuInventory::applyPreviousEquipmentSet() {
+	if (active_equipment_set > 1) {
+		applyEquipmentSet(active_equipment_set-1);
+	}
+	else {
+		applyEquipmentSet(max_equipment_set);
+	}
+}
+
+void MenuInventory::updateEquipmentSetWidgets() {
+	Widget* first_active_slot = NULL;
+	Widget* current_tablist_widget = tablist.getWidgetByIndex(tablist.getCurrent());
+	bool reset_tablist_cursor = false;
+
+	for (int i=0; i<MAX_EQUIPPED; i++) {
+		if (isActive(i)) {
+			if (!first_active_slot) {
+				first_active_slot = inventory[EQUIPMENT].slots[i];
+			}
+			inventory[EQUIPMENT].slots[i]->visible = true;
+			inventory[EQUIPMENT].slots[i]->enable_tablist_nav = true;
+		}
+		else {
+			inventory[EQUIPMENT].slots[i]->visible = false;
+			inventory[EQUIPMENT].slots[i]->enable_tablist_nav = false;
+			if (current_tablist_widget && inventory[EQUIPMENT].slots[i] == current_tablist_widget) {
+				reset_tablist_cursor = true;
+			}
+		}
+	}
+
+	if (!equipmentSetButton.empty()) {
+		for (size_t i=0; i<equipmentSetButton.size(); i++) {
+			if (active_equipment_set > 0) {
+				if (i == active_equipment_set-1) {
+					equipmentSetButton[i]->enabled = false;
+					if (current_tablist_widget && equipmentSetButton[i] == current_tablist_widget) {
+						reset_tablist_cursor = true;
+					}
+
+				}
+				else {
+					equipmentSetButton[i]->enabled = true;
+				}
+			}
+		}
+	}
+
+	if (reset_tablist_cursor && first_active_slot) {
+		tablist.setCurrent(first_active_slot);
+	}
+
+	if (equipmentSetLabel) {
+		std::stringstream label;
+		label << active_equipment_set << "/" << max_equipment_set;
+		equipmentSetLabel->setText(label.str());
+		equipmentSetLabel->setColor(font->getColor(FontEngine::COLOR_MENU_NORMAL));
+	}
+
+	changed_equipment = true;
+}
+
+
+bool MenuInventory::isActive(size_t equipped) {
+	if (equipment_set[equipped] == 0 || equipment_set[equipped]==active_equipment_set) {
+		return true;
+	}
+
+	return false;
+}
+
 int MenuInventory::getEquippedCount() {
 	return static_cast<int>(equipped_area.size());
 }
@@ -1144,50 +1416,58 @@ void MenuInventory::clearHighlight() {
  * Sort equipment storage array, so items order matches slots order
  */
 void MenuInventory::fillEquipmentSlots() {
-	// create temporary arrays
-	int slot_number = MAX_EQUIPPED;
-	ItemID *equip_item = new ItemID[slot_number];
-	int *equip_quantity = new int[slot_number];;
+	// create temporary array
+	ItemStack *equip_stack = new ItemStack[MAX_EQUIPPED];
 
-	// initialize arrays
-	for (int i=0; i<slot_number; i++) {
-		equip_item[i] = inventory[EQUIPMENT].storage[i].item;
-		equip_quantity[i] = inventory[EQUIPMENT].storage[i].quantity;
-	}
-	// clean up storage[]
-	for (int i=0; i<slot_number; i++) {
+	for (int i = 0; i < MAX_EQUIPPED; ++i) {
+		// initialize array
+		// if an item is set, ensure the quantity is >= 1
+		equip_stack[i] = inventory[EQUIPMENT].storage[i];
+		if (equip_stack[i].item > 0)
+			equip_stack[i].quantity = std::max(1, equip_stack[i].quantity);
+		else
+			equip_stack[i].clear();
+
+		// clean up storage[]
 		inventory[EQUIPMENT].storage[i].clear();
+
+		// if items were in the correct slot, put them back
+		if (!equip_stack[i].empty() && inventory[EQUIPMENT].storage[i].empty() && items->items[equip_stack[i].item].type == slot_type[i]) {
+			inventory[EQUIPMENT].storage[i] = equip_stack[i];
+			equip_stack[i].clear();
+		}
 	}
 
-	// fill slots with items
-	for (int i=0; i<slot_number; i++) {
+	// for items that weren't in a matching slot, try to find one
+	// if all else fails, add them to the inventory
+	for (int i = 0; i < MAX_EQUIPPED; ++i) {
+		if (equip_stack[i].empty())
+			continue;
+
 		bool found_slot = false;
-		for (int j=0; j<slot_number; j++) {
-			// search for empty slot with needed type. If item is not NULL, put it there
-			if (equip_item[i] > 0 && inventory[EQUIPMENT].storage[j].empty()) {
-				if (items->items[equip_item[i]].type == slot_type[j]) {
-					inventory[EQUIPMENT].storage[j].item = equip_item[i];
-					inventory[EQUIPMENT].storage[j].quantity = (equip_quantity[i] > 0) ? equip_quantity[i] : 1;
+		for (int j = 0; j < MAX_EQUIPPED; ++j) {
+			// search for empty slot with needed type
+			if (inventory[EQUIPMENT].storage[j].empty()) {
+				if (items->items[equip_stack[i].item].type == slot_type[j]) {
+					inventory[EQUIPMENT].storage[j] = equip_stack[i];
 					found_slot = true;
 					break;
 				}
 			}
 		}
+
 		// couldn't find a slot, adding to inventory
-		if (!found_slot && equip_item[i] > 0) {
-			ItemStack stack;
-			stack.item = equip_item[i];
-			stack.quantity = (equip_quantity[i] > 0) ? equip_quantity[i] : 1;
-			add(stack, CARRIED, ItemStorage::NO_SLOT, !ADD_PLAY_SOUND, !ADD_AUTO_EQUIP);
+		if (!found_slot) {
+			add(equip_stack[i], CARRIED, ItemStorage::NO_SLOT, !ADD_PLAY_SOUND, !ADD_AUTO_EQUIP);
 		}
 	}
-	delete [] equip_item;
-	delete [] equip_quantity;
+
+	delete [] equip_stack;
 }
 
 int MenuInventory::getMaxPurchasable(ItemStack item, int vendor_tab) {
 	if (vendor_tab == ItemManager::VENDOR_BUY)
-		return currency / items->items[item.item].getPrice();
+		return currency / items->items[item.item].getPrice(ItemManager::USE_VENDOR_RATIO);
 	else if (vendor_tab == ItemManager::VENDOR_SELL)
 		return currency / items->items[item.item].getSellPrice(item.can_buyback);
 	else
@@ -1202,6 +1482,9 @@ int MenuInventory::getEquipSlotFromItem(ItemID item, bool only_empty_slots) {
 
 	// find first empty(or just first) slot for item to equip
 	for (int i = 0; i < MAX_EQUIPPED; i++) {
+		if (!isActive(i))
+			continue;
+
 		if (slot_type[i] == items->items[item].type) {
 			if (inventory[EQUIPMENT].storage[i].empty()) {
 				// empty and matching, no need to search more
@@ -1234,7 +1517,7 @@ PowerID MenuInventory::getPowerMod(PowerID meta_power) {
 
 void MenuInventory::disableEquipmentSlot(const std::string& disable_slot_type) {
 	for (int i=0; i<MAX_EQUIPPED; ++i) {
-		if (slot_type[i] == disable_slot_type) {
+		if (isActive(i) && slot_type[i] == disable_slot_type) {
 			if (!inventory[EQUIPMENT].storage[i].empty()) {
 				add(inventory[EQUIPMENT].storage[i], CARRIED, ItemStorage::NO_SLOT, ADD_PLAY_SOUND, !ADD_AUTO_EQUIP);
 				inventory[EQUIPMENT].storage[i].clear();
@@ -1268,6 +1551,60 @@ int MenuInventory::getEquippedSetCount(size_t set_id) {
 	return quantity;
 }
 
+bool MenuInventory::canEquipItem(const Point& position) {
+	// clicked a carried item
+	int slot = inventory[CARRIED].slotOver(position);
+	if (slot == -1)
+		return false;
+
+	// empty slot
+	if (inventory[CARRIED][slot].empty())
+		return false;
+
+	ItemID item_id = inventory[CARRIED][slot].item;
+
+	return (pc->stats.humanoid && !items->items[item_id].type.empty() && getEquipSlotFromItem(item_id, !ONLY_EMPTY_SLOTS) >= 0);
+}
+
+bool MenuInventory::canUseItem(const Point& position) {
+	// clicked a carried item
+	int slot = inventory[CARRIED].slotOver(position);
+	if (slot == -1)
+		return false;
+
+	// empty slot
+	if (inventory[CARRIED][slot].empty())
+		return false;
+
+	ItemID item_id = inventory[CARRIED][slot].item;
+
+	return canActivateItem(item_id);
+}
+
+// same as ItemStorage::contain(), but accounts for active equipment set
+bool MenuInventory::equipmentContain(ItemID item, int quantity) {
+	int total_quantity = 0;
+	for (int i = 0; i < MAX_EQUIPPED; ++i) {
+		if (!isActive(i))
+			continue;
+
+		if (inventory[EQUIPMENT][i].item == item)
+			total_quantity += inventory[EQUIPMENT][i].quantity;
+
+		if (total_quantity >= quantity)
+			return true;
+	}
+	return false;
+}
+
 MenuInventory::~MenuInventory() {
 	delete closeButton;
+	for (size_t i=0; i<equipmentSetButton.size(); i++) {
+		delete equipmentSetButton[i];
+	}
+
+	delete equipmentSetNext;
+	delete equipmentSetPrevious;
+	delete equipmentSetLabel;
+
 }
