@@ -29,6 +29,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EntityManager.h"
 #include "EngineSettings.h"
 #include "FileParser.h"
+#include "FogOfWar.h"
 #include "FontEngine.h"
 #include "InputState.h"
 #include "MapCollision.h"
@@ -46,6 +47,12 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 
 #include <cmath>
 
+PixelEntity::PixelEntity(int _x, int _y, Color* _color) {
+	x = _x;
+	y = _y;
+	color = _color;
+}
+
 MenuMiniMap::MenuMiniMap()
 	: color_wall(128,128,128)
 	, color_obst(64,64,64)
@@ -61,12 +68,12 @@ MenuMiniMap::MenuMiniMap()
 	, label(new WidgetLabel())
 	, compass(NULL)
 	, button_config(NULL)
+	, visible_radius(0)
 	, current_zoom(1)
 	, lock_zoom_change(false)
 	, clicked_config(false)
-{
-	std::string bg_filename;
 
+{
 	// Load config settings
 	FileParser infile;
 	// @CLASS MenuMiniMap|Description of menus/minimap.txt
@@ -83,10 +90,6 @@ MenuMiniMap::MenuMiniMap()
 			else if(infile.key == "text_pos") {
 				label->setFromLabelInfo(Parse::popLabelInfo(infile.val));
 			}
-			// @ATTR background|filename|Optional background image.
-			else if (infile.key == "background") {
-				bg_filename = infile.val;
-			}
 			// @ATTR color_wall|color, int : Color, Alpha|Color used for walls.
 			else if (infile.key == "color_wall") {
 				color_wall = Parse::toRGBA(infile.val);
@@ -130,10 +133,9 @@ MenuMiniMap::MenuMiniMap()
 		infile.close();
 	}
 
-	label->setColor(font->getColor(FontEngine::COLOR_MENU_NORMAL));
+	visible_radius = static_cast<float>(std::max(pos.w, pos.h))*0.7071f;
 
-	if (!bg_filename.empty())
-		setBackground(bg_filename);
+	label->setColor(font->getColor(FontEngine::COLOR_MENU_NORMAL));
 
 	// load compass image
 	Image *gfx = NULL;
@@ -194,7 +196,17 @@ void MenuMiniMap::createMapSurface(Sprite **target_surface, int w, int h) {
 }
 
 void MenuMiniMap::logic() {
-	if (!settings->show_hud || settings->minimap_mode == Settings::MINIMAP_HIDDEN)
+	if (!settings->show_hud)
+		return;
+
+	if (inpt->pressing[Input::MINIMAP_MODE] && !inpt->lock[Input::MINIMAP_MODE]) {
+		inpt->lock[Input::MINIMAP_MODE] = true;
+		settings->minimap_mode++;
+		if (settings->minimap_mode > Settings::MINIMAP_2X)
+			settings->minimap_mode = Settings::MINIMAP_NORMAL;
+	}
+
+	if (settings->minimap_mode == Settings::MINIMAP_HIDDEN)
 		return;
 
 	if (inpt->usingMouse()) {
@@ -252,8 +264,6 @@ void MenuMiniMap::prerender(MapCollision *collider, int map_w, int map_h) {
 	map_size.x = map_w;
 	map_size.y = map_h;
 
-	clearEntities();
-
 	if (eset->tileset.orientation == eset->tileset.TILESET_ISOMETRIC) {
 		prerenderIso(collider, &map_surface, &map_surface_entities, 1);
 		prerenderIso(collider, &map_surface_2x, &map_surface_entities_2x, 2);
@@ -264,6 +274,17 @@ void MenuMiniMap::prerender(MapCollision *collider, int map_w, int map_h) {
 		prerenderOrtho(collider, &map_surface_2x, &map_surface_entities_2x, 2);
 	}
 }
+void MenuMiniMap::update(MapCollision *collider, Rect *bounds) {
+	if (eset->tileset.orientation == eset->tileset.TILESET_ISOMETRIC) {
+		updateIso(collider, &map_surface, 1, bounds);
+		updateIso(collider, &map_surface_2x, 2, bounds);
+	}
+	else {
+		// eset->tileset.TILESET_ORTHOGONAL
+		updateOrtho(collider, &map_surface, 1, bounds);
+		updateOrtho(collider, &map_surface_2x, 2, bounds);
+	}
+}
 
 void MenuMiniMap::renderMapSurface(const FPoint& hero_pos) {
 
@@ -278,12 +299,18 @@ void MenuMiniMap::renderMapSurface(const FPoint& hero_pos) {
 		hero_offset = Point(hero_pos);
 	}
 
+	Point entity_offset;
+	entity_offset.x = (current_zoom * hero_offset.x) - pos.w/2;
+	entity_offset.y = (current_zoom * hero_offset.y) - pos.h/2;
+
 	Rect clip;
 	clip.x = (current_zoom * hero_offset.x) - pos.w/2;
 	clip.y = (current_zoom * hero_offset.y) - pos.h/2;
 	clip.w = pos.w;
 	clip.h = pos.h;
 
+	Rect clip_entities(0, 0, pos.w, pos.h);
+
 	Sprite* target_surface = NULL;
 	Sprite* target_surface_entities = NULL;
 	if (settings->minimap_mode == Settings::MINIMAP_NORMAL && map_surface) {
@@ -303,14 +330,14 @@ void MenuMiniMap::renderMapSurface(const FPoint& hero_pos) {
 
 	if (target_surface_entities) {
 		if (eset->tileset.orientation == eset->tileset.TILESET_ISOMETRIC) {
-			renderEntitiesIso(target_surface_entities, current_zoom);
+			renderEntitiesIso(target_surface_entities, current_zoom, entity_offset);
 		}
 		else {
 			// eset->tileset.TILESET_ORTHOGONAL
-			renderEntitiesOrtho(target_surface_entities, current_zoom);
+			renderEntitiesOrtho(target_surface_entities, current_zoom, entity_offset);
 		}
 
-		target_surface_entities->setClipFromRect(clip);
+		target_surface_entities->setClipFromRect(clip_entities);
 		target_surface_entities->setDestFromRect(map_area);
 		render_device->render(target_surface_entities);
 	}
@@ -319,21 +346,46 @@ void MenuMiniMap::renderMapSurface(const FPoint& hero_pos) {
 void MenuMiniMap::prerenderOrtho(MapCollision *collider, Sprite** tile_surface, Sprite** entity_surface, int zoom) {
 	int surface_size = std::max(map_size.x + zoom, map_size.y + zoom) * zoom;
 	createMapSurface(tile_surface, surface_size, surface_size);
-	createMapSurface(entity_surface, surface_size, surface_size);
+	createMapSurface(entity_surface, pos.w, pos.h);
+
+	if (!(*tile_surface))
+		return;
+
+	Rect bounds(0, 0, map_size.x, map_size.y);
+	updateOrtho(collider, tile_surface, zoom, &bounds);
+}
+
+void MenuMiniMap::updateOrtho(MapCollision *collider, Sprite** tile_surface, int zoom, Rect *bounds) {
 
 	if (!(*tile_surface))
 		return;
 
 	Image* target_img = (*tile_surface)->getGraphics();
-	const int target_w = (*tile_surface)->getGraphicsWidth();
-	const int target_h = (*tile_surface)->getGraphicsHeight();
 
 	Color draw_color;
 
-	target_img->beginPixelBatch();
+	if (bounds->x == 0 && bounds->y == 0 && bounds->w == map_size.x && bounds->h == map_size.y) {
+		target_img->beginPixelBatch();
+	}
+	else {
+		Point hero(pc->stats.pos);
 
-	for (int i=0; i<std::min(target_w, map_size.x); i++) {
-		for (int j=0; j<std::min(target_h, map_size.y); j++) {
+		Rect clip;
+		clip.x = (zoom * hero.x) - pos.w/2;
+		clip.y = (zoom * hero.y) - pos.h/2;
+		clip.w = pos.w;
+		clip.h = pos.h;
+
+		if (clip.x<0) clip.x=0;
+		if (clip.y<0) clip.y=0;
+		if (clip.x + clip.w > target_img->getWidth()) clip.w = target_img->getWidth() - clip.x;
+		if (clip.y + clip.h > target_img->getHeight()) clip.h = target_img->getHeight() - clip.y;
+
+		target_img->beginPixelBatch(clip);
+	}
+
+	for (int i=bounds->x; i<bounds->w; i++) {
+		for (int j=bounds->y; j<bounds->h; j++) {
 			bool draw_tile = true;
 			int tile_type = collider->colmap[i][j];
 
@@ -341,10 +393,15 @@ void MenuMiniMap::prerenderOrtho(MapCollision *collider, Sprite** tile_surface,
 			else if (tile_type == 2 || tile_type == 6) draw_color = color_obst;
 			else draw_tile = false;
 
+			if (eset->misc.fogofwar > 0) {
+				tile_type = mapr->layers[fow->dark_layer_id][i][j];
+				if (tile_type != 0) draw_tile = false;
+			}
+
 			if (draw_tile && draw_color.a != 0) {
 				for (int l = 0; l < zoom; l++) {
 					for (int k =0; k < zoom; k++) {
-						target_img->drawPixel((zoom*i)+k, (zoom*j)+l, draw_color);
+						target_img->drawPixel((zoom*i)+k-1, (zoom*j)+l-1, draw_color);
 					}
 				}
 			}
@@ -357,117 +414,72 @@ void MenuMiniMap::prerenderOrtho(MapCollision *collider, Sprite** tile_surface,
 void MenuMiniMap::prerenderIso(MapCollision *collider, Sprite** tile_surface, Sprite** entity_surface, int zoom) {
 	int surface_size = std::max(map_size.x + zoom, map_size.y + zoom) * 2 * zoom;
 	createMapSurface(tile_surface, surface_size, surface_size);
-	createMapSurface(entity_surface, surface_size, surface_size);
+	createMapSurface(entity_surface, pos.w, pos.h);
 
 	if (!(*tile_surface))
 		return;
 
-	// a 2x1 pixel area correlates to a tile, so we can traverse tiles using pixel counting
-	Color draw_color;
-	int tile_type;
-
-	Point tile_cursor;
-	tile_cursor.x = -std::max(map_size.x, map_size.y)/2;
-	tile_cursor.y = std::max(map_size.x, map_size.y)/2;
-
-	bool odd_row = false;
-
-	Image* target_img = (*tile_surface)->getGraphics();
-	const int target_w = (*tile_surface)->getGraphicsWidth();
-	const int target_h = (*tile_surface)->getGraphicsHeight();
-
-	target_img->beginPixelBatch();
-
-	// for each pixel row
-	for (int j=0; j<target_h; j++) {
-
-		// for each 2-px wide column
-		for (int i=0; i<target_w; i+=2) {
-
-			// if this tile is the max map size
-			if (tile_cursor.x >= 0 && tile_cursor.y >= 0 && tile_cursor.x < map_size.x && tile_cursor.y < map_size.y) {
-
-				tile_type = collider->colmap[tile_cursor.x][tile_cursor.y];
-				bool draw_tile = true;
-
-				// walls and low obstacles show as different colors
-				if (tile_type == 1 || tile_type == 5) draw_color = color_wall;
-				else if (tile_type == 2 || tile_type == 6) draw_color = color_obst;
-				else draw_tile = false;
-
-				if (draw_tile && draw_color.a != 0) {
-					if (odd_row) {
-						for (int l = 0; l < zoom; l++) {
-							for (int k = 0; k < zoom * 2; k++) {
-								target_img->drawPixel((zoom*i)+k, (zoom*j)+l, draw_color);
-							}
-						}
-					}
-					else {
-						for (int l = 0; l < zoom; l++) {
-							for (int k = -((zoom * 2) - zoom); k < zoom; k++) {
-								target_img->drawPixel((zoom*i)+k, (zoom*j)+l, draw_color);
-							}
-						}
-					}
-				}
-			}
-
-			// moving screen-right in isometric is +x -y in map coordinates
-			tile_cursor.x++;
-			tile_cursor.y--;
-		}
-
-		// return tile cursor to next row of tiles
-		if (odd_row) {
-			odd_row = false;
-			tile_cursor.x -= target_w/2;
-			tile_cursor.y += (target_w/2 +1);
-		}
-		else {
-			odd_row = true;
-			tile_cursor.x -= (target_w/2 -1);
-			tile_cursor.y += target_w/2;
-		}
-	}
-
-	target_img->endPixelBatch();
+	Rect bounds(0, 0, map_size.x, map_size.y);
+	updateIso(collider, tile_surface, zoom, &bounds);
 }
 
-void MenuMiniMap::renderEntitiesOrtho(Sprite* entity_surface, int zoom) {
-	if (!entity_surface)
+void MenuMiniMap::updateIso(MapCollision *collider, Sprite** tile_surface, int zoom, Rect *bounds) {
+
+	if (!(*tile_surface))
 		return;
 
 	Color draw_color;
 	int tile_type;
 
-	Image* target_img = entity_surface->getGraphics();
-	const int target_w = entity_surface->getGraphicsWidth();
-	const int target_h = entity_surface->getGraphicsHeight();
+	Point ent_pos;
+	Image* target_img = (*tile_surface)->getGraphics();
 
-	target_img->fillWithColor(Color(0,0,0,0));
+	if (bounds->x == 0 && bounds->y == 0 && bounds->w == map_size.x && bounds->h == map_size.y) {
+		target_img->beginPixelBatch();
+	}
+	else {
+		Point hero(pc->stats.pos);
+		Point hero_offset;
+		hero_offset.x = hero.x - hero.y + std::max(map_size.x, map_size.y);
+		hero_offset.y = hero.x + hero.y;
 
-	clearEntities();
-	fillEntities();
+		Rect clip;
+		clip.x = (zoom * hero_offset.x) - pos.w/2;
+		clip.y = (zoom * hero_offset.y) - pos.h/2;
+		clip.w = pos.w;
+		clip.h = pos.h;
 
-	target_img->beginPixelBatch();
+		if (clip.x<0) clip.x=0;
+		if (clip.y<0) clip.y=0;
+		if (clip.x + clip.w > target_img->getWidth()) clip.w = target_img->getWidth() - clip.x;
+		if (clip.y + clip.h > target_img->getHeight()) clip.h = target_img->getHeight() - clip.y;
 
-	for (int i=0; i<std::min(target_w, map_size.x); i++) {
-		for (int j=0; j<std::min(target_h, map_size.y); j++) {
+		target_img->beginPixelBatch(clip);
+	}
+
+	for (int i=bounds->x; i<bounds->w; i++) {
+		for (int j=bounds->y; j<bounds->h; j++) {
+			tile_type = collider->colmap[i][j];
 			bool draw_tile = true;
-			tile_type = entities[i][j];
 
-			if (tile_type == TILE_HERO) draw_color = color_hero;
-			else if (tile_type == TILE_ENEMY) draw_color = color_enemy;
-			else if (tile_type == TILE_NPC) draw_color = color_npc;
-			else if (tile_type == TILE_TELEPORT) draw_color = color_teleport;
-			else if (tile_type == TILE_ALLY) draw_color = color_ally;
+			if (tile_type == 1 || tile_type == 5) draw_color = color_wall;
+			else if (tile_type == 2 || tile_type == 6) draw_color = color_obst;
 			else draw_tile = false;
 
-			if (draw_tile && draw_color.a != 0) {
+			// fog of war
+			if (eset->misc.fogofwar > 0) {
+				tile_type = mapr->layers[fow->dark_layer_id][i][j];
+				if (tile_type != 0) draw_tile = false;
+			}
+
+			if (draw_tile) {
+				ent_pos.x = zoom*(i - j + std::max(map_size.x, map_size.y));
+				ent_pos.y = zoom*(i + j) - 1;
+
 				for (int l = 0; l < zoom; l++) {
-					for (int k =0; k < zoom; k++) {
-						target_img->drawPixel((zoom*i)+k, (zoom*j)+l, draw_color);
+					for (int k = 0; k < zoom; k++) {
+						target_img->drawPixel(ent_pos.x+k, ent_pos.y+l, draw_color);
+						target_img->drawPixel(ent_pos.x+k-zoom, ent_pos.y+l, draw_color);
 					}
 				}
 			}
@@ -477,24 +489,11 @@ void MenuMiniMap::renderEntitiesOrtho(Sprite* entity_surface, int zoom) {
 	target_img->endPixelBatch();
 }
 
-void MenuMiniMap::renderEntitiesIso(Sprite* entity_surface, int zoom) {
+void MenuMiniMap::renderEntitiesOrtho(Sprite* entity_surface, int zoom, const Point& entity_offset) {
 	if (!entity_surface)
 		return;
 
-	// a 2x1 pixel area correlates to a tile, so we can traverse tiles using pixel counting
-	Color draw_color;
-	int tile_type;
-
-	Point tile_cursor;
-	tile_cursor.x = -std::max(map_size.x, map_size.y)/2;
-	tile_cursor.y = std::max(map_size.x, map_size.y)/2;
-
-	bool odd_row = false;
-
 	Image* target_img = entity_surface->getGraphics();
-	const int target_w = entity_surface->getGraphicsWidth();
-	const int target_h = entity_surface->getGraphicsHeight();
-
 	target_img->fillWithColor(Color(0,0,0,0));
 
 	clearEntities();
@@ -502,78 +501,60 @@ void MenuMiniMap::renderEntitiesIso(Sprite* entity_surface, int zoom) {
 
 	target_img->beginPixelBatch();
 
-	// for each pixel row
-	for (int j=0; j<target_h; j++) {
-
-		// for each 2-px wide column
-		for (int i=0; i<target_w; i+=2) {
-
-			// if this tile is the max map size
-			if (tile_cursor.x >= 0 && tile_cursor.y >= 0 && tile_cursor.x < map_size.x && tile_cursor.y < map_size.y) {
-
-				tile_type = entities[tile_cursor.x][tile_cursor.y];
-				bool draw_tile = true;
-
-				if (tile_type == TILE_HERO) draw_color = color_hero;
-				else if (tile_type == TILE_ENEMY) draw_color = color_enemy;
-				else if (tile_type == TILE_NPC) draw_color = color_npc;
-				else if (tile_type == TILE_TELEPORT) draw_color = color_teleport;
-				else if (tile_type == TILE_ALLY) draw_color = color_ally;
-				else draw_tile = false;
-
-				if (draw_tile && draw_color.a != 0) {
-					if (odd_row) {
-						for (int l = 0; l < zoom; l++) {
-							for (int k = 0; k < zoom * 2; k++) {
-								target_img->drawPixel((zoom*i)+k, (zoom*j)+l, draw_color);
-							}
-						}
-					}
-					else {
-						for (int l = 0; l < zoom; l++) {
-							for (int k = -((zoom * 2) - zoom); k < zoom; k++) {
-								target_img->drawPixel((zoom*i)+k, (zoom*j)+l, draw_color);
-							}
-						}
-					}
-				}
+	for (size_t i=0; i<entities.size(); i++) {
+		for (int l = 0; l < zoom; l++) {
+			for (int k = 0; k < zoom; k++) {
+				target_img->drawPixel(zoom*entities[i]->x-entity_offset.x+k-1, zoom*entities[i]->y-entity_offset.y+l-1, *entities[i]->color);
 			}
-
-			// moving screen-right in isometric is +x -y in map coordinates
-			tile_cursor.x++;
-			tile_cursor.y--;
-		}
-
-		// return tile cursor to next row of tiles
-		if (odd_row) {
-			odd_row = false;
-			tile_cursor.x -= target_w/2;
-			tile_cursor.y += (target_w/2 +1);
-		}
-		else {
-			odd_row = true;
-			tile_cursor.x -= (target_w/2 -1);
-			tile_cursor.y += target_w/2;
 		}
 	}
 
 	target_img->endPixelBatch();
 }
 
-void MenuMiniMap::clearEntities() {
-	entities.resize(map_size.x);
-	for (size_t i=0; i<entities.size(); ++i) {
-		entities[i].resize(map_size.y);
-		for (size_t j=0; j<entities[i].size(); ++j) {
-			entities[i][j] = 0;
+void MenuMiniMap::renderEntitiesIso(Sprite* entity_surface, int zoom, const Point& entity_offset) {
+	if (!entity_surface)
+		return;
+
+	Point ent_pos;
+
+	Image* target_img = entity_surface->getGraphics();
+	target_img->fillWithColor(Color(0,0,0,0));
+
+	clearEntities();
+	fillEntities();
+
+	target_img->beginPixelBatch();
+
+	for (size_t i=0; i<entities.size(); i++) {
+		ent_pos.x = zoom*(entities[i]->x - entities[i]->y + std::max(map_size.x, map_size.y)) - entity_offset.x;
+		ent_pos.y = zoom*(entities[i]->x + entities[i]->y) - entity_offset.y - 1;
+
+		for (int l = 0; l < zoom; l++) {
+			for (int k = 0; k < zoom; k++) {
+				target_img->drawPixel(ent_pos.x+k, ent_pos.y+l, *entities[i]->color);
+				target_img->drawPixel(ent_pos.x+k-zoom, ent_pos.y+l, *entities[i]->color);
+			}
 		}
 	}
+
+	target_img->endPixelBatch();
 }
 
+
+void MenuMiniMap::clearEntities() {
+	for (size_t i=0; i<entities.size(); i++) {
+		delete entities[i];
+	}
+	entities.clear();
+}
+
+
 void MenuMiniMap::fillEntities() {
 	Point hero = Point(pc->stats.pos);
+
 	if (hero.x >= 0 && hero.y >= 0 && hero.x < map_size.x && hero.y < map_size.y) {
-		entities[hero.x][hero.y] = TILE_HERO;
+		entities.push_back(new PixelEntity(hero.x, hero.y, &color_hero));
 	}
 
 	for (size_t i=0; i<mapr->events.size(); ++i) {
@@ -582,14 +563,27 @@ void MenuMiniMap::fillEntities() {
 			continue;
 
 		if (mapr->events[i].getComponent(EventComponent::NPC_HOTSPOT) && EventManager::isActive(mapr->events[i])) {
-			entities[mapr->events[i].location.x][mapr->events[i].location.y] = TILE_NPC;
+			if (mapr->fogofwar) {
+				float delta = Utils::calcDist(pc->stats.pos, mapr->events[i].center);
+				if (delta > static_cast<float>(fow->mask_radius)) {
+					continue;
+				}
+			}
+			if (Utils::calcDist(pc->stats.pos, FPoint(mapr->events[i].location.x, mapr->events[i].location.y)) <= visible_radius) {
+				entities.push_back(new PixelEntity(mapr->events[i].location.x, mapr->events[i].location.y, &color_npc));
+			}
 		}
 		else if ((mapr->events[i].activate_type == Event::ACTIVATE_ON_TRIGGER || mapr->events[i].activate_type == Event::ACTIVATE_ON_INTERACT) && mapr->events[i].getComponent(EventComponent::INTERMAP) && EventManager::isActive(mapr->events[i])) {
 			// TODO use location when hotspot is inappropriate?
 			Point event_pos(mapr->events[i].location.x, mapr->events[i].location.y);
 			for (int j=event_pos.x; j<event_pos.x + mapr->events[i].location.w; ++j) {
 				for (int k=event_pos.y; k<event_pos.y + mapr->events[i].location.h; ++k) {
-					entities[j][k] = TILE_TELEPORT;
+					if (mapr->fogofwar)
+						if (mapr->layers[fow->dark_layer_id][event_pos.x][event_pos.y] == FogOfWar::TILE_HIDDEN) continue;
+
+					if (Utils::calcDist(pc->stats.pos, FPoint(j, k)) <= visible_radius) {
+						entities.push_back(new PixelEntity(j, k, &color_teleport));
+					}
 				}
 			}
 		}
@@ -598,11 +592,21 @@ void MenuMiniMap::fillEntities() {
 	for (size_t i=0; i<entitym->entities.size(); ++i) {
 		Entity *e = entitym->entities[i];
 		if (e->stats.hp > 0) {
+			if (mapr->fogofwar) {
+				float delta = Utils::calcDist(pc->stats.pos, e->stats.pos);
+				if (delta > static_cast<float>(fow->mask_radius)) {
+					continue;
+				}
+			}
 			if (e->stats.hero_ally) {
-				entities[static_cast<int>(e->stats.pos.x)][static_cast<int>(e->stats.pos.y)] = TILE_ALLY;
+				if (Utils::calcDist(pc->stats.pos, FPoint(e->stats.pos.x, e->stats.pos.y)) <= visible_radius) {
+					entities.push_back(new PixelEntity(static_cast<int>(e->stats.pos.x), static_cast<int>(e->stats.pos.y), &color_ally));
+				}
 			}
 			else if (e->stats.in_combat) {
-				entities[static_cast<int>(e->stats.pos.x)][static_cast<int>(e->stats.pos.y)] = TILE_ENEMY;
+				if (Utils::calcDist(pc->stats.pos, FPoint(e->stats.pos.x, e->stats.pos.y)) <= visible_radius) {
+					entities.push_back(new PixelEntity(static_cast<int>(e->stats.pos.x), static_cast<int>(e->stats.pos.y), &color_enemy));
+				}
 			}
 		}
 	}
@@ -617,4 +621,6 @@ MenuMiniMap::~MenuMiniMap() {
 	delete label;
 	delete compass;
 	delete button_config;
+
+	clearEntities();
 }
