@@ -70,10 +70,8 @@ MenuTalker::MenuTalker()
 	, npc(NULL)
 	, advanceButton(new WidgetButton("images/menus/buttons/right.png"))
 	, closeButton(new WidgetButton("images/menus/buttons/button_x.png"))
-	, npc_from_map(true) {
-
-	setBackground("images/menus/dialog_box.png");
-
+	, npc_from_map(true)
+{
 	// Load config settings
 	FileParser infile;
 	// @CLASS MenuTalker|Description of menus/talker.txt
@@ -132,6 +130,10 @@ MenuTalker::MenuTalker()
 
 	textbox = new WidgetScrollBox(text_pos.w, text_pos.h-(text_offset.y*2));
 	textbox->setBasePos(text_pos.x, text_pos.y + text_offset.y, Utils::ALIGN_TOPLEFT);
+	textbox->show_focus_when_scrollbar_disabled = false;
+
+	if (!background)
+		setBackground("images/menus/dialog_box.png");
 
 	align();
 }
@@ -187,6 +189,11 @@ void MenuTalker::logic() {
 	if (!visible || !npc)
 		return;
 
+	if (!inpt->usingMouse() && tablist.getCurrent() == -1) {
+		tablist.setCurrent(textbox);
+	}
+	tablist.enable_activate = !actions.empty();
+
 	tablist.logic();
 
 	if (advanceButton->checkClick() || closeButton->checkClick()) {
@@ -431,6 +438,7 @@ void MenuTalker::setNPC(NPC* _npc) {
 	if (_npc == NULL) {
 		visible = false;
 		first_interaction = false;
+		tablist.defocus();
 		return;
 	}
 
@@ -469,6 +477,7 @@ void MenuTalker::createActionButtons(int node_id) {
 	}
 
 	for (size_t i = 0; i< actions.size(); ++i) {
+		actions[i].btn->tablist_nav_align = TabList::NAV_ALIGN_LEFT;
 		textbox->addChildWidget(actions[i].btn);
 	}
 }
@@ -540,15 +549,6 @@ void MenuTalker::setupTabList() {
 	tablist.clear();
 
 	tablist.add(textbox);
-
-	if (advanceButton->enabled) {
-		tablist.add(advanceButton);
-		tablist.setCurrent(advanceButton);
-	}
-	else if (closeButton->enabled) {
-		tablist.add(closeButton);
-		tablist.setCurrent(closeButton);
-	}
 }
 
 void MenuTalker::addAction(const std::string& label, int node_id, bool is_vendor) {
