@@ -53,11 +53,10 @@ MenuVendor::MenuVendor()
 	, activetab(ItemManager::VENDOR_BUY)
 	, tip (new WidgetTooltip())
 	, npc(NULL)
-	, buyback_stock() {
-	setBackground("images/menus/vendor.png");
-
-	tabControl->setTabTitle(ItemManager::VENDOR_BUY, msg->get("Inventory"));
-	tabControl->setTabTitle(ItemManager::VENDOR_SELL, msg->get("Buyback"));
+	, buyback_stock()
+{
+	tabControl->setupTab(ItemManager::VENDOR_BUY, msg->get("Inventory"), &tablist_buy);
+	tabControl->setupTab(ItemManager::VENDOR_SELL, msg->get("Buyback"), &tablist_sell);
 
 	// Load config settings
 	FileParser infile;
@@ -105,9 +104,7 @@ MenuVendor::MenuVendor()
 	stock[ItemManager::VENDOR_BUY].initGrid(VENDOR_SLOTS, slots_area, slots_cols);
 	stock[ItemManager::VENDOR_SELL].initGrid(VENDOR_SLOTS, slots_area, slots_cols);
 
-	tablist.add(tabControl);
-	tablist_buy.setPrevTabList(&tablist);
-	tablist_sell.setPrevTabList(&tablist);
+	tablist.setNextTabList(&tablist_buy);
 
 	tablist_buy.lock();
 	tablist_sell.lock();
@@ -119,6 +116,9 @@ MenuVendor::MenuVendor()
 		tablist_sell.add(stock[ItemManager::VENDOR_SELL].slots[i]);
 	}
 
+	if (!background)
+		setBackground("images/menus/vendor.png");
+
 	align();
 }
 
@@ -155,10 +155,16 @@ void MenuVendor::logic() {
 	}
 	activetab = tabControl->getActiveTab();
 
-	if (activetab == ItemManager::VENDOR_BUY)
+	if (activetab == ItemManager::VENDOR_BUY) {
+		tablist_buy.unlock();
+		tablist_sell.lock();
 		tablist.setNextTabList(&tablist_buy);
-	else if (activetab == ItemManager::VENDOR_SELL)
+	}
+	else if (activetab == ItemManager::VENDOR_SELL) {
+		tablist_buy.lock();
+		tablist_sell.unlock();
 		tablist.setNextTabList(&tablist_sell);
+	}
 
 	if (settings->touchscreen) {
 		if (activetab == ItemManager::VENDOR_BUY && tablist_buy.getCurrent() == -1)
@@ -221,7 +227,6 @@ ItemStack MenuVendor::click(const Point& position) {
  * Cancel the dragging initiated by the clic()
  */
 void MenuVendor::itemReturn(ItemStack stack) {
-	items->playSound(stack.item);
 	stock[activetab].itemReturn(stack);
 	saveInventory();
 }
@@ -244,7 +249,7 @@ void MenuVendor::renderTooltips(const Point& position) {
 		return;
 
 	int vendor_view = (activetab == ItemManager::VENDOR_BUY) ? ItemManager::VENDOR_BUY : ItemManager::VENDOR_SELL;
-	TooltipData tip_data = stock[activetab].checkTooltip(position, &pc->stats, vendor_view);
+	TooltipData tip_data = stock[activetab].checkTooltip(position, &pc->stats, vendor_view, ItemManager::TOOLTIP_INPUT_HINT);
 	tooltipm->push(tip_data, position, TooltipData::STYLE_FLOAT);
 }
 
@@ -346,6 +351,11 @@ void MenuVendor::defocusTabLists() {
 	tablist_sell.defocus();
 }
 
+void MenuVendor::resetDrag() {
+	stock[ItemManager::VENDOR_BUY].drag_prev_slot = -1;
+	stock[ItemManager::VENDOR_SELL].drag_prev_slot = -1;
+}
+
 MenuVendor::~MenuVendor() {
 	delete closeButton;
 	delete tabControl;
