@@ -22,11 +22,9 @@ FLARE.  If not, see http://www.gnu.org/licenses/
  * class NPC
  */
 
-#include "Animation.h"
-#include "AnimationManager.h"
-#include "AnimationSet.h"
 #include "CampaignManager.h"
 #include "EntityBehavior.h"
+#include "EntityManager.h"
 #include "EventManager.h"
 #include "FileParser.h"
 #include "ItemManager.h"
@@ -56,6 +54,9 @@ NPC::NPC(const Entity& e)
 	, vendor(false)
 	, reset_buyback(true)
 	, stock()
+	, vendor_ratio_buy(0)
+	, vendor_ratio_sell(0)
+	, vendor_ratio_sell_old(0)
 	, dialog()
 {
 	stock.init(VENDOR_MAX_STOCK);
@@ -176,57 +177,57 @@ void NPC::load(const std::string& npc_id) {
 				}
 
 				if (infile.key == "name") {
-					// @ATTR name|string|NPC's name.
+					// @ATTR npc.name|string|NPC's name.
 					name = msg->get(infile.val);
 				}
 				else if (infile.key == "animations" || infile.key == "gfx") {
 					// TODO "gfx" is deprecated
 				}
 				else if (infile.key == "direction") {
-					// @ATTR direction|direction|The direction to use for this NPC's stance animation.
+					// @ATTR npc.direction|direction|The direction to use for this NPC's stance animation.
 					direction = Parse::toDirection(infile.val);
 				}
 				else if (infile.key == "show_on_minimap") {
-					// @ATTR show_on_minimap|bool|If true, this NPC will be shown on the minimap. The default is true.
+					// @ATTR npc.show_on_minimap|bool|If true, this NPC will be shown on the minimap. The default is true.
 					show_on_minimap = Parse::toBool(infile.val);
 				}
 
 				// handle talkers
 				else if (infile.key == "talker") {
-					// @ATTR talker|bool|Allows this NPC to be talked to.
+					// @ATTR npc.talker|bool|Allows this NPC to be talked to.
 					talker = Parse::toBool(infile.val);
 				}
 				else if (infile.key == "portrait") {
-					// @ATTR portrait|filename|Filename of the default portrait image.
+					// @ATTR npc.portrait|filename|Filename of the default portrait image.
 					portrait_filenames[0] = infile.val;
 				}
 
 				// handle vendors
 				else if (infile.key == "vendor") {
-					// @ATTR vendor|bool|Allows this NPC to buy/sell items.
+					// @ATTR npc.vendor|bool|Allows this NPC to buy/sell items.
 					vendor = Parse::toBool(infile.val);
 				}
 				else if (infile.key == "vendor_requires_status") {
-					// @ATTR vendor_requires_status|list(string)|The player must have these statuses in order to use this NPC as a vendor.
+					// @ATTR npc.vendor_requires_status|list(string)|The player must have these statuses in order to use this NPC as a vendor.
 					while (infile.val != "") {
 						vendor_requires_status.push_back(camp->registerStatus(Parse::popFirstString(infile.val)));
 					}
 				}
 				else if (infile.key == "vendor_requires_not_status") {
-					// @ATTR vendor_requires_not_status|list(string)|The player must not have these statuses in order to use this NPC as a vendor.
+					// @ATTR npc.vendor_requires_not_status|list(string)|The player must not have these statuses in order to use this NPC as a vendor.
 					while (infile.val != "") {
 						vendor_requires_not_status.push_back(camp->registerStatus(Parse::popFirstString(infile.val)));
 					}
 				}
 				else if (infile.key == "constant_stock") {
-					// @ATTR constant_stock|repeatable(list(item_id))|A list of items this vendor has for sale. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
+					// @ATTR npc.constant_stock|repeatable(list(item_id))|A list of items this vendor has for sale. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
 					while (infile.val != "") {
 						stack = Parse::toItemQuantityPair(Parse::popFirstString(infile.val));
 						stock.add(stack, ItemStorage::NO_SLOT);
 					}
 				}
 				else if (infile.key == "status_stock") {
-					// @ATTR status_stock|repeatable(string, list(item_id)) : Required status, Item(s)|A list of items this vendor will have for sale if the required status is met. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
+					// @ATTR npc.status_stock|repeatable(string, list(item_id)) : Required status, Item(s)|A list of items this vendor will have for sale if the required status is met. Quantity can be specified by appending ":Q" to the item_id, where Q is an integer.
 					if (camp->checkStatus(camp->registerStatus(Parse::popFirstString(infile.val)))) {
 						while (infile.val != "") {
 							stack = Parse::toItemQuantityPair(Parse::popFirstString(infile.val));
@@ -235,7 +236,7 @@ void NPC::load(const std::string& npc_id) {
 					}
 				}
 				else if (infile.key == "random_stock") {
-					// @ATTR random_stock|list(loot)|Use a loot table to add random items to the stock; either a filename or an inline definition.
+					// @ATTR npc.random_stock|list(loot)|Use a loot table to add random items to the stock; either a filename or an inline definition.
 					if (clear_random_table) {
 						random_table.clear();
 						clear_random_table = false;
@@ -245,7 +246,7 @@ void NPC::load(const std::string& npc_id) {
 					loot->parseLoot(infile.val, &random_table.back(), &random_table);
 				}
 				else if (infile.key == "random_stock_count") {
-					// @ATTR random_stock_count|int, int : Min, Max|Sets the minimum (and optionally, the maximum) amount of random items this npc can have.
+					// @ATTR npc.random_stock_count|int, int : Min, Max|Sets the minimum (and optionally, the maximum) amount of random items this npc can have.
 					random_table_count.x = Parse::popFirstInt(infile.val);
 					random_table_count.y = Parse::popFirstInt(infile.val);
 					if (random_table_count.x != 0 || random_table_count.y != 0) {
@@ -253,10 +254,22 @@ void NPC::load(const std::string& npc_id) {
 						random_table_count.y = std::max(random_table_count.y, random_table_count.x);
 					}
 				}
+				else if (infile.key == "vendor_ratio_buy") {
+					// @ATTR npc.vendor_ratio_buy|float|NPC-specific version of vendor_ratio_buy from engine/loot.txt. Uses the global setting when set to 0.
+					vendor_ratio_buy = Parse::toFloat(infile.val);
+				}
+				else if (infile.key == "vendor_ratio_sell") {
+					// @ATTR npc.vendor_ratio_sell|float|NPC-specific version of vendor_ratio_sell from engine/loot.txt. Uses the global setting when set to 0.
+					vendor_ratio_sell = Parse::toFloat(infile.val);
+				}
+				else if (infile.key == "vendor_ratio_sell_old") {
+					// @ATTR npc.vendor_ratio_sell_old|float|NPC-specific version of vendor_ratio_sell_old from engine/loot.txt. Uses the global setting when set to 0.
+					vendor_ratio_sell_old = Parse::toFloat(infile.val);
+				}
 
 				// handle vocals
 				else if (infile.key == "vox_intro") {
-					// @ATTR vox_intro|repeatable(filename)|Filename of a sound file to play when initially interacting with the NPC.
+					// @ATTR npc.vox_intro|repeatable(filename)|Filename of a sound file to play when initially interacting with the NPC.
 					loadSound(infile.val, VOX_INTRO);
 				}
 
@@ -268,7 +281,8 @@ void NPC::load(const std::string& npc_id) {
 		infile.close();
 	}
 
-	loadGraphics();
+	loadAnimations();
+	loadGraphics(); // TODO rename?
 
 	// fill inventory with items from random stock table
 	unsigned rand_count = Math::randBetween(random_table_count.x, random_table_count.y);
@@ -293,13 +307,6 @@ void NPC::load(const std::string& npc_id) {
 }
 
 void NPC::loadGraphics() {
-
-	if (stats.animations != "") {
-		anim->increaseCount(stats.animations);
-		animationSet = anim->getAnimationSet(stats.animations);
-		activeAnimation = animationSet->getAnimation("");
-	}
-
 	portraits.resize(portrait_filenames.size(), NULL);
 
 	for (size_t i = 0; i < portrait_filenames.size(); ++i) {
@@ -631,7 +638,19 @@ bool NPC::processDialog(unsigned int dialog_node, unsigned int &event_cursor) {
 			}
 		}
 		else if (dialog[dialog_node][event_cursor].type == EventComponent::NPC_TAKE_A_PARTY) {
-			stats.hero_ally = dialog[dialog_node][event_cursor].x == 0 ? false : true;
+			bool new_hero_ally = dialog[dialog_node][event_cursor].x == 0 ? false : true;
+			if (stats.hero_ally != new_hero_ally) {
+				stats.hero_ally = new_hero_ally;
+				if (stats.hero_ally) {
+					entitym->entities.push_back(this);
+				}
+				else {
+					for (size_t i = entitym->entities.size(); i > 0; --i) {
+						if (entitym->entities[i-1] == this)
+							entitym->entities.erase(entitym->entities.begin() + i - 1);
+					}
+				}
+			}
 		}
 		else if (dialog[dialog_node][event_cursor].type == EventComponent::NONE) {
 			// conversation ends
@@ -666,15 +685,10 @@ bool NPC::isDialogType(const int &event_type) {
 }
 
 NPC::~NPC() {
-
 	for (size_t i = 0; i < portraits.size(); ++i) {
 		delete portraits[i];
 	}
 
-	if (stats.animations != "") {
-		anim->decreaseCount(stats.animations);
-	}
-
 	while (!vox_intro.empty()) {
 		snd->unload(vox_intro.back());
 		vox_intro.pop_back();
