@@ -30,6 +30,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EntityManager.h"
 #include "EngineSettings.h"
 #include "EventManager.h"
+#include "FogOfWar.h"
 #include "ItemManager.h"
 #include "MapRenderer.h"
 #include "NPC.h"
@@ -48,7 +49,13 @@ NPCManager::NPCManager()
 
 void NPCManager::addRenders(std::vector<Renderable> &r) {
 	for (unsigned i=0; i<npcs.size(); i++) {
-		r.push_back(npcs[i]->getRender());
+		if (mapr->fogofwar > FogOfWar::TYPE_MINIMAP) {
+			float delta = Utils::calcDist(pc->stats.pos, npcs[i]->stats.pos);
+			if (delta > fow->mask_radius-1.0) {
+				continue;
+			}
+		}
+		npcs[i]->addRenders(r);
 	}
 }
 
@@ -61,7 +68,7 @@ void NPCManager::handleNewMap() {
 
 	// remove existing NPCs
 	for (unsigned i=0; i<npcs.size(); i++) {
-		if (npcs[i]->stats.hero_ally) {
+		if (npcs[i]->stats.hero_ally && !npcs[i]->stats.corpse && npcs[i]->stats.cur_state != StatBlock::ENTITY_DEAD && npcs[i]->stats.cur_state != StatBlock::ENTITY_CRITDEAD && npcs[i]->stats.speed > 0.0f) {
 			allies[npcs[i]->filename] = npcs[i];
 		}
 		else {
@@ -72,21 +79,11 @@ void NPCManager::handleNewMap() {
 	npcs.clear();
 
 	// read the queued NPCs in the map file
-	while (!mapr->npcs.empty()) {
-		mn = mapr->npcs.front();
-		mapr->npcs.pop();
+	while (!mapr->map_npcs.empty()) {
+		mn = mapr->map_npcs.front();
+		mapr->map_npcs.pop();
 
-		bool status_reqs_met = true;
-		//if the status requirements arent met, dont load the enemy
-		for (unsigned i = 0; i < mn.requires_status.size(); ++i)
-			if (!camp->checkStatus(mn.requires_status[i]))
-				status_reqs_met = false;
-
-		for (unsigned i = 0; i < mn.requires_not_status.size(); ++i)
-			if (camp->checkStatus(mn.requires_not_status[i]))
-				status_reqs_met = false;
-
-		if(!status_reqs_met)
+		if (!camp->checkRequirementsInVector(mn.requirements))
 			continue;
 
 		// ally npc that was moved from another map should not be loaded once again
@@ -94,7 +91,15 @@ void NPCManager::handleNewMap() {
 			continue;
 		}
 
-		NPC *npc = new NPC(*entitym->getEntityPrototype(mn.id));
+		NPC *npc;
+		Entity *entity = entitym->getEntityPrototype(mn.id);
+		if (entity) {
+			npc = new NPC(*entity);
+			delete entity;
+		}
+		else {
+			npc = new NPC(Entity());
+		}
 
 		npc->load(mn.id);
 
@@ -102,6 +107,8 @@ void NPCManager::handleNewMap() {
 		npc->stats.pos.y = mn.pos.y;
 		npc->stats.hero_ally = false;
 		npc->stats.npc = true;
+		if (mn.direction != -1)
+			npc->stats.direction = static_cast<unsigned char>(mn.direction);
 
 		// npc->stock.sort();
 		npcs.push_back(npc);
@@ -122,6 +129,8 @@ void NPCManager::handleNewMap() {
 		createMapEvent(*npc, npcs.size());
 
 		mapr->collider.block(npc->stats.pos.x, npc->stats.pos.y, !MapCollision::IS_ALLY);
+
+		entitym->entities.push_back(npc);
 	}
 
 }
@@ -150,24 +159,19 @@ void NPCManager::createMapEvent(const NPC& npc, size_t _npcs) {
 	ec.s = npc.name;
 	ev.components.push_back(ec);
 
-	// The hitbox for hovering/clicking on an npc is based on their first frame of animation
-	// This might cause some undesired behavior for npcs that have packed animations and a lot of variation
-	// However, it is sufficient for all of our current game data (fantasycore, no-name mod, polymorphable)
-	if (npc.activeAnimation) {
-		Renderable ren = npc.activeAnimation->getCurrentFrame(npc.direction);
-		ec.type = EventComponent::NPC_HOTSPOT;
-		ec.x = static_cast<int>(npc.stats.pos.x);
-		ec.y = static_cast<int>(npc.stats.pos.y);
-		ec.z = ren.offset.x;
-		ec.a = ren.offset.y;
-		ec.b = ren.src.w;
-		ec.c = ren.src.h;
-		ev.components.push_back(ec);
-		ev.type = npc.filename;
-	}
-	else {
-		Utils::logError("NPCManager: Unable to set click hotspot for '%s' due to lack of animation.", npc.filename.c_str());
+	ec.type = EventComponent::NPC_HOTSPOT;
+	ec.x = static_cast<int>(npc.stats.pos.x);
+	ec.y = static_cast<int>(npc.stats.pos.y);
+	ec.id = npcs.size();
+	for (size_t i = 0; i < npcs.size(); ++i) {
+		if (&npc == npcs[i]) {
+			ec.id = i;
+			break;
+		}
 	}
+	ev.components.push_back(ec);
+
+	ev.type = npc.filename;
 
 	mapr->events.push_back(ev);
 }
@@ -184,10 +188,19 @@ int NPCManager::getID(const std::string& npcName) {
 	}
 
 	// could not find NPC, try loading it here
-	NPC *n = new NPC(*entitym->getEntityPrototype(npcName));
-	if (n) {
-		n->load(npcName);
-		npcs.push_back(n);
+	NPC *npc;
+	Entity *entity = entitym->getEntityPrototype(npcName);
+	if (entity) {
+		npc = new NPC(*entity);
+		delete entity;
+	}
+	else {
+		npc = new NPC(Entity());
+	}
+
+	if (npc) {
+		npc->load(npcName);
+		npcs.push_back(npc);
 		return static_cast<int>(npcs.size()-1);
 	}
 
@@ -205,15 +218,7 @@ Entity* NPCManager::npcFocus(const Point& mouse, const FPoint& cam, bool alive_o
 			continue;
 		}
 
-		p = Utils::mapToScreen(npcs[i]->stats.pos.x, npcs[i]->stats.pos.y, cam.x, cam.y);
-
-		Renderable ren = npcs[i]->getRender();
-		r.w = ren.src.w;
-		r.h = ren.src.h;
-		r.x = p.x - ren.offset.x;
-		r.y = p.y - ren.offset.y;
-
-		if (Utils::isWithinRect(r, mouse)) {
+		if (Utils::isWithinRect(npcs[i]->getRenderBounds(cam), mouse)) {
 			return npcs[i];
 		}
 	}
