@@ -967,13 +967,16 @@ bool PowerManager::isValidEffect(const std::string& type) {
 	}
 
 	for (unsigned i=0; i<eset->elements.list.size(); ++i) {
-		if (type == eset->elements.list[i].id + "_resist")
+		if (type == eset->elements.list[i].resist_id)
 			return true;
 	}
 
 	if (getEffectDef(type) != NULL)
 		return true;
 
+	if (Effect::getTypeFromString(type) != Effect::NONE)
+		return true;
+
 	return false;
 }
 
@@ -1794,7 +1797,7 @@ bool PowerManager::checkRequiredItems(const Power &pow, const StatBlock *src_sta
 	for (size_t i = 0; i < pow.required_items.size(); ++i) {
 		if (pow.required_items[i].id > 0) {
 			if (pow.required_items[i].equipped) {
-				if (!menu->inv->inventory[MenuInventory::EQUIPMENT].contain(pow.required_items[i].id, 1)) {
+				if (!menu->inv->equipmentContain(pow.required_items[i].id, 1)) {
 					return false;
 				}
 			}
