@@ -274,9 +274,9 @@ int SDLSoftwareRenderDevice::createContextInternal() {
 			renderer = SDL_CreateRenderer(window, -1, r_flags);
 			if (renderer) {
 				if (settings->texture_filter && !eset->resolutions.ignore_texture_filter)
-					SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "1");
+					SDL_SetHintWithPriority(SDL_HINT_RENDER_SCALE_QUALITY, "1", SDL_HINT_OVERRIDE);
 				else
-					SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "0");
+					SDL_SetHintWithPriority(SDL_HINT_RENDER_SCALE_QUALITY, "0", SDL_HINT_OVERRIDE);
 
 				windowResize();
 			}
@@ -302,6 +302,10 @@ int SDLSoftwareRenderDevice::createContextInternal() {
 
 			Utils::logInfo("RenderDevice: Fullscreen=%d, Hardware surfaces=%d, Vsync=%d, Texture Filter=%d", fullscreen, hwsurface, vsync, texture_filter);
 
+			SDL_RendererInfo renderer_info;
+			SDL_GetRendererInfo(renderer, &renderer_info);
+			Utils::logInfo("RenderDevice: Renderer driver is '%s'.", renderer_info.name);
+
 #if SDL_VERSION_ATLEAST(2, 0, 4)
 			SDL_GetDisplayDPI(0, &ddpi, 0, 0);
 			Utils::logInfo("RenderDevice: Display DPI is %f", ddpi);
@@ -486,10 +490,10 @@ void SDLSoftwareRenderDevice::drawRectangle(const Point& p0, const Point& p1, co
 	if (SDL_MUSTLOCK(screen)) {
 		SDL_LockSurface(screen);
 	}
-	this->drawLine(p0.x, p0.y, p1.x, p0.y, color);
-	this->drawLine(p1.x, p0.y, p1.x, p1.y, color);
-	this->drawLine(p0.x, p0.y, p0.x, p1.y, color);
-	this->drawLine(p0.x, p1.y, p1.x, p1.y, color);
+	drawLine(p0.x, p0.y, p1.x, p0.y, color);
+	drawLine(p1.x, p0.y, p1.x, p1.y, color);
+	drawLine(p0.x, p0.y, p0.x, p1.y, color);
+	drawLine(p0.x, p1.y, p1.x+1, p1.y, color);
 	if (SDL_MUSTLOCK(screen)) {
 		SDL_UnlockSurface(screen);
 	}
@@ -569,7 +573,7 @@ Image *SDLSoftwareRenderDevice::createImage(int width, int height) {
 		return NULL;
 
 	Uint32 rmask, gmask, bmask, amask;
-	setSDL_RGBA(&rmask, &gmask, &bmask, &amask);
+	Utils::setSDL_RGBA(&rmask, &gmask, &bmask, &amask);
 
 	image->surface = SDL_CreateRGBSurface(0, width, height, BITS_PER_PIXEL, rmask, gmask, bmask, amask);
 
@@ -643,20 +647,6 @@ Image *SDLSoftwareRenderDevice::loadImage(const std::string& filename, int error
 	return image;
 }
 
-void SDLSoftwareRenderDevice::setSDL_RGBA(Uint32 *rmask, Uint32 *gmask, Uint32 *bmask, Uint32 *amask) {
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-	*rmask = 0xff000000;
-	*gmask = 0x00ff0000;
-	*bmask = 0x0000ff00;
-	*amask = 0x000000ff;
-#else
-	*rmask = 0x000000ff;
-	*gmask = 0x0000ff00;
-	*bmask = 0x00ff0000;
-	*amask = 0xff000000;
-#endif
-}
-
 void SDLSoftwareRenderDevice::getWindowSize(short unsigned *screen_w, short unsigned *screen_h) {
 	int w,h;
 	SDL_GetWindowSize(window, &w, &h);
