@@ -33,6 +33,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "CommonIncludes.h"
 #include "EngineSettings.h"
 #include "FileParser.h"
+#include "FogOfWar.h"
 #include "GameStatePlay.h"
 #include "MapRenderer.h"
 #include "Menu.h"
@@ -124,6 +125,9 @@ void SaveLoad::saveGame() {
 		outfile << "equipped_quantity=" << menu->inv->inventory[MenuInventory::EQUIPMENT].getQuantities() << "\n";
 		outfile << "equipped=" << menu->inv->inventory[MenuInventory::EQUIPMENT].getItems() << "\n";
 
+		// active equipped set
+		outfile << "active_equipment_set=" << menu->inv->active_equipment_set << "\n";
+
 		// carried items
 		outfile << "carried_quantity=" << menu->inv->inventory[MenuInventory::CARRIED].getQuantities() << "\n";
 		outfile << "carried=" << menu->inv->inventory[MenuInventory::CARRIED].getItems() << "\n";
@@ -244,6 +248,41 @@ void SaveLoad::saveGame() {
 		}
 	}
 
+	// Save fow dark layer
+	if (mapr->fogofwar && mapr->save_fogofwar && !mapr->getFilename().empty() && fow->dark_layer_id < mapr->layernames.size()) {
+		ss.str("");
+		ss << settings->path_user << "saves/" << eset->misc.save_prefix << "/" << game_slot << "/fow/" << Utils::hashString(mapr->getFilename()) << ".txt";
+
+		outfile.open(Filesystem::convertSlashes(ss.str()).c_str(), std::ios::out);
+
+		if (outfile.is_open()) {
+			outfile << "# " << mapr->getFilename() << std::endl;
+			outfile << "[layer]" << std::endl;
+			outfile << "type=" << mapr->layernames[fow->dark_layer_id] << std::endl;
+			outfile << "data=" << std::endl;
+
+			std::string layer = "";
+			for (int line = 0; line < mapr->h; line++) {
+				std::stringstream map_row;
+				for (int tile = 0; tile < mapr->w; tile++) {
+					unsigned short val = mapr->layers[fow->dark_layer_id][tile][line];
+					map_row << val << ",";
+				}
+				layer += map_row.str();
+				layer += '\n';
+			}
+			layer.erase(layer.end()-2, layer.end());
+			layer += '\n';
+			outfile << layer << std::endl;
+
+			if (outfile.bad()) Utils::logError("SaveLoad: Unable to save map data. No write access or disk is full!");
+			outfile.close();
+			outfile.clear();
+
+			platform.FSCommit();
+		}
+	}
+
 	settings->prev_save_slot = game_slot-1;
 
 	// display a log message saying that we saved the game
@@ -309,6 +348,9 @@ void SaveLoad::loadGame() {
 			else if (infile.key == "equipped_quantity") {
 				menu->inv->inventory[MenuInventory::EQUIPMENT].setQuantities(infile.val);
 			}
+			else if (infile.key == "active_equipment_set") {
+				menu->inv->applyEquipmentSet(Parse::toInt(infile.val));
+			}
 			else if (infile.key == "carried") {
 				menu->inv->inventory[MenuInventory::CARRIED].setItems(infile.val);
 			}
@@ -555,4 +597,3 @@ void SaveLoad::loadPowerTree() {
 	// fall back to the default power tree
 	menu->pow->loadPowerTree("powers/trees/default.txt");
 }
-
