@@ -28,9 +28,9 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "Avatar.h"
 #include "CampaignManager.h"
 #include "CombatText.h"
+#include "EngineSettings.h"
 #include "Entity.h"
 #include "EntityManager.h"
-#include "EngineSettings.h"
 #include "FileParser.h"
 #include "Hazard.h"
 #include "LootManager.h"
@@ -38,6 +38,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "MapRenderer.h"
 #include "MenuPowers.h"
 #include "MessageEngine.h"
+#include "ModManager.h"
 #include "PowerManager.h"
 #include "Settings.h"
 #include "SharedGameResources.h"
@@ -57,6 +58,10 @@ const float StatBlock::DIRECTION_DELTA_X[8] =   {-1, -1, -1,  0,  1,  1,  1,  0}
 const float StatBlock::DIRECTION_DELTA_Y[8] =   { 1,  0, -1, -1, -1,  0,  1,  1};
 const float StatBlock::SPEED_MULTIPLIER[8] = { static_cast<float>(1.0/M_SQRT2), 1.0f, static_cast<float>(1.0/M_SQRT2), 1.0f, static_cast<float>(1.0/M_SQRT2), 1.0f, static_cast<float>(1.0/M_SQRT2), 1.0f};
 
+size_t StatBlock::getFullStatCount() {
+	return Stats::COUNT + eset->damage_types.count + eset->elements.list.size();
+}
+
 StatBlock::StatBlock()
 	: statsLoaded(false)
 	, alive(true)
@@ -92,10 +97,10 @@ StatBlock::StatBlock()
 	, check_title(false)
 	, stat_points_per_level(1)
 	, power_points_per_level(1)
-	, starting(Stats::COUNT + eset->damage_types.count, 0)
-	, base(Stats::COUNT + eset->damage_types.count, 0)
-	, current(Stats::COUNT + eset->damage_types.count, 0)
-	, per_level(Stats::COUNT + eset->damage_types.count, 0)
+	, starting(getFullStatCount(), 0)
+	, base(getFullStatCount(), 0)
+	, current(getFullStatCount(), 0)
+	, per_level(getFullStatCount(), 0)
 	, character_class("")
 	, character_subclass("")
 	, hp(0)
@@ -109,8 +114,6 @@ StatBlock::StatBlock()
 	, absorb_max_add(0)
 	, speed(0.1f)
 	, charge_speed(0.0f)
-	, vulnerable(eset->elements.list.size(), 100)
-	, vulnerable_base(eset->elements.list.size(), 100)
 	, transform_duration(0)
 	, transform_duration_total(0)
 	, manual_untransform(false)
@@ -188,6 +191,10 @@ StatBlock::StatBlock()
 	, summons()
 	, summoner(NULL)
 	, abort_npc_interact(false)
+	, layer_reference_order()
+	, layer_def(8, std::vector<unsigned>())
+	, animation_slots()
+	, critdie_enabled(false)
 {
 	primary.resize(eset->primary_stats.list.size(), 0);
 	primary_starting.resize(eset->primary_stats.list.size(), 0);
@@ -195,7 +202,7 @@ StatBlock::StatBlock()
 	per_primary.resize(eset->primary_stats.list.size());
 
 	for (size_t i = 0; i < per_primary.size(); ++i) {
-		per_primary[i].resize(Stats::COUNT + eset->damage_types.count, 0);
+		per_primary[i].resize(getFullStatCount(), 0);
 	}
 
 	cooldown.reset(Timer::END);
@@ -208,7 +215,7 @@ StatBlock::~StatBlock() {
 }
 
 bool StatBlock::loadCoreStat(FileParser *infile) {
-	// @CLASS StatBlock: Core stats|Description of engine/stats.txt and enemies in enemies/
+	// @CLASS StatBlock: Core stats|Description of engine/stats.txt, enemies/..., and npcs/...
 
 	if (infile->key == "speed") {
 		// @ATTR speed|float|Movement speed
@@ -228,7 +235,7 @@ bool StatBlock::loadCoreStat(FileParser *infile) {
 		return true;
 	}
 	else if (infile->key == "stat") {
-		// @ATTR stat|string, int : Stat name, Value|The starting value for this stat.
+		// @ATTR stat|stat_id, int : Stat ID, Value|The starting value for this stat.
 		std::string stat = Parse::popFirstString(infile->val);
 		int value = Parse::popFirstInt(infile->val);
 
@@ -249,9 +256,16 @@ bool StatBlock::loadCoreStat(FileParser *infile) {
 				return true;
 			}
 		}
+
+		for (size_t i = 0; i < eset->elements.list.size(); ++i) {
+			if (eset->elements.list[i].resist_id == stat) {
+				starting[Stats::COUNT + eset->damage_types.count + i] = value;
+				return true;
+			}
+		}
 	}
 	else if (infile->key == "stat_per_level") {
-		// @ATTR stat_per_level|predefined_string, int : Stat name, Value|The value for this stat added per level.
+		// @ATTR stat_per_level|stat_id, int : Stat ID, Value|The value for this stat added per level.
 		std::string stat = Parse::popFirstString(infile->val);
 		int value = Parse::popFirstInt(infile->val);
 
@@ -272,9 +286,16 @@ bool StatBlock::loadCoreStat(FileParser *infile) {
 				return true;
 			}
 		}
+
+		for (size_t i = 0; i < eset->elements.list.size(); ++i) {
+			if (eset->elements.list[i].resist_id == stat) {
+				per_level[Stats::COUNT + eset->damage_types.count + i] = value;
+				return true;
+			}
+		}
 	}
 	else if (infile->key == "stat_per_primary") {
-		// @ATTR stat_per_primary|predefined_string, predefined_string, int : Primary Stat, Stat name, Value|The value for this stat added for every point allocated to this primary stat.
+		// @ATTR stat_per_primary|predefined_string, stat_id, int : Primary Stat, Stat ID, Value|The value for this stat added for every point allocated to this primary stat.
 		std::string prim_stat = Parse::popFirstString(infile->val);
 		size_t prim_stat_index = eset->primary_stats.getIndexByID(prim_stat);
 		if (prim_stat_index == eset->primary_stats.list.size()) {
@@ -302,15 +323,24 @@ bool StatBlock::loadCoreStat(FileParser *infile) {
 				return true;
 			}
 		}
+
+		for (size_t i = 0; i < eset->elements.list.size(); ++i) {
+			if (eset->elements.list[i].resist_id == stat) {
+				per_primary[prim_stat_index][Stats::COUNT + eset->damage_types.count + i] = value;
+				return true;
+			}
+		}
 	}
 	else if (infile->key == "vulnerable") {
-		// @ATTR vulnerable|predefined_string, int : Element, Value|Percentage weakness to this element.
+		// @ATTR vulnerable|predefined_string, int : Element, Value|(Deprecated in v1.12.91; use a '..._resist' value with 'stat' instead) Percentage weakness to this element.
 		std::string element = Parse::popFirstString(infile->val);
-		int value = Parse::popFirstInt(infile->val);
+		int value = (Parse::popFirstInt(infile->val) * -1) + 100;
+
+		infile->error("StatBlock: 'vulnerable' is deprecated. Use 'stat=%s_resist,%d' instead.", element.c_str(), value);
 
 		for (unsigned int i=0; i<eset->elements.list.size(); i++) {
 			if (element == eset->elements.list[i].id) {
-				vulnerable[i] = vulnerable_base[i] = value;
+				starting[Stats::COUNT + eset->damage_types.count + i] = value;
 				return true;
 			}
 		}
@@ -346,9 +376,9 @@ bool StatBlock::loadCoreStat(FileParser *infile) {
  * Set paths for sound effects
  */
 bool StatBlock::loadSfxStat(FileParser *infile) {
-	// @CLASS StatBlock: Sound effects|Description of heroes in engine/avatar/ and enemies in enemies/
+	// @CLASS StatBlock: Sound effects|Description of sound effect properties in engine/stats.txt, enemies/..., and npcs/...
 
-	if (infile->new_section) {
+	if (infile->new_section && (infile->section.empty() || infile->section == "stats")) {
 		sfx_attack.clear();
 		sfx_hit.clear();
 		sfx_die.clear();
@@ -379,45 +409,126 @@ bool StatBlock::loadSfxStat(FileParser *infile) {
 				sfx_attack[found_index].second.push_back(filename);
 			}
 		}
+
+		return true;
 	}
 	else if (infile->key == "sfx_hit") {
 		// @ATTR sfx_hit|repeatable(filename)|Filename of sound effect for being hit.
 		if (std::find(sfx_hit.begin(), sfx_hit.end(), infile->val) == sfx_hit.end()) {
 			sfx_hit.push_back(infile->val);
 		}
+
+		return true;
 	}
 	else if (infile->key == "sfx_die") {
 		// @ATTR sfx_die|repeatable(filename)|Filename of sound effect for dying.
 		if (std::find(sfx_die.begin(), sfx_die.end(), infile->val) == sfx_die.end()) {
 			sfx_die.push_back(infile->val);
 		}
+
+		return true;
 	}
 	else if (infile->key == "sfx_critdie") {
 		// @ATTR sfx_critdie|repeatable(filename)|Filename of sound effect for dying to a critical hit.
 		if (std::find(sfx_critdie.begin(), sfx_critdie.end(), infile->val) == sfx_critdie.end()) {
 			sfx_critdie.push_back(infile->val);
 		}
+
+		return true;
 	}
 	else if (infile->key == "sfx_block") {
 		// @ATTR sfx_block|repeatable(filename)|Filename of sound effect for blocking an incoming hit.
 		if (std::find(sfx_block.begin(), sfx_block.end(), infile->val) == sfx_block.end()) {
 			sfx_block.push_back(infile->val);
 		}
+
+		return true;
 	}
 	else if (infile->key == "sfx_levelup") {
 		// @ATTR sfx_levelup|filename|Filename of sound effect for leveling up.
 		sfx_levelup = infile->val;
+
+		return true;
 	}
 	else if (infile->key == "sfx_lowhp") {
 		// @ATTR sfx_lowhp|filename, bool: Sound file, loop|Filename of sound effect for low health warning. Optionally, it can be looped.
 		sfx_lowhp = Parse::popFirstString(infile->val);
 		if (infile->val != "") sfx_lowhp_loop = Parse::toBool(infile->val);
-	}
-	else {
-		return false;
+
+		return true;
 	}
 
-	return true;
+	return false;
+}
+
+bool StatBlock::loadRenderLayerStat(FileParser *infile) {
+	// @CLASS StatBlock: Render layers|Description of 'render_layers' section in engine/stats.txt, enemies/..., and npcs/...
+
+	if (infile->section == "render_layers") {
+		if (infile->new_section) {
+			layer_def = std::vector<std::vector<unsigned> >(8, std::vector<unsigned>());
+			layer_reference_order = std::vector<std::string>();
+			animation_slots.clear();
+		}
+
+		if (infile->key == "layer") {
+			// @ATTR render_layers.layer|direction, list(string) : Direction, Layer name(s)|Defines the layer order of slots for a given direction.
+			unsigned dir = Parse::toDirection(Parse::popFirstString(infile->val));
+			if (dir>7) {
+				infile->error("StatBlock: Render layer direction must be in range [0,7]");
+				Utils::logErrorDialog("StatBlock: Render layer direction must be in range [0,7]");
+				mods->resetModConfig();
+				Utils::Exit(1);
+			}
+
+			std::string layer = Parse::popFirstString(infile->val);
+			while (layer != "") {
+				// check if already in layer_reference:
+				unsigned ref_pos;
+				for (ref_pos = 0; ref_pos < layer_reference_order.size(); ++ref_pos)
+					if (layer == layer_reference_order[ref_pos])
+						break;
+				if (ref_pos == layer_reference_order.size())
+					layer_reference_order.push_back(layer);
+				layer_def[dir].push_back(ref_pos);
+
+				animation_slots[layer] = "";
+
+				layer = Parse::popFirstString(infile->val);
+			}
+
+			// There are the positions of the items relative to layer_reference_order
+			// so if layer_reference_order=main,body,head,off
+			// and we got a layer=3,off,body,head,main
+			// then the layer_def[3] looks like (3,1,2,0)
+
+			return true;
+		}
+	}
+
+	return false;
+}
+
+bool StatBlock::loadAnimationSlotStat(FileParser *infile) {
+	// @CLASS StatBlock: Animation slots|Description of 'animation_slots' section in enemies/... and npcs/...
+	if (infile->section == "animation_slots") {
+		if (infile->key == "slot") {
+			// @ATTR animation_slots.slot|string, filename : Slot name, Animation filename|Assigns an animation to one of the slots defined in the render_layers section.
+			std::string slot_id = Parse::popFirstString(infile->val);
+			std::string slot_filename = Parse::popFirstString(infile->val);
+
+			std::map<std::string, std::string>::iterator it;
+			it = animation_slots.find(slot_id);
+			if (it != animation_slots.end())
+				it->second = slot_filename;
+			else
+				infile->error("StatBlock: Slot %s does not having a matching render layer", slot_id.c_str());
+
+			return true;
+		}
+	}
+
+	return false;
 }
 
 bool StatBlock::isNPCStat(FileParser *infile) {
@@ -457,14 +568,14 @@ void StatBlock::load(const std::string& filename) {
 	bool flee_range_defined = false;
 
 	while (infile.next()) {
-		if (infile.new_section) {
+		if (infile.new_section && (infile.section.empty() || infile.section == "stats")) {
 			// APPENDed file
 			clear_loot = true;
 		}
 
 		int num = Parse::toInt(infile.val);
 		float fnum = Parse::toFloat(infile.val);
-		bool valid = loadCoreStat(&infile) || loadSfxStat(&infile) || isNPCStat(&infile);
+		bool valid = loadCoreStat(&infile) || loadSfxStat(&infile) || loadRenderLayerStat(&infile) || loadAnimationSlotStat(&infile) || isNPCStat(&infile);
 
 		// @ATTR name|string|Name
 		if (infile.key == "name") name = msg->get(infile.val);
@@ -690,7 +801,7 @@ void StatBlock::takeDamage(int dmg, bool crit, int source_type) {
 				loot->addEnemyLoot(this);
 			}
 
-			if (crit)
+			if (crit && critdie_enabled)
 				cur_state = StatBlock::ENTITY_CRITDEAD;
 			else
 				cur_state = StatBlock::ENTITY_DEAD;
@@ -739,7 +850,7 @@ void StatBlock::calcBase() {
 	const int lev0 = std::max(level - 1, 0);
 
 	if (per_primary.empty()) {
-		for (size_t i = 0; i < Stats::COUNT + eset->damage_types.count; ++i) {
+		for (size_t i = 0; i < getFullStatCount(); ++i) {
 			base[i] = starting[i] + (lev0 * per_level[i]);
 		}
 	}
@@ -747,7 +858,7 @@ void StatBlock::calcBase() {
 		for (size_t j = 0; j < per_primary.size(); ++j) {
 			const int current_primary = std::max(get_primary(j) - 1, 0);
 			const std::vector<int>& per_primary_vec = per_primary[j];
-			for (size_t i = 0; i < Stats::COUNT + eset->damage_types.count; ++i) {
+			for (size_t i = 0; i < getFullStatCount(); ++i) {
 				if (j==0)
 					base[i] = starting[i] + (lev0 * per_level[i]);
 				base[i] += (current_primary * per_primary_vec[i]);
@@ -792,14 +903,10 @@ void StatBlock::applyEffects() {
 
 	calcBase();
 
-	for (size_t i = 0; i < Stats::COUNT + eset->damage_types.count; ++i) {
+	for (size_t i = 0; i < getFullStatCount(); ++i) {
 		current[i] = base[i] + effects.bonus[i];
 	}
 
-	for (size_t i = 0; i < effects.bonus_resist.size(); ++i) {
-		vulnerable[i] = vulnerable_base[i] - effects.bonus_resist[i];
-	}
-
 	current[Stats::HP_MAX] += (current[Stats::HP_MAX] * current[Stats::HP_PERCENT]) / 100;
 	current[Stats::MP_MAX] += (current[Stats::MP_MAX] * current[Stats::MP_PERCENT]) / 100;
 
@@ -874,7 +981,7 @@ void StatBlock::logic() {
 		mp_f = static_cast<float>(mp);
 
 	// HP regen
-	if (hp < get(Stats::HP_MAX) && hp > 0) {
+	if (hp <= get(Stats::HP_MAX) && hp > 0) {
 		float hp_regen_per_frame;
 		if (!in_combat && !hero_ally && !hero && pc->stats.alive) {
 			// enemies heal rapidly (full heal in 5 seconds) while not in combat
@@ -884,14 +991,14 @@ void StatBlock::logic() {
 			hp_regen_per_frame = static_cast<float>(get(Stats::HP_REGEN)) / 60.f / settings->max_frames_per_sec;
 		}
 		hp_f += hp_regen_per_frame;
-		hp = std::min(static_cast<int>(hp_f), get(Stats::HP_MAX));
+		hp = std::max(0, std::min(static_cast<int>(hp_f), get(Stats::HP_MAX)));
 	}
 
 	// MP regen
-	if (mp < get(Stats::MP_MAX) && hp > 0) {
+	if (mp <= get(Stats::MP_MAX) && hp > 0) {
 		float mp_regen_per_frame = static_cast<float>(get(Stats::MP_REGEN)) / 60.f / settings->max_frames_per_sec;
 		mp_f += mp_regen_per_frame;
-		mp = std::min(static_cast<int>(mp_f), get(Stats::MP_MAX));
+		mp = std::max(0, std::min(static_cast<int>(mp_f), get(Stats::MP_MAX)));
 	}
 
 	// handle buff/debuff durations
@@ -1041,7 +1148,7 @@ void StatBlock::loadHeroStats() {
 		while (infile.next()) {
 			int value = Parse::toInt(infile.val);
 
-			bool valid = loadCoreStat(&infile);
+			bool valid = loadCoreStat(&infile) || loadRenderLayerStat(&infile);
 
 			if (infile.key == "max_points_per_stat") {
 				// @ATTR max_points_per_stat|int|Maximum points for each primary stat.
@@ -1264,3 +1371,7 @@ void StatBlock::setPowerCooldown(PowerID power_id, int power_cooldown) {
 		}
 	}
 }
+
+int StatBlock::getResist(size_t resist_type) const {
+	return current[Stats::COUNT + eset->damage_types.count + resist_type];
+}
