@@ -95,6 +95,8 @@ public:
 	static const float DIRECTION_DELTA_Y[8];
 	static const float SPEED_MULTIPLIER[8];
 
+	static size_t getFullStatCount();
+
 	StatBlock();
 	~StatBlock();
 
@@ -116,6 +118,9 @@ public:
 	int getPowerCooldown(PowerID power_id);
 	void setPowerCooldown(PowerID power_id, int power_cooldown);
 
+	bool loadRenderLayerStat(FileParser *infile);
+	bool loadAnimationSlotStat(FileParser *infile);
+
 	bool alive;
 	bool corpse; // creature is dead and done animating
 	Timer corpse_timer;
@@ -175,6 +180,7 @@ public:
 	int getDamageMax(size_t dmg_type) const {
 		return current[Stats::COUNT + (dmg_type * 2) + 1];
 	}
+	int getResist(size_t resist_type) const;
 
 	// additional values to base stats, given by items
 	std::vector<int> primary_additional;
@@ -209,8 +215,6 @@ public:
 	float charge_speed;
 
 	std::set<std::string> equip_flags;
-	std::vector<int> vulnerable;
-	std::vector<int> vulnerable_base;
 
 	// buff and debuff stats
 	int transform_duration;
@@ -329,10 +333,16 @@ public:
 
 	std::vector<PowerID> power_filter;
 
-	std::vector<StatusID> invincible_requires_status;
-	std::vector<StatusID> invincible_requires_not_status;
+	std::vector<EventComponent> invincible_requirements;
 
 	bool abort_npc_interact;
+
+	std::vector<std::string> layer_reference_order;
+	std::vector<std::vector<unsigned> > layer_def;
+
+	std::map<std::string, std::string> animation_slots;
+
+	bool critdie_enabled;
 };
 
 #endif
