@@ -82,6 +82,11 @@ FPoint::FPoint(Point _p)
 	, y(static_cast<float>(_p.y))
 {}
 
+FPoint::FPoint(int _x, int _y)
+	: x(static_cast<float>(_x))
+	, y(static_cast<float>(_y))
+{}
+
 void FPoint::align() {
 	// this rounds the float values to the nearest multiple of 1/(2^4)
 	// 1/(2^4) was chosen because it's a "nice" floating point number, removing 99% of rounding errors
@@ -330,7 +335,7 @@ unsigned char Utils::calcDirection(float x0, float y0, float x1, float y1) {
 	float val = theta / (static_cast<float>(M_PI)/4);
 	int dir = static_cast<int>(((val < 0) ? ceilf(val-0.5f) : floorf(val+0.5f)) + 4);
 	dir = (dir + 1) % 8;
-	if (dir >= 0 && dir < 8)
+	if (dir >= 0)
 		return static_cast<unsigned char>(dir);
 	else
 		return 0;
@@ -563,6 +568,9 @@ void Utils::createSaveDir(int slot) {
 
 	ss << slot;
 	Filesystem::createDir(ss.str());
+
+	ss << "/fow/";
+	Filesystem::createDir(ss.str());
 }
 
 void Utils::removeSaveDir(int slot) {
@@ -664,9 +672,6 @@ std::string Utils::substituteVarsInString(const std::string &_s, Avatar* avatar)
 		else if (var == "${INPUT_ATTACK}") {
 			s.replace(begin, var_len, inpt->getAttackString());
 		}
-		else if (var == "${INPUT_CONTINUE}") {
-			s.replace(begin, var_len, inpt->getContinueString());
-		}
 		else {
 			logError("'%s' is not a valid string variable name.", var.c_str());
 			// strip the brackets from the variable
@@ -855,3 +860,17 @@ void Utils::lockFileCheck() {
 	lockFileWrite(1);
 }
 
+void Utils::setSDL_RGBA(Uint32 *rmask, Uint32 *gmask, Uint32 *bmask, Uint32 *amask) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+	*rmask = 0xff000000;
+	*gmask = 0x00ff0000;
+	*bmask = 0x0000ff00;
+	*amask = 0x000000ff;
+#else
+	*rmask = 0x000000ff;
+	*gmask = 0x0000ff00;
+	*bmask = 0x00ff0000;
+	*amask = 0xff000000;
+#endif
+}
+
