@@ -57,6 +57,7 @@ public:
 	FPoint();
 	FPoint(float _x, float _y);
 	explicit FPoint(Point _p);
+	FPoint(int _x, int _y);
 	void align();
 };
 
@@ -91,7 +92,7 @@ public:
 		BEGIN = 1
 	};
 
-	Timer(unsigned _duration = 0);
+	explicit Timer(unsigned _duration = 0);
 	unsigned getCurrent();
 	unsigned getDuration();
 	void setCurrent(unsigned val);
@@ -177,6 +178,8 @@ namespace Utils {
 	void lockFileRead();
 	void lockFileWrite(int increment);
 	void lockFileCheck();
+
+	void setSDL_RGBA(Uint32 *rmask, Uint32 *gmask, Uint32 *bmask, Uint32 *amask);
 }
 
 #endif
