@@ -31,6 +31,7 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 WidgetScrollBox::WidgetScrollBox(int width, int height)
 	: contents(NULL)
 	, update(true)
+	, show_focus_when_scrollbar_disabled(true)
 	, bg(0,0,0,0)
 	, tablist()
 	, cursor(0)
@@ -196,6 +197,15 @@ void WidgetScrollBox::logic(int x, int y) {
 		if (cursor > cursor_target)
 			cursor = cursor_target;
 	}
+
+	// getPrev() and getNext() aren't called when pressing left and right
+	int current_child = tablist.getCurrent();
+	if (current_child != -1) {
+		if (children[current_child]->in_focus) {
+			if (children[current_child]->pos.y < static_cast<int>(cursor) || children[current_child]->pos.y > static_cast<int>(cursor) + pos.h)
+				scrollToSmooth(children[current_child]->pos.y);
+		}
+	}
 }
 
 void WidgetScrollBox::resize(int w, int h) {
@@ -255,6 +265,8 @@ void WidgetScrollBox::refresh() {
 }
 
 void WidgetScrollBox::render() {
+	update = false;
+
 	Rect src,dest;
 	dest = pos;
 	src.x = 0;
@@ -262,7 +274,11 @@ void WidgetScrollBox::render() {
 	src.w = pos.w;
 	src.h = pos.h;
 
+	int content_height = 0;
+
+	// draw content buffer, minus child widgets
 	if (contents) {
+		content_height = contents->getGraphicsHeight();
 		contents->local_frame = local_frame;
 		contents->setOffset(local_offset);
 		contents->setClipFromRect(src);
@@ -270,20 +286,22 @@ void WidgetScrollBox::render() {
 		render_device->render(contents);
 	}
 
+	// draw child widgets
 	for (unsigned i = 0; i < children.size(); i++) {
 		children[i]->local_frame = pos;
 		children[i]->local_offset.y = static_cast<int>(cursor);
 		children[i]->render();
 	}
 
-	if (contents && contents->getGraphicsHeight() > pos.h && scrollbar) {
+	// draw scrollbar
+	if (content_height > pos.h && scrollbar) {
 		scrollbar->local_frame = local_frame;
 		scrollbar->local_offset = local_offset;
 		scrollbar->render();
 	}
-	update = false;
 
-	if (in_focus && children.empty()) {
+	// draw focus rectangle around the scrollbar
+	if (in_focus && children.empty() && (show_focus_when_scrollbar_disabled || content_height > pos.h)) {
 		Point topLeft;
 		Point bottomRight;
 		Rect sb_rect = scrollbar->getBounds();
@@ -304,7 +322,7 @@ void WidgetScrollBox::render() {
 			draw = false;
 		}
 		if (draw) {
-			render_device->drawRectangle(topLeft, bottomRight, eset->widgets.selection_rect_color);
+			render_device->drawRectangleCorners(eset->widgets.selection_rect_corner_size, topLeft, bottomRight, eset->widgets.selection_rect_color);
 		}
 	}
 }
@@ -328,13 +346,14 @@ bool WidgetScrollBox::getNext() {
 		tablist.setCurrent(children[currentChild]);
 	}
 	else {
-		// TODO neaten this up?
-		currentChild = 0;
-		tablist.setCurrent(children[currentChild]);
-		currentChild = tablist.getNextRelativeIndex(TabList::WIDGET_SELECT_DOWN);
-		tablist.setCurrent(children[currentChild]);
-		currentChild = tablist.getNextRelativeIndex(TabList::WIDGET_SELECT_UP);
-		tablist.setCurrent(children[currentChild]);
+		if (!children[0]->enable_tablist_nav) {
+			tablist.getNext(!TabList::GET_INNER, TabList::WIDGET_SELECT_AUTO);
+			currentChild = tablist.getCurrent();
+		}
+		else {
+			currentChild = 0;
+			tablist.setCurrent(children[currentChild]);
+		}
 	}
 
 	if (currentChild != -1) {
@@ -366,12 +385,14 @@ bool WidgetScrollBox::getPrev() {
 		tablist.setCurrent(children[currentChild]);
 	}
 	else {
-		currentChild = 0;
-		tablist.setCurrent(children[currentChild]);
-		currentChild = tablist.getNextRelativeIndex(TabList::WIDGET_SELECT_DOWN);
-		tablist.setCurrent(children[currentChild]);
-		currentChild = tablist.getNextRelativeIndex(TabList::WIDGET_SELECT_UP);
-		tablist.setCurrent(children[currentChild]);
+		if (!children[0]->enable_tablist_nav) {
+			currentChild = tablist.getNextRelativeIndex(TabList::WIDGET_SELECT_DOWN);
+			tablist.setCurrent(children[currentChild]);
+		}
+		else {
+			currentChild = 0;
+			tablist.setCurrent(children[currentChild]);
+		}
 	}
 
 	if (currentChild != -1) {
