@@ -27,14 +27,15 @@ FLARE.  If not, see http://www.gnu.org/licenses/
 #include "EngineSettings.h"
 #include "FontEngine.h"
 #include "IconManager.h"
+#include "InputState.h"
 #include "RenderDevice.h"
 #include "SharedResources.h"
+#include "Utils.h"
 #include "WidgetSlot.h"
 
-WidgetSlot::WidgetSlot(int _icon_id, int _ACTIVATE)
+WidgetSlot::WidgetSlot(int _icon_id)
 	: Widget()
 	, slot_selected(NULL)
-	, slot_checked(NULL)
 	, label_amount_bg(NULL)
 	, label_hotkey_bg(NULL)
 	, icon_id(_icon_id)
@@ -42,14 +43,12 @@ WidgetSlot::WidgetSlot(int _icon_id, int _ACTIVATE)
 	, amount(1)
 	, max_amount(1)
 	, amount_str("")
-	, activate_key(_ACTIVATE)
 	, hotkey(-1)
+	, activated(false)
 	, enabled(true)
-	, checked(false)
-	, pressed(false)
 	, continuous(false)
+	, visible(true)
 {
-	focusable = true;
 	label_amount.setFromLabelInfo(eset->widgets.slot_quantity_label);
 	label_amount.setColor(eset->widgets.slot_quantity_color);
 	label_hotkey.setFromLabelInfo(eset->widgets.slot_hotkey_label);
@@ -72,7 +71,6 @@ WidgetSlot::WidgetSlot(int _icon_id, int _ACTIVATE)
 	src.w = src.h = eset->resolutions.icon_size;
 
 	selected_filename = "images/menus/slot_selected.png";
-	checked_filename = "images/menus/slot_checked.png";
 
 	Image *graphics;
 	graphics = render_device->loadImage(selected_filename, RenderDevice::ERROR_NORMAL);
@@ -81,13 +79,6 @@ WidgetSlot::WidgetSlot(int _icon_id, int _ACTIVATE)
 		slot_selected->setClipFromRect(src);
 		graphics->unref();
 	}
-
-	graphics = render_device->loadImage(checked_filename, RenderDevice::ERROR_NORMAL);
-	if (graphics) {
-		slot_checked = graphics->createSprite();
-		slot_checked->setClipFromRect(src);
-		graphics->unref();
-	}
 }
 
 void WidgetSlot::setPos(int offset_x, int offset_y) {
@@ -108,29 +99,18 @@ void WidgetSlot::setPos(int offset_x, int offset_y) {
 }
 
 void WidgetSlot::activate() {
-	pressed = true;
-}
-
-void WidgetSlot::deactivate() {
-	pressed = false;
-	checked = false;
+	activated = true;
 }
 
 void WidgetSlot::defocus() {
 	in_focus = false;
-	pressed = false;
-	checked = false;
 }
 
 bool WidgetSlot::getNext() {
-	pressed = false;
-	checked = false;
 	return false;
 }
 
 bool WidgetSlot::getPrev() {
-	pressed = false;
-	checked = false;
 	return false;
 }
 
@@ -139,62 +119,46 @@ WidgetSlot::CLICK_TYPE WidgetSlot::checkClick() {
 }
 
 WidgetSlot::CLICK_TYPE WidgetSlot::checkClick(int x, int y) {
-	// disabled slots can't be clicked;
-	if (!enabled) return NO_CLICK;
+	if (!enabled) {
+		return NO_CLICK;
+	}
 
 	Point mouse(x,y);
+	bool mouse_in_rect = Utils::isWithinRect(pos, mouse);
 
-	if (continuous && pressed && checked && (inpt->lock[Input::MAIN2] || inpt->lock[activate_key] || (inpt->touch_locked && Utils::isWithinRect(pos, mouse))))
-		return ACTIVATED;
-
-	// main button already in use, new click not allowed
-	if (inpt->lock[Input::MAIN1]) return NO_CLICK;
-	if (inpt->lock[Input::MAIN2]) return NO_CLICK;
-	if (inpt->lock[activate_key]) return NO_CLICK;
-
-	if (pressed && !inpt->lock[Input::MAIN1] && !inpt->lock[Input::MAIN2]) { // this is a button release
-		pressed = false;
-
-		checked = !checked;
-		if (checked)
-			return CHECKED;
-		else if (continuous)
-			return NO_CLICK;
-		else
-			return ACTIVATED;
+	if (mouse_in_rect && inpt->pressing[Input::MAIN1] && !inpt->lock[Input::MAIN1]) {
+		inpt->lock[Input::MAIN1] = true;
+		return DRAG;
+	}
+	else if (mouse_in_rect && inpt->pressing[Input::MAIN2] && !inpt->lock[Input::MAIN2]) {
+		inpt->lock[Input::MAIN2] = true;
+		return ACTIVATE;
+	}
+	else if (in_focus && inpt->pressing[Input::ACCEPT] && !inpt->lock[Input::ACCEPT]) {
+		// TODO This probably isn't needed, since Input::ACCEPT is handled by TabList (which calls activate())
+		inpt->lock[Input::ACCEPT] = true;
+		return DRAG;
+	}
+	else if (in_focus && inpt->pressing[Input::MENU_ACTIVATE] && !inpt->lock[Input::MENU_ACTIVATE]) {
+		inpt->lock[Input::MENU_ACTIVATE] = true;
+		return ACTIVATE;
+	}
+	else if (activated) {
+		// activate() was called
+		activated = false;
+		return DRAG;
 	}
 
-	// detect new click
-	// use MAIN1 only for selecting
-	if (inpt->pressing[Input::MAIN1]) {
-		if (Utils::isWithinRect(pos, mouse)) {
+	if (continuous) {
+		bool continuous_mouse = mouse_in_rect && (inpt->lock[Input::MAIN2] || inpt->touch_locked);
+		bool continuous_button = in_focus && inpt->lock[Input::MENU_ACTIVATE];
 
-			inpt->lock[Input::MAIN1] = true;
-			pressed = true;
-			checked = false;
-		}
-	}
-	// use MAIN2 only for activating
-	if (inpt->pressing[Input::MAIN2]) {
-		if (Utils::isWithinRect(pos, mouse)) {
-
-			inpt->lock[Input::MAIN2] = true;
-			pressed = true;
-			checked = true;
-		}
-	}
-
-	// handle touch presses for action bar
-	if (continuous && inpt->touch_locked) {
-		if (Utils::isWithinRect(pos, mouse)) {
-			pressed = true;
-			checked = true;
-			return ACTIVATED;
+		if (continuous_mouse || continuous_button) {
+			return ACTIVATE;
 		}
 	}
 
 	return NO_CLICK;
-
 }
 
 int WidgetSlot::getIcon() {
@@ -246,7 +210,7 @@ void WidgetSlot::setHotkey(int key) {
 
 	if (hotkey != -1 && !eset->widgets.slot_hotkey_label.hidden) {
 		label_hotkey.setPos(pos.x + icons->text_offset.x, pos.y + icons->text_offset.y);
-		label_hotkey.setText(inpt->getBindingString(hotkey, InputState::BINDING_DEFAULT, InputState::GET_SHORT_STRING));
+		label_hotkey.setText(inpt->getBindingString(hotkey, InputState::GET_SHORT_STRING));
 		label_hotkey.local_frame = local_frame;
 		label_hotkey.local_offset = local_offset;
 
@@ -274,6 +238,8 @@ void WidgetSlot::setHotkey(int key) {
 }
 
 void WidgetSlot::render() {
+	if (!visible) return;
+
 	Rect src;
 
 	if (icon_id != -1 && icons) {
@@ -307,25 +273,18 @@ void WidgetSlot::render() {
  * We can use this function if slot is grayed out to refresh selection frame
  */
 void WidgetSlot::renderSelection() {
-	if (in_focus) {
-		if (slot_checked && checked) {
-			slot_checked->local_frame = local_frame;
-			slot_checked->setOffset(local_offset);
-			slot_checked->setDestFromRect(pos);
-			render_device->render(slot_checked);
-		}
-		else if (slot_selected) {
-			slot_selected->local_frame = local_frame;
-			slot_selected->setOffset(local_offset);
-			slot_selected->setDestFromRect(pos);
-			render_device->render(slot_selected);
-		}
+	if (!visible) return;
+
+	if (in_focus && slot_selected) {
+		slot_selected->local_frame = local_frame;
+		slot_selected->setOffset(local_offset);
+		slot_selected->setDestFromRect(pos);
+		render_device->render(slot_selected);
 	}
 }
 
 WidgetSlot::~WidgetSlot() {
 	delete slot_selected;
-	delete slot_checked;
 	delete label_amount_bg;
 	delete label_hotkey_bg;
 }
